#!/usr/bin/perl
###########################################################################
# Copyright 2003-2005 UNINETT AS
# http://www.uninett.no
#
# This file is part of Nemo.
#
# Nemo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Nemo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Nemo; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

# Reads the cpumem statistic files and puts it into the NEMO database.
#
# Usage:
# -verbose         turns on additional output
# -dir <directory> gives the full path to a spesific directory to be read
# -gatherall       processes all available files
#
# If no -dir parameter is given, it defaults for the directory for today.
# If neither -allgather or -dir are set, the processing stops when statistics
# already in the database are encountered.
#
# By Gunnar Gauslaa Bergem, June 2003

use strict;

require "nemo_db.pl";
require "timelocal.pl";

my $day = (gmtime())[3];
my $month = (gmtime())[4]+1;
my $year = (gmtime())[5]+1900;
if ($day < 10) { $day = "0$day"; }
if ($month < 10) { $month = "0$month"; }
my $datestring = "$year-$month-$day";

my $file;
my $sth;
my @dirs;
my $basedir ='/local/net-status/data/cpumem/poll';
my $net = 'uninett';

my $inserted_records = 0;
my $skipped_records = 0;
my $updated_records = 0;

# initialize command-line parameters
my $cpudir = "$basedir/$datestring/";
my $gather_all = 0;
my $verbose = 0;

while ($#ARGV >= 0) {
    if ($ARGV[0] =~ /-verbose/) {
	shift(@ARGV);
	$verbose = 1;
    }
    elsif ($ARGV[0] =~ /-dir/) {
        shift(@ARGV);
        $cpudir = shift(@ARGV);
	push(@dirs, $cpudir);
	$gather_all = 1;  # if a dir is supplied, read the whole thing
    }
    elsif ($ARGV[0] =~ /-gatherall/) {
        shift(@ARGV);
	opendir (BASEDIR, $basedir) or die $!;
	for $cpudir (readdir BASEDIR) {
	    push(@dirs, "$basedir/$cpudir");
	}
	closedir (BASEDIR);
        $gather_all = 1;
    }
    else {
        shift(@ARGV);
    }
}

if ($#dirs < 0) {
    # no dirs supplied in the command-line arguments, use default (todays) file
    push(@dirs, $cpudir);
}

my $dbh = DBI->connect($NemoDB::dsn, $NemoDB::user, $NemoDB::password);
if (!$dbh) {die "Error: Can not connect to database..."}

for my $dir (@dirs) {
    print "Opening directory $dir\n" if $verbose;
    opendir (CPU_DIR, $dir) or die $!;
    my @files = readdir(CPU_DIR);
    closedir(CPU_DIR);

  file: foreach $file (@files) {
      # open a cpu statistic file
      open (CPU_FILE, "$dir/$file") or die $!;
      my $line;

      # check if the cpu we're trying to add statistics for exists in the database
      $sth = $dbh->prepare("SELECT eq_id, location_id FROM equipment WHERE eq_id = '$file.uninett.no'");
      $sth->execute();
      my $row = $sth->fetchrow_hashref();
      my $eq_id = $row->{eq_id};
      my $loc_id = $row->{location_id};

      if (defined $eq_id) {
	  my @all_lines = <CPU_FILE>;
	  my $line_num = @all_lines;
	  # start to read from the end of the file, stop when we encounter statistics already gathered
	  for $line (reverse(@all_lines)) {
	      $line_num--;
	    
	      if ($line =~ /(^\d+-\d+-\d+\s+\d+:\d+:\d+)\s+avgBusy5\s+(\d+)/) {
		  my $datestring = $1;   # yyyy-mm-dd hh:mm:ss
		  my $avgbusy5_value = $2+0;   # the value recorded
		  my $timestamp = calculate_timestamp($datestring);
		  my $i_start = $timestamp-30;  # not every piece of equipment at one location is measured at exactly
		  my $i_end = $timestamp+30;    # the same time, so we need some 'slack'

		  my $avgbusy1_value;
		  my $secondline = $all_lines[$line_num-1];
		  if ($secondline =~ /(^\d+-\d+-\d+\s+\d+:\d+:\d+)\s+avgBusy1\s+(\d+)/) {
		      $avgbusy1_value = $2+0;   # the value recorded
		  }
                
		  my $busyper_value;
		  my $thirdline = $all_lines[$line_num-2];
		  if ($thirdline =~ /(^\d+-\d+-\d+\s+\d+:\d+:\d+)\s+busyPer\s+(\d+)/) {
		      $busyper_value = $2+0;   # the value recorded
		  }
                
		  # check if we already have the equipment statistic in the database for this timestamp
		  $sth = $dbh->prepare("SELECT COUNT(*) FROM eq_cpuload_5min WHERE eq_id = '$eq_id' AND ".
				       "timestamp = $timestamp");
		  $sth->execute();
		  my $num_rows = ($sth->fetchrow_array())[0];
		  if ($num_rows == 0) {
		      # no statistic for this timestamp is recorded, so we add it
		      $sth = $dbh->prepare("INSERT INTO eq_cpuload_5min (eq_id, timestamp, five_sec_load, one_min_load, five_min_load) VALUES ".
					   "('$eq_id', $timestamp, $busyper_value, $avgbusy1_value, $avgbusy5_value)");
		      $sth->execute();
		      $inserted_records++;
		  }
		  elsif (!$gather_all) {
		      # if both the timestamp and statistic value already exists, we have reached the point in the file
		      # where we have recorded before, so jump to next file
		      $skipped_records++;
		      next file;
		  }
		
		  if (defined $loc_id) {
		      # check if it is already recorded a statistic in the location
		      $sth = $dbh->prepare("SELECT five_sec_load, one_min_load, five_min_load, timestamp FROM point_cpuload_5min WHERE point_id = $loc_id ".
					   "AND timestamp BETWEEN $i_start AND $i_end");
		      $sth->execute();
		      my $row = $sth->fetchrow_hashref();
		      my $point_busyper_value = $row->{five_sec_load}+0;
		      my $point_avgbusy1_value = $row->{one_min_load}+0;
		      my $point_avgbusy5_value = $row->{five_min_load}+0;
		      my $point_timestamp = $row->{timestamp}+0;
		    
		      if ($point_timestamp == 0) {
			  # no location statistic recorded for the timestamp, add a new row
			  $sth = $dbh->prepare("INSERT INTO point_cpuload_5min (point_id, timestamp, five_sec_load, one_min_load, ".
					       "five_min_load) VALUES ($loc_id, $timestamp, $busyper_value, $avgbusy1_value, $avgbusy5_value)");
			  $sth->execute();
			  $inserted_records++;
		      }
		      else {
			  # a row for the timestamp exists, update the point
			  $point_busyper_value = $busyper_value if ($busyper_value > $point_busyper_value);
			  $point_avgbusy1_value = $avgbusy1_value if ($avgbusy1_value > $point_avgbusy1_value);
			  $point_avgbusy5_value = $avgbusy5_value if ($avgbusy5_value > $point_avgbusy5_value);
			  
			  $sth = $dbh->prepare("UPDATE point_cpuload_5min SET five_sec_load = $point_busyper_value, one_min_load = $point_avgbusy1_value, ".
					       "five_min_load = $point_avgbusy5_value WHERE point_id = $loc_id AND timestamp = $point_timestamp");
			  $sth->execute();
			  $updated_records++;
		      }
		  } #endif defined loc_id
	      } #endif line
	  } #endfor line
      } #endif defined eq_id
  } #endfor file
} #endfor dir

print "Inserted records: $inserted_records   Updated records: $updated_records   Skipped records: $skipped_records\n" if $verbose;

$sth->finish();
$dbh->disconnect();

# -------------------------------------------------------------
#
# Calculates timestamp in seconds
#
# -------------------------------------------------------------
sub calculate_timestamp {

    my $line = shift;

    if(!defined $line) {
	return 0;
    }

    if ($line =~ /^(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)/){
	my $year = $1;
	my $month = $2-1;
	my $day = $3;
	my $hour = $4;
	my $min = $5;
	my $sec = $6;
	return timelocal($sec,$min,$hour,$day,$month,$year);
    }
}
