#!/usr/bin/perl
###########################################################################
# Copyright 2003-2005 UNINETT AS
# http://www.uninett.no
#
# This file is part of Nemo.
#
# Nemo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Nemo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Nemo; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

# Reads all the line load day report files and puts it into the NEMO database.
#
# Usage:
# -verbose         turns on additional output
# -file <filename> gives the full path to a spesific file to be read
# -gatherall       processes all available files
# -dir <report-dir>  point to the zino report-dir
#
# If no -file parameter is given, it defaults to the file for yesterday.
#
# By Gunnar Gauslaa Bergem, June 2003

use strict;

require "nemo_db.pl";
require "timelocal.pl";

my $net = 'uninett';
my $dbh = DBI->connect($NemoDB::dsn, $NemoDB::user, $NemoDB::password);
if (!$dbh) {die "Error: Can not connect to database."}

my $report_dir = "/local/net-status/zino/reports";
my $filename = undef;
my $opt_gatherall = 0;

# parse current date to correspond to a lineload statistic file
my @lt = localtime(time()-(3600*24));
my $day = @lt[3];
my $month = @lt[4]+1;
my $year = @lt[5]+1900;
if ($day < 10) { $day = "0$day"; }
if ($month < 10) { $month = "0$month"; }
my $datestring = "$year-$month/$day";

my $inserted_records = 0;
my $skipped_records = 0;

# initialize command-line parameters
my $gather_all = 0;
my $verbose = 0;

my $month;
my $day;
my $file;
my $sth;
my @files;

while ($#ARGV >= 0) {
    if ($ARGV[0] =~ /-verbose/) {
	shift(@ARGV);
	$verbose = 1;
    }    
    elsif ($ARGV[0] =~ /-dir/) {
        shift(@ARGV);
        $report_dir = shift(@ARGV);
    }    
    elsif ($ARGV[0] =~ /-file/) {
        shift(@ARGV);
        $filename = shift(@ARGV);
	push(@files, $filename);
	$gather_all = 1;  # if a filename is supplied, read the whole thing 
    }
    elsif ($ARGV[0] =~ /-gatherall/) {
        shift(@ARGV);
	$opt_gatherall=1;
        $gather_all = 1;
    } elsif ($ARGV[0] =~ /-file/) {
        shift(@ARGV);
	$report_dir=shift(@ARGV);
    }
    else {
        shift(@ARGV);
    }
}

my $filename = "$report_dir/$datestring/traffic.kbit";

if ( $opt_gatherall){
    opendir (REPORT_DIR, $report_dir) or die $!;
    my @months = readdir (REPORT_DIR);
    for $month (@months) {
	opendir (MONTH_DIR, "$report_dir/$month") or die $!;
	for $day (readdir MONTH_DIR) {
	    if ($day =~ /\d+/ and $month =~ /\d+-\d+/) {
		push(@files, "$report_dir/$month/$day/traffic.kbit");
	    }
	}
    }
    closedir (REPORT_DIR);
    closedir (MONTH_DIR);
}

if ($#files < 0) {
    # no files supplied in command-line arguments, use default (yesterdays) file
    push(@files, $filename);
}

foreach $file (@files) {
    if ($file =~ /\/(\d+)-(\d+)\/(\d+)\/traffic\.kbit/) {
	$year = $1;
	$month = $2;
	$day = $3;
    }
    open (DAY_FILE, "$file") or die "Error opening file $file:\n$!";
    print "Opening file $file\n" if $verbose;
                
    my $line;
    my $timestamp;
    eval {
        $timestamp = timelocal(0, 0, 12, $day, $month-1, $year);
    };
    if ($@) {
        print $@ if $verbose;
        print "Invalid time value, skipping to next one.\n" if $verbose;
        next;
    }
    while (defined ($line = <DAY_FILE>)) {
	if ($line =~ /^(\S+)\s+\d+\s+\d+\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/) {
	    my $linename = $1;
	    my $peak_in = $2;
	    my $peak_out = $3;
	    my $avg_in = $4;
	    my $avg_out = $5;
	    my $capacity = $6;
	    my $percent_peak_in = int ($peak_in * 100 / $capacity);
	    my $percent_peak_out = int ($peak_out * 100 / $capacity);
	    my $percent_avg_in = int ($avg_in * 100 / $capacity);
	    my $percent_avg_out = int ($avg_out * 100 / $capacity);
	    
	    
	    $sth = $dbh->prepare("SELECT line_id FROM line WHERE line_name = '$linename'");
	    $sth->execute();
	    while (my @row = $sth->fetchrow_array()) {
		my $lineid = $row[0];
		my $inner_sth = $dbh->prepare("SELECT COUNT(*) from line_load_day WHERE line_id = '$lineid' AND timestamp = '$timestamp'");
		$inner_sth->execute();                            
		my $num_rows = ($inner_sth->fetchrow_array())[0];
		if ($num_rows == 0) {
		    $inner_sth = $dbh->prepare("INSERT INTO line_load_day VALUES ('$lineid', '$timestamp', '$peak_in', '$peak_out', '$avg_in', ".
					       "'$avg_out', '$percent_peak_in', '$percent_peak_out', '$percent_avg_in', '$percent_avg_out', '$capacity')");
		    $inner_sth->execute();
		    $inserted_records++;
		}
		else {
		    $skipped_records++;
		}
	    }
	}
    }
}

print "Inserted records: $inserted_records   Skipped records: $skipped_records\n" if $verbose;

$sth->finish();
$dbh->disconnect();
