#!/usr/bin/perl
###########################################################################
# Copyright 2003-2005 UNINETT AS
# http://www.uninett.no
#
# This file is part of Nemo.
#
# Nemo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Nemo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Nemo; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

# Reads the 5min mping statistic files and puts it into the NEMO database.
#
# Usage:
# -verbose         turns on additional output
# -file <filename> gives the full path to a spesific file to be read
# -gatherall       processes all available files
#
# If no -file parameter is given, it defaults to the file for the current day.
# If -allgather is not set, the processing stops when statistics already in
# the database are encountered.
#
# By Gunnar Gauslaa Bergem, June 2003

use strict;

require "nemo_db.pl";
require "timelocal.pl";

my $mping_dir = '/local/net-status/mping/data/uninett/data';
my $net = 'uninett';
my $dbh = DBI->connect($NemoDB::dsn, $NemoDB::user, $NemoDB::password);
if (!$dbh) {die "Error: Can not connect to database."}

my @files;
my $file;
my $sth;

# parse current date to correspond to an mping statistic file
my $day = (gmtime())[3];
my $month = (gmtime())[4]+1;
my $year = (gmtime())[5]-100;
if ($day < 10) { $day = "0$day"; }
if ($month < 10) { $month = "0$month"; }
if ($year < 10) { $year = "0$year"; }
my $datestring = "$year$month$day";

my $inserted_records = 0;
my $skipped_records = 0;
my $updated_records = 0;

# initialize command-line parameters
my $filename = "$mping_dir/ping\.$datestring";
my $gather_all = 0;
my $verbose = 0;

while ($#ARGV >= 0) {
    if ($ARGV[0] =~ /-verbose/) {
	shift(@ARGV);
	$verbose = 1;
    }    
    elsif ($ARGV[0] =~ /-file/) {
        shift(@ARGV);
        $filename = shift(@ARGV);
	push(@files, $filename);
	$gather_all = 1;  # if a filename is supplied, read the whole thing 
    }
    elsif ($ARGV[0] =~ /-gatherall/) {
        shift(@ARGV);
	opendir (MPING_DIR, $mping_dir) or die "could not open $mping_dir : $!";
	for $file (readdir MPING_DIR) {
	    push(@files, "$mping_dir/$file");
        }
	closedir (MPING_DIR);
        $gather_all = 1;
    }
    else {
        shift(@ARGV);
    }
}

if ($#files < 0) {
    # no files supplied in command-line arguments, use default (todays) file
    push(@files, $filename);
}

for $file (@files) {
    if ($file =~ /ping.(\d\d)(\d\d)(\d\d)/) {        
	print "Opening file $file\n" if $verbose;
	open (MPING_FILE, "$file") or die "Could not open $file : $!";
	my $line;
	my @all_lines = <MPING_FILE>;
	my $line_num = @all_lines;

        for $line (reverse(@all_lines)) {
	    my $timestamp;
	    my $stat_line;
	    my $packets_sent;

	    $line_num--;
	    
	    if ($line =~ /(\w+\s+\w+\s+\d+\s\d+:\d+:\d+\s\w+\s\d+)/) {
		$timestamp = calculate_timestamp($1);

		# first check if the timestamp is already here, and quit if the gather_all flag isnt set
		if (!$gather_all) {
		    $sth = $dbh->prepare("SELECT COUNT(*) FROM eq_response_5min WHERE timestamp = '$timestamp'");
		    $sth->execute();
		    my $num_rows = ($sth->fetchrow_array())[0];
		    if ($num_rows > 0) {
			$sth->finish();
			$dbh->disconnect();
			print "\nAt location: $line   Timestamp $timestamp exists in the database. Done.\n" if $verbose;
			exit;
		    }
		}
		for $stat_line (@all_lines[$line_num+1..$#all_lines]) {
		    if ($stat_line =~ /(\d+) packets transmitted to each host/) {
			$packets_sent = $1;
		    }
		    if ($stat_line =~ /^([\w\.\-]+)\s+(\d+)\s+(\d+)[\.\d]*\/(\d+)[\.\d]*\/(\d+)[\.\d]*\/(\d+)[\.\d]*\/(\d+)[\.\d]*/) {
			my $address = $1;
			my $packetloss = ($packets_sent-$2)*100/$packets_sent;
			my $min = $3;
			my $avg = $4;
			my $max = $5;
			my $cube_sum = $6;
			my $median = $7;
			
			# look up the DNS name of the equipment based on IP address (also checks if DNS exists if we got that instead of IP)
			$sth = $dbh->prepare("SELECT eq_id FROM equipment_address WHERE ip_address = '$address' ".
					     "OR eq_id = '$address'");
			$sth->execute();
			my $eq_id = ($sth->fetchrow_array())[0];
			if (!defined $eq_id) {
			    # no match in IP or DNS tables, skip to next line
			    next;
			}			
			$sth = $dbh->prepare("SELECT point_id, packetloss, min, average, max, median ".
					     "FROM point_response_5min, equipment ".
					     "WHERE point_response_5min.point_id = equipment.location_id AND ".
					     "equipment.eq_id = '$eq_id' AND timestamp = '$timestamp'");
			$sth->execute();
			my $row = $sth->fetchrow_hashref();
			my $point_packetloss = $row->{packetloss};
			my $point_min = $row->{min};
			my $point_max = $row->{max};
			my $point_avg = $row->{average};
			my $point_median = $row->{median};
			my $point_id = $row->{point_id};
			
			# check if we have a point that the equipment belongs to, and update or insert it
			if (defined $point_id) {
			    $point_packetloss = $packetloss if ($packetloss > $point_packetloss);
			    $point_min = $min if ($min > $point_min);
			    $point_max = $max if ($max > $point_max);
			    $point_avg = $avg if ($avg > $point_avg);
			    $point_median = $median if ($median > $point_median);
			    
			    $sth = $dbh->prepare("UPDATE point_response_5min SET packetloss = '$point_packetloss', min = '$point_min', ".
						 "max = '$point_max', average = '$point_avg', median = '$point_median' WHERE ".
						 "point_id = '$point_id' AND timestamp = '$timestamp'");
			    $sth->execute();
			    $updated_records++;
			}
			else {
			    $sth = $dbh->prepare("SELECT location_id FROM equipment WHERE eq_id = '$eq_id'");
			    $sth->execute();
			    $point_id = ($sth->fetchrow_array())[0];
			    if (defined $point_id) {
				$sth = $dbh->prepare("INSERT INTO point_response_5min VALUES ('$point_id', '$timestamp', '$median', ".
						     "'$min', '$max', '$avg', '$packetloss')");
				$sth->execute();
				$inserted_records++;
			    }
			}
			
			# check if equipment statistic is recorded in the database already
			$sth = $dbh->prepare("SELECT eq_id, median, min, max, average, packetloss FROM eq_response_5min WHERE ".
                                             "eq_id = '$eq_id' AND timestamp = '$timestamp'");
			$sth->execute();
                        my $row = $sth->fetchrow_hashref();
                        my $db_packetloss = $row->{packetloss};
                        my $db_min = $row->{min};
                        my $db_max = $row->{max};
                        my $db_avg = $row->{average};
                        my $db_median = $row->{median};
			my $db_eq_id = $row->{eq_id};
			
			if (! $db_eq_id) {
			    # insert equipment statistic into the database
			    $sth = $dbh->prepare("INSERT INTO eq_response_5min VALUES ('$eq_id', '$timestamp', '$median', '$min', ".
						 "'$max', '$avg', '$packetloss')");
			    $sth->execute();
			    $inserted_records++;
			}
                        else {
                            if (($packetloss > $db_packetloss) || ($min > $db_min) || ($max > $db_max) ||
                                ($avg > $db_avg) || ($median > $db_median)) {
                                $sth = $dbh->prepare("UPDATE eq_response_5min SET median = '$median', min = '$min', max = '$max', ".
                                                     "average = '$avg', packetloss = '$packetloss' WHERE eq_id = '$eq_id' AND ".
                                                     "timestamp = '$timestamp'");
                                $sth->execute();
                                $updated_records++;
                            }
                        }
		    } # endif statline
		    last if ($stat_line =~ /(\w+\s+\w+\s+\d+\s\d+:\d+:\d+\s\w+\s\d+)/)
		} # endfor statline
	    } #endif line
	} #endfor line
    } #endif file
} #endfor file

print "Inserted records: $inserted_records   Updated records: $updated_records   Skipped records: $skipped_records\n" if $verbose;

$sth->finish();
$dbh->disconnect();



# -------------------------------------------------------------
#
# Calculates timestamp in seconds
#
# -------------------------------------------------------------
sub calculate_timestamp {
    my $datestring = shift;

    if (!defined $datestring) {
	return 0;
    }

    my %month_num = (
		     Jan => 0,
		     Feb => 1,
		     Mar => 2,
		     Apr => 3,
		     May => 4,
		     Jun => 5,
		     Jul => 6,
		     Aug => 7,
		     Sep => 8,
		     Oct => 9,
		     Nov => 10,
		     Dec => 11
		     );
    if ($datestring =~ /\w+\s+(\w+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s\w+\s(\d+)/) {
	my $month = $1;
	my $day = $2;
	my $hour = $3;
	my $min = $4;
	my $sec = $5;
	my $year = $6;

	print ("Reading entry $2 $month $year $hour:$min:$sec\n") if $verbose;
	return timelocal($sec,$min,$hour,$day,$month_num{$month},$year);
    }
}
