#!/usr/bin/perl
###########################################################################
# Copyright 2003-2005 UNINETT AS
# http://www.uninett.no
#
# This file is part of Nemo.
#
# Nemo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Nemo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Nemo; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

# Reads the day mping statistic files and puts it into the NEMO database.
#
# Usage:
# -verbose         turns on additional output
# -file <filename> gives the full path to a spesific file to be read
# -gatherall       processes all available files
#
# If no -file parameter is given, it defaults to the file for yesterday.
# If neither -allgather or -file are set, the processing stops when statistics
# already in the database are encountered.
#
# By Gunnar Gauslaa Bergem, June 2003

use strict;

require "nemo_db.pl";
require "timelocal.pl";

my $mping_dir = '/local/net-status/mping/data/uninett/report';
my $net = 'uninett';
my $dbh = DBI->connect($NemoDB::dsn, $NemoDB::user, $NemoDB::password);
if (!$dbh) {die "Error: Can not connect to database."}

my @files;
my $file;
my $sth;

my $inserted_records = 0;
my $skipped_records = 0;
my $updated_records = 0;

# parse current date to correspond to an mping statistic file
my @lt = localtime(time()-(3600*24));
my $day = @lt[3];
my $month = @lt[4]+1;
my $year = @lt[5]-100;
if ($day < 10) { $day = "0$day"; }
if ($month < 10) { $month = "0$month"; }
if ($year < 10) { $year = "0$year"; }
my $datestring = "$year$month$day";

# initialize command-line parameters
my $filename = "$mping_dir/report\.$datestring";
my $gather_all = 0;
my $verbose = 0;

while ($#ARGV >= 0) {
    if ($ARGV[0] =~ /-verbose/) {
	shift(@ARGV);
	$verbose = 1;
    }    
    elsif ($ARGV[0] =~ /-file/) {
        shift(@ARGV);
        $filename = shift(@ARGV);
	push(@files, $filename);
	$gather_all = 1;  # if a filename is supplied, read the whole thing 
    }
    elsif ($ARGV[0] =~ /-gatherall/) {
        shift(@ARGV);
	opendir (MPING_DIR, $mping_dir) or die $!;
	for $file (readdir MPING_DIR) {
	    push(@files, "$mping_dir/$file");
        }
	closedir (MPING_DIR);
        $gather_all = 1;
    }
    else {
        shift(@ARGV);
    }
}

if ($#files < 0) {
    # no files supplied in command-line arguments, use default (yesterdays) file
    push(@files, $filename);
}

for $file (@files) {
    if ($file =~ /report.(\d\d)(\d\d)(\d\d)/) {
	print "Opening file $file\n" if $verbose;
        open (MPING_FILE, "$file") or die $!;
        my $line;
        my $year = $1;
        my $month = $2;
        my $day = $3;
        my $timestamp = timelocal(0, 0, 12, $day, $month-1, $year);
        
        while (defined ($line = <MPING_FILE>)) {
            if ($line =~ /(\S+\.\S+\.no).*;(\d+)[\.\d]*;(\d+)[\.\d]*;(\d+)[\.\d+]*;(\d+\.\d+);(\d+\.\d+);(\d+\.\d+);(\d+\.\d+);(\d+\.\d+);(\d+\.\d+)/) {
                my $hostname = $1;
                my $median = $2+0;
                my $max = $3+0;
                my $packetloss = $9+0;
                
		$sth = $dbh->prepare("SELECT eq_id FROM equipment WHERE eq_id = '$hostname'");
		$sth->execute();
		my $eq_id = ($sth->fetchrow_array())[0];
		if (!defined $eq_id) {
		    # no match in equipment table, skip to next line
		    next;
		}

		$sth = $dbh->prepare("SELECT point_id, packetloss, max, median FROM point_response_day WHERE point_id = ".
				     "(SELECT location_id FROM equipment WHERE eq_id = '$eq_id') AND timestamp = 
'$timestamp'");
		$sth->execute();
		my $row = $sth->fetchrow_hashref();
		my $point_packetloss = $row->{packetloss};
		my $point_max = $row->{max};
		my $point_median = $row->{median};
		my $point_id = $row->{point_id};
		
                # check if we have a point that the equipment belongs to, and update or insert it
		if (defined $point_id) {
		    $point_packetloss = $packetloss if ($packetloss > $point_packetloss);
		    $point_max = $max if ($max > $point_max);
		    $point_median = $median if ($median > $point_median);

		    $sth = $dbh->prepare("UPDATE point_response_day SET packetloss = '$point_packetloss', ".
					 "max = '$point_max', median = '$point_median' WHERE ".
					 "point_id = '$point_id' AND timestamp = '$timestamp'");
		    $sth->execute();
		    $updated_records++;
		}
		else {
		    $sth = $dbh->prepare("SELECT location_id FROM equipment WHERE eq_id = '$eq_id'");
		    $sth->execute();
		    $point_id = ($sth->fetchrow_array())[0];
		    if (defined $point_id) {
			$sth = $dbh->prepare("INSERT INTO point_response_day VALUES ('$point_id', '$timestamp', '$median', ".
					     "'$max', '$packetloss')");
			$sth->execute();
			$inserted_records++;
		    }
		}

		# check if equipment statistic is recorded in the database already
		$sth = $dbh->prepare("SELECT eq_id FROM eq_response_day WHERE eq_id = '$eq_id' AND timestamp = '$timestamp'");
		$sth->execute();
		my $db_eq_id = ($sth->fetchrow_array())[0];
		
		if (! $db_eq_id) {
		    # insert statistics into the equipment table
		    $sth = $dbh->prepare("INSERT INTO eq_response_day VALUES ('$eq_id', '$timestamp', '$median', ".
					 "'$max', '$packetloss')");
		    $sth->execute();
		    $inserted_records++;
		}
		else {
		    $skipped_records++;
		}
            }
        }
    }
}

print "Inserted records: $inserted_records   Updated records: $updated_records   Skipped records: $skipped_records\n" if $verbose;

$sth->finish();
$dbh->disconnect();
