#!/usr/bin/perl
###########################################################################
# Copyright 2003-2005 UNINETT AS
# http://www.uninett.no
#
# This file is part of Nemo.
#
# Nemo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Nemo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Nemo; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

# Reads the mping 5min table and accumulates the data into
# hours and puts it into the NEMO database.
#
# Usage:
# -verbose          turns on additional output
# -day <dd.mm.yyyy> gives date to accumulate statistics for
# -gatherall        processes all available files
#
# If no -day parameter is given, it defaults to yesterday.
#
# By Gunnar Gauslaa Bergem, June 2003

use strict;

require "nemo_db.pl";
require "timelocal.pl";

my $net = 'uninett';
my $dbh = DBI->connect($NemoDB::dsn, $NemoDB::user, $NemoDB::password);
if (!$dbh) {die "Error: Can not connect to database."}

my $min_timestamp;
my $max_timestamp;
my $sth;

my $inserted_records = 0;
my $skipped_records = 0;

# initialize command-line parameters
my $day;
my $gather_all = 0;
my $verbose = 0;

while ($#ARGV >= 0) {
    if ($ARGV[0] =~ /-verbose/) {
	shift(@ARGV);
	$verbose = 1;
    }
    elsif ($ARGV[0] =~ /-day/) {
        shift(@ARGV);
        $day = shift(@ARGV);
    }
    elsif ($ARGV[0] =~ /-gatherall/) {
        $gather_all = 1;
	shift(@ARGV);
    }
    else {
        shift(@ARGV);
    }
}

# no day given in the command-line parameters, default to yesterday
if (!defined $day and !$gather_all) {
    my @t = localtime(time());
    $t[4] += 1;
    $t[5] += 1900;
    $day = "$t[3].$t[4].$t[5]";
}

# calculate start and end unix timestamps for the given day
if ($day =~ /(\d+)\.(\d+)\.(\d+)/) {
    $min_timestamp = timelocal(0, 0, 0, $1, $2-1, $3);
    $max_timestamp = $min_timestamp + (3600 * 24);
}

elsif ($gather_all) {
    # find the minimum timestamp from the database, rounded down to the closest whole hour
    $sth = $dbh->prepare("SELECT timestamp FROM eq_cpuload_5min ORDER BY timestamp ASC LIMIT 1");
    $sth->execute();
    $min_timestamp = ($sth->fetchrow_array())[0];
    my @t = localtime($min_timestamp);
    $min_timestamp = timelocal(0, 0, $t[2], $t[3], $t[4], $t[5]+1900);

    # find the maximum timestamp from the database, rounded down to the closest whole hour
    $sth = $dbh->prepare("SELECT timestamp FROM eq_cpuload_5min ORDER BY timestamp DESC LIMIT 1");
    $sth->execute();
    $max_timestamp = ($sth->fetchrow_array())[0];
    my @t = localtime($max_timestamp);
    $max_timestamp = timelocal(0, 0, $t[2], $t[3], $t[4], $t[5]+1900);
}

print "min_timestamp: $min_timestamp max_timestamp: $max_timestamp\n" if $verbose;

# calculate the number of hours we'll gather statistics for
my $steps = ($max_timestamp - $min_timestamp) / 3600;

for (my $hour = 0; $hour < $steps; $hour++) {
    my $start_timestamp = $min_timestamp + $hour * 3600;
    my $stop_timestamp = $min_timestamp + ($hour+1) * 3600;

    # aggregate values in the database for equipment statistics from 5min -> hour intervals
    $sth = $dbh->prepare("SELECT eq_id, AVG(median), MAX(max), AVG(packetloss) ".
			 "FROM eq_response_5min ".
			 "WHERE timestamp BETWEEN $start_timestamp AND $stop_timestamp".
			 "GROUP BY eq_id");
    $sth->execute();    
    while (my @row = $sth->fetchrow_array()) {
	my $eq_id = $row[0];
	my $median = $row[1];
	my $max = $row[2];
	my $packetloss = $row[3];
	
	# check for existence of the timestamp and equipment
	my $count_sth = $dbh->prepare("SELECT COUNT(*) FROM eq_response_hour ".
				      "WHERE timestamp = $start_timestamp AND eq_id = '$eq_id'");
	$count_sth->execute();
	my $row_count = ($count_sth->fetchrow_array())[0];
	if ($row_count > 0) {
	    $skipped_records++;
	    next;
	}
	else {
	    my $insert_sth = $dbh->prepare("INSERT INTO eq_response_hour VALUES ".
					   "('$eq_id', $start_timestamp, $median, $max, $packetloss)");
	    $insert_sth->execute();
	    $inserted_records++;
	}
    }

    # aggregate values in the database for point statistics from 5min -> hour intervals
    $sth = $dbh->prepare("SELECT point_id, AVG(median), MAX(max), AVG(packetloss) ".
			 "FROM point_response_5min ".
			 "WHERE timestamp BETWEEN $start_timestamp AND $stop_timestamp".
			 "GROUP BY point_id");
    $sth->execute();
    while (my @row = $sth->fetchrow_array()) {
	my $point_id = $row[0];
	my $median = $row[1];
	my $max = $row[2];
	my $packetloss = $row[3];
	
	# check for existence of the timestamp and equipment
	my $count_sth = $dbh->prepare("SELECT COUNT(*) FROM point_response_hour ".
				      "WHERE timestamp = $start_timestamp AND point_id = $point_id");
	$count_sth->execute();
	my $row_count = ($count_sth->fetchrow_array())[0];
	if ($row_count > 0) {
	    $skipped_records++;
	    next;
	}
	else {
	    my $insert_sth = $dbh->prepare("INSERT INTO point_response_hour VALUES ".
					   "($point_id, $start_timestamp, $median, $max, $packetloss)");
	    $insert_sth->execute();
	    $inserted_records++;
	}
    }
}

print "Inserted records: $inserted_records   Skipped records: $skipped_records\n" if $verbose;	

$sth->finish();
$dbh->disconnect();
