/*
 * LineElement.java
 *
 * Created on July 4, 2003, 9:22 AM
 */

import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.font.*;

/**
 * This class contains the coordinates, current statistic values and various other
 * information about a line. It contains methods for clipping the line if it extends
 * the border of the map and methods for drawing it on the map.
 *
 * @author Gunnar Gauslaa Bergem
 */
public class LineElement implements Selectable, Comparable {
    private MapElement parentMap;     // map that contains this line
    private PointElement origin;      // origin point of the line
    private PointElement destination; // destination point of the line
    private String lineName;          // name of the line
    private String lineType;          // type of the line
    private int originSize;           // thickness of the origin half of the line
    private int destinationSize;      // thickness of the destination half of the line
    private Color destinationColour;  // colour of the destination half of the line
    private Color originColour;       // colour of the origin half of the line
    private Polygon polygonOutline;   // a polygon outline of the line, used for hit testing
    private Date fromDate;            // when the line started to exist
    private Date toDate;              // when the line ceased to exist
    private int bitmapX1;             // origin x coordinate
    private int bitmapY1;             // origin y coordinate
    private int bitmapX2;             // destination x coordinate
    private int bitmapY2;             // destination y coordinate
    private Vector dateStrings;       // dates for the line statistics
    private Vector measureStrings;    // names for the line statistics
    private StatisticDialog dialog;   // dialog that shows the line statistics and urls
    private static Color defaultColour = new Color(150, 150, 150);  // grey default colour
    private static int defaultSize = 4;                             // default size
    private boolean enabled;                                        // true if the line should be drawn, false otherwise
    private PopupMenu popup;
    private ArrayList URLnames;
    private ArrayList URLvalues;
    
    /** Creates a new instance of LineElement */
    public LineElement(MapElement parent, PointElement origin, PointElement destination, String name, String type, Date from, Date to) {
        parentMap = parent;
        this.origin = origin;
        this.destination = destination;
        lineName = name;
        lineType = type;
        fromDate = from;
        toDate = to;
        originColour = defaultColour;
        destinationColour = defaultColour;
        originSize = defaultSize;
        destinationSize = defaultSize;
        bitmapX1 = origin.getBitmapX();
        bitmapY1 = origin.getBitmapY();
        bitmapX2 = destination.getBitmapX();
        bitmapY2 = destination.getBitmapY();
        dateStrings = new Vector();
        measureStrings = new Vector();
        polygonOutline = new Polygon();
        checkCoordBounds();
        createPolygonOutline();
        URLnames=new ArrayList();
        URLvalues=new ArrayList();
    }
    
    /**
     * Check if the line crosses any border of the map, and update the origin or destination
     * pointelements to lie on the map border instead.
     */
    private void checkCoordBounds() {
        double x1, y1, x2, y2, x3, y3, x4, y4;
        x1 = y1 = x2 = y2 = x3 = y3 = x4 = y4 = 0;
        
        Rectangle rectangle = parentMap.getIntersectionRectangle();
        Rectangle contour = parentMap.getContourRectangle();
        // bottom side of rectangle
        if (Line2D.linesIntersect(bitmapX1, bitmapY1, bitmapX2, bitmapY2, rectangle.getMinX(), rectangle.getMaxY(), rectangle.getMaxX(), rectangle.getMaxY())) {
            x3 = rectangle.getMinX();
            y3 = rectangle.getMaxY();
            x4 = rectangle.getMaxX();
            y4 = rectangle.getMaxY();
        }
        // top side of rectangle
        else if(Line2D.linesIntersect(bitmapX1, bitmapY1, bitmapX2, bitmapY2, rectangle.getMinX(), rectangle.getMinY(), rectangle.getMaxX(), rectangle.getMinY())) {
            x3 = rectangle.getMinX();
            y3 = rectangle.getMinY();
            x4 = rectangle.getMaxX();
            y4 = rectangle.getMinY();
        }
        // left side of rectangle
        else if(Line2D.linesIntersect(bitmapX1, bitmapY1, bitmapX2, bitmapY2, rectangle.getMinX(), rectangle.getMinY(), rectangle.getMinX(), rectangle.getMaxY())) {
            x3 = rectangle.getMinX();
            y3 = rectangle.getMinY();
            x4 = rectangle.getMinX();
            y4 = rectangle.getMaxY();
        }
        // right side of rectangle
        else if(Line2D.linesIntersect(bitmapX1, bitmapY1, bitmapX2, bitmapY2, rectangle.getMaxX(), rectangle.getMinY(), rectangle.getMaxX(), rectangle.getMaxY())) {
            x3 = rectangle.getMaxX();
            y3 = rectangle.getMinY();
            x4 = rectangle.getMaxX();
            y4 = rectangle.getMaxY();
        }
        
        if (contour.outcode(bitmapX1, bitmapY1) != 0) {
            // the origin is outside of the map
            x1 = bitmapX1;
            y1 = bitmapY1;
            x2 = bitmapX2;
            y2 = bitmapY2;
            double ua = ( (x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3) ) / ( (y4 - y3)*(x2 - x1) - (x4 - x3) * (y2 - y1) + 1);
            double ub = ( (x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3) ) / ( (y4 - y3)*(x2 - x1) - (x4 - x3) * (y2 - y1) + 1);
            int newx = (int)(x1 + ua*(x2 - x1));
            int newy = (int)(y1 + ua*(y2 - y1));
            NemoUtils.debug("Moving "+lineName+" ("+origin+") from ("+bitmapX1+", "+bitmapY1+") to ("+newx+", "+newy+")");
            bitmapX1 = newx;
            bitmapY1 = newy;
            origin.updateBitmapX(bitmapX1);
            origin.updateBitmapY(bitmapY1);
        }
        
        if (contour.outcode(bitmapX2, bitmapY2) != 0) {
            // the destination is outside of the map
            x1 = bitmapX2;
            y1 = bitmapY2;
            x2 = bitmapX1;
            y2 = bitmapY1;
            double ua = ( (x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3) ) / ( (y4 - y3)*(x2 - x1) - (x4 - x3) * (y2 - y1) + 1);
            double ub = ( (x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3) ) / ( (y4 - y3)*(x2 - x1) - (x4 - x3) * (y2 - y1) + 1);
            int newx = (int)(x1 + ua*(x2 - x1));
            int newy = (int)(y1 + ua*(y2 - y1));
            NemoUtils.debug("Moving "+lineName+" ("+destination+") from ("+bitmapX2+", "+bitmapY2+") to ("+newx+", "+newy+")");
            bitmapX2 = newx;
            bitmapY2 = newy;
            destination.updateBitmapX(bitmapX2);
            destination.updateBitmapY(bitmapY2);
        }
    }
    
    public String getType() {
        return lineType;
    }
    
    public PointElement getOrigin() {
        return origin;
    }
    
    public PointElement getDestination() {
        return destination;
    }
    
    public void setOriginSize(int size) {
        originSize = size;
    }
    
    public void setDestinationSize(int size) {
        destinationSize = size;
    }
    
    public void setOriginColour(Color colour) {
        originColour = colour;
    }
    
    public void setDestinationColour(Color colour) {
        destinationColour = colour;
    }
    
    public int getX1() {
        return bitmapX1;
    }
    
    public int getY1() {
        return bitmapY1;
    }
    
    public int getX2() {
        return bitmapX2;
    }
    
    public int getY2() {
        return bitmapY2;
    }
    
    public void setX1(int x1) {
        bitmapX1 = x1;
    }
    
    public void setX2(int x2) {
        bitmapX2 = x2;
    }
    
    public void setY1(int y1) {
        bitmapY1 = y1;
    }
    
    public void setY2(int y2) {
        bitmapY2 = y2;
    }
    
    public Polygon getPolygonOutline() {
        return polygonOutline;
    }
    
    /**
     * Creates strings that displays the name of the statistic and the time at which
     * it was measured and adds them the list of strings.
     */
    public void addTimestamp(Date statisticDate, String statName) {
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss  dd. MMM yyyy");
        String dateString = formatter.format(statisticDate);
        String measureString = "Measure point for "+statName+":";
        dateStrings.addElement(dateString);
        measureStrings.addElement(measureString);
    }
    
    /**
     * Returns the length of the line.
     */
    public int getSize() {
        return (int)Math.sqrt( Math.pow(Math.abs(bitmapX1 - bitmapX2), 2) +
                Math.pow(Math.abs(bitmapY1 - bitmapY2), 2) );
    }
    
    /**
     * Sets all the statistics to the default values.
     */
    public void resetStatistics() {
        originSize = defaultSize;
        destinationSize = defaultSize;
        originColour = defaultColour;
        destinationColour = defaultColour;
        dateStrings.removeAllElements();
        measureStrings.removeAllElements();
    }
    
    public void resetBitmapCoordinates() {
        bitmapX1 = origin.getBitmapX();
        bitmapY1 = origin.getBitmapY();
        bitmapX2 = destination.getBitmapX();
        bitmapY2 = destination.getBitmapY();
        //FIXME should save values so that the lines arent displaced if the overlapping line is
        //removed because the date changes
    }
    
    /**
     * Checks if the polygon outline of this line contains the given point. Used
     * for hit testing.
     */
    public boolean contains(Point point) {
        if (!enabled) {
            return false;
        }
        return polygonOutline.contains(point);
    }
    
    /**
     * Enables this line if it exists at the given date, otherwise it is disabled.
     */
    public void checkEnabled(Date date) {
        if (fromDate.after(date) || toDate.before(date)) {
            enabled = false;
            NemoUtils.debug(lineName+" disabled, does not exist at the given date.");
        }
        else {
            enabled = true;
        }
    }
    
    public boolean isEnabled() {
        return enabled;
    }
    
    /**
     * Creates a polygon outline for this line to be used for hit testing.
     */
    public void createPolygonOutline() {
        int length = (int)Math.sqrt( Math.pow(Math.abs(bitmapX1 - bitmapX2), 2) +
                Math.pow(Math.abs(bitmapY1 - bitmapY2), 2) ) / 2;
        // Just to avoid divison by zero
        if (length == 0) length = 1;
        
        int dx = bitmapX2 - bitmapX1;
        int dy = bitmapY2 - bitmapY1;
        int dt = (originSize/2 + 4) / 2;
        int ddx = (int)(dy * dt / length);
        int ddy = (int)(dx * dt / length);
        
        int[] x_points = new int[4];
        int[] y_points = new int[4];
        
        x_points[0] = bitmapX1 + ddx;
        x_points[1] = bitmapX2 + ddx;
        x_points[2] = bitmapX2 - ddx;
        x_points[3] = bitmapX1 - ddx;
        
        y_points[0] = bitmapY1 - ddy;
        y_points[1] = bitmapY2 - ddy;
        y_points[2] = bitmapY2 + ddy;
        y_points[3] = bitmapY1 + ddy;
        
        polygonOutline = new Polygon(x_points, y_points, 4);
    }
    
    /**
     * Draw the selected version of the line.
     */
    public void drawSelected(Graphics _g) {
        Graphics2D g = (Graphics2D)_g;
        double angle = Math.atan2( (bitmapY2-bitmapY1), (bitmapX2-bitmapX1) ) ;
        int length = (int)Math.sqrt( Math.pow((bitmapX1 - bitmapX2), 2) +
                Math.pow((bitmapY1 - bitmapY2), 2) ) / 2 + 1;
        int headwidth = originSize / 2;
        int headlength = originSize * 2;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        GeneralPath arrow = new GeneralPath();
        
        arrow.moveTo(0, -originSize/2);
        arrow.lineTo(length - headlength, -originSize/2);
        arrow.lineTo(length - headlength, -originSize/2 - headwidth);
        arrow.lineTo(length, 0);
        arrow.lineTo(length - headlength, originSize/2 + headwidth);
        arrow.lineTo(length - headlength, originSize/2);
        arrow.lineTo(0, originSize/2);
        arrow.lineTo(0, -originSize/2);
        
        AffineTransform graphicsDefaultTransform = g.getTransform();
        AffineTransform transform = new AffineTransform();
        GradientPaint selectedPaint = new GradientPaint(0, 0, new Color(176, 176, 176), length, 0, new Color(0, 0, 0));
        transform.translate(bitmapX1, bitmapY1);
        g.transform(transform);
        g.rotate(angle);
        g.setPaint(selectedPaint);
        g.fill(arrow);
        g.setTransform(graphicsDefaultTransform);
        
        headwidth = destinationSize / 2;
        headlength = destinationSize * 2;
        arrow.reset();
        arrow.moveTo(0, -destinationSize/2);
        arrow.lineTo(length - headlength, -destinationSize/2);
        arrow.lineTo(length - headlength, -destinationSize/2 - headwidth);
        arrow.lineTo(length, 0);
        arrow.lineTo(length - headlength, destinationSize/2 + headwidth);
        arrow.lineTo(length - headlength, destinationSize/2);
        arrow.lineTo(0, destinationSize/2);
        arrow.lineTo(0, -destinationSize/2);
        
        transform.translate(-bitmapX1, -bitmapY1);
        transform.translate(bitmapX2, bitmapY2);
        g.transform(transform);
        g.rotate(angle+Math.PI);
        g.setPaint(selectedPaint);
        g.fill(arrow);
        g.setTransform(graphicsDefaultTransform);
        
        FontRenderContext frc = g.getFontRenderContext();
        Font font = new Font("Times", Font.ITALIC, 12);
        TextLayout tl = new TextLayout(lineName, font, frc);
        g.setColor(new Color(51, 204, 255));
        int xpos = (bitmapX2-bitmapX1)/2 + bitmapX1;
        if ((xpos-(int)tl.getVisibleAdvance()/2) < parentMap.getContourRectangle().getMinX()) {
            xpos = (int)(tl.getVisibleAdvance()/2+parentMap.getContourRectangle().getMinX());
        }
        int ypos = (bitmapY2-bitmapY1)/2 + bitmapY1;
        g.fillRect(xpos-(int)tl.getVisibleAdvance()/2, (int)(ypos-tl.getAscent()-tl.getDescent()), (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
        g.setColor(Color.black);
        g.drawRect(xpos-(int)tl.getVisibleAdvance()/2, (int)(ypos-tl.getAscent()-tl.getDescent()), (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
        tl.draw(g, (float)xpos-tl.getVisibleAdvance()/2+1, (float)ypos-tl.getDescent());
    }
    
    /**
     * Draw the unselected version of the line.
     */
    public void drawMe(Graphics _g) {
        Graphics2D g = (Graphics2D)_g;
        
        if (!enabled) return;
        double angle = Math.atan2( (bitmapY2-bitmapY1), (bitmapX2-bitmapX1) ) ;
        int length = (int)Math.sqrt( Math.pow((bitmapX1 - bitmapX2), 2) +
                Math.pow((bitmapY1 - bitmapY2), 2) ) / 2 + 1;
        int headwidth = originSize / 2;
        int headlength = originSize * 2;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        GeneralPath arrow = new GeneralPath();
        
        arrow.moveTo(0, -originSize/2);
        arrow.lineTo(length - headlength, -originSize/2);
        arrow.lineTo(length - headlength, -originSize/2 - headwidth);
        arrow.lineTo(length, 0);
        arrow.lineTo(length - headlength, originSize/2 + headwidth);
        arrow.lineTo(length - headlength, originSize/2);
        arrow.lineTo(0, originSize/2);
        arrow.lineTo(0, -originSize/2);
        
        AffineTransform graphicsDefaultTransform = g.getTransform();
        AffineTransform transform = new AffineTransform();
        transform.translate(bitmapX1, bitmapY1);
        g.transform(transform);
        g.rotate(angle);
        // originColour is actually the "Traffic Out" statistic
        g.setColor(originColour);
        g.fill(arrow);
        g.setColor(originColour.darker());
        g.draw(arrow);
        g.setTransform(graphicsDefaultTransform);
        
        headwidth = destinationSize / 2;
        headlength = destinationSize * 2;
        arrow.reset();
        arrow.moveTo(0, -destinationSize/2);
        arrow.lineTo(length - headlength, -destinationSize/2);
        arrow.lineTo(length - headlength, -destinationSize/2 - headwidth);
        arrow.lineTo(length, 0);
        arrow.lineTo(length - headlength, destinationSize/2 + headwidth);
        arrow.lineTo(length - headlength, destinationSize/2);
        arrow.lineTo(0, destinationSize/2);
        arrow.lineTo(0, -destinationSize/2);
        
        transform.translate(-bitmapX1, -bitmapY1);
        transform.translate(bitmapX2, bitmapY2);
        g.transform(transform);
        g.rotate(angle+Math.PI);
        // destinationColour is actually the "Traffic In" statistic
        g.setColor(destinationColour);
        g.fill(arrow);
        g.setColor(destinationColour.darker());
        g.draw(arrow);
        g.setTransform(graphicsDefaultTransform);
        
        createPolygonOutline();
    }
    
    /**
     * Opens a statistic dialog showing stats and urls when the mouse is clicked on the line.
     */
    public void showStatistics(MapFrame mapFrame, MouseEvent e) {
        //popupMenu.show(mapFrame, e.getX(), e.getY());
        dialog = new StatisticDialog(mapFrame, false, "Statistics for "+lineName, measureStrings, dateStrings);
        Point p = mapFrame.getLocationOnScreen();
        dialog.setLocation((int)p.getX()+50, (int)p.getY()+50);
        dialog.show();
        addURLlist();
        mapFrame.retrieveLineValueStatistics(this);
    }
    
    public void performMouseClickAction(MapFrame mapFrame,MouseEvent e){
    	URLnames=new ArrayList();
        URLvalues=new ArrayList();
    	mapFrame.retrieveLineURL(this);
    	popup = new PopupMenu(this,mapFrame);
    	popup.addURLkeys(URLnames);
    	popup.addURLvalues(URLvalues);
    	popup.show(e.getComponent(),e.getX(), e.getY());
    }
    
    
    /**
     * Adds a column of statistics to the dialog showing stats for this line.
     */
    public void addStatisticColumn(Hashtable hash, String title) {
        if (dialog == null || hash.isEmpty())
            return;
        dialog.addStatisticColumn(hash, title);
    }
    public void addURL(String name,String value){
    	URLnames.add(name);
    	URLvalues.add(value);
    }
    /**
     * Adds an url button the the dialog showing stats for this line.
     */
    public void addURLlist() {
        if (dialog == null)
            return;
        
        String linkContent="";
        for (int i=0;i<URLnames.size();i++){
        	linkContent+="<p><a href="+URLvalues.get(i)+">"+URLnames.get(i)+"</a>\n";
        }
        dialog.addURLlist(linkContent);
    }
    
    public String toString() {
        return lineName;
    }
    
    /**
     * Compares the length of lines.
     */
    public int compareTo(Object o) {
        return -(getSize() - ((LineElement)o).getSize());
    }
}
