/*
 * MapElement.java
 *
 * Created on July 3, 2003, 10:17 AM
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.net.*;

import javax.jnlp.BasicService;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;

/**
 * MapElement contains the graphical image of a map along with lists of its points and lines.
 * It has methods for adjusting the position of overlapping lines and for placing text on
 * the map. Further it can assign statistic values to lines and points.
 *
 * @author Gunnar Gauslaa Bergem
 */
public class MapElement implements Selectable, Comparable {
    
    // list of lines and points
    private Vector lineList;
    private Vector filteredLineList;
    private Vector pointList;
    private Vector filteredPointList;
    
    private String mapName;    // name of the map
    private String coordType;  // type of coordinate system e.g. UTM-32, UTM-34 or bitmap
    
    // bitmap dimensions
    private int bitmapWidth;
    private int bitmapHeight;
    
    // lower left and upper right coordinates
    private int coordX1;
    private int coordY1;
    private int coordX2;
    private int coordY2;
    
    private int zoomLevel;          // indicates what detail level the map has
    private int UTMZone;            // UTM zone, e.g. 32
    private Image backgroundImage;  // the clean map image
    private URL myURL;              // URL pointing to the clean map image (if external is false) or an external system
    private float bitmapFactor;     // the ratio of coords/pixels
    private Rectangle rectangleOutline;  // rectangle border of the map
    private int[][] textMap;             // two-dimensional array used in text placement
    private boolean isSuperRegion;       // marks map as a superregion, the current top level map
    private boolean external;
    
    public static final int BORDER_SIZE = 20;  // border size between the map and the GUI
    public static final int GRID_SIZE = 12;    // text placement grid size
    private static final int OCCUPIED = 1;     // text placed in grid slot
    private static final int FREE = 0;         // text not placed in grid slot
    
    /** Creates a new instance of MapElement */
    public MapElement(String name, String coord, int width, int height, int x1, int y1, int x2, int y2, int zoom, URL imageURL, float factor, boolean external) {
        mapName = name;
        coordType = coord;
        bitmapWidth = width;
        bitmapHeight = height;
        coordX1 = x1;  // left x coordinate
        coordY1 = y1;  // bottom y coordinate
        coordX2 = x2;  // right x coordinate
        coordY2 = y2;  // top y coordinate
        zoomLevel = zoom;
        this.myURL = imageURL;
        bitmapFactor = factor;
        
        lineList = new Vector();
        filteredLineList = new Vector();
        pointList = new Vector();
        filteredPointList = new Vector();
        rectangleOutline = new Rectangle();
        this.external=external;
        
        // check if the coordType has a format like UTM-32
        if (coordType.startsWith("UTM")) {
            StringTokenizer st = new StringTokenizer(coordType, "-");
            st.nextToken(); // skip the "UTM" substring
            try {
                // extract the UTM zone
                UTMZone = Integer.parseInt(st.nextToken());
            } catch (NumberFormatException e) {
            }
        }
    }
    
    public void setPointList(Vector pointList) {
        this.pointList = pointList;
        filteredPointList.addAll(pointList);
    }
    
    public void setLineList(Vector lineList) {
        this.lineList = lineList;
        filteredLineList.addAll(lineList);
    }
    
    /**
     * Returns true if the image has been loaded, false otherwise.
     */
    public boolean isLoaded() {
        if (backgroundImage != null) {
            return true;
        }
        return false;
    }
    
    /**
     * Checks points along the border of the map and moves those that overlap.
     */
    public void adjustOverlappingPoints() {
        Rectangle intersection = getIntersectionRectangle();
        
        final int BORDER_GRID_SIZE = 30;
        
        int maxSquareX = backgroundImage.getWidth(null) / BORDER_GRID_SIZE;
        int maxSquareY = backgroundImage.getHeight(null) / BORDER_GRID_SIZE;
        int minX = (int)intersection.getX();
        int minY = (int)intersection.getY();
        int maxX = (int)(intersection.getX()+intersection.getWidth());
        int maxY = (int)(intersection.getY()+intersection.getHeight());
        
        // initialize the collision map
        int[][] pointBorderMap = new int[maxSquareX+1][maxSquareY+1];
        for (int i = 0; i < maxSquareX; i++) {
            for (int j = 0; j < maxSquareY; j++) {
                pointBorderMap[i][j] = FREE;
            }
        }
        for (int i = 0; i < filteredPointList.size(); i++) {
            int deltaX = 0;  // movement along the x axis
            int deltaY = 0;  // movement along the y axis
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            if (!pointElement.isEnabled())
                continue;
            int squareX = pointElement.getBitmapX() / BORDER_GRID_SIZE;
            int squareY = pointElement.getBitmapY() / BORDER_GRID_SIZE;
            int pointX = pointElement.getBitmapX();
            int pointY = pointElement.getBitmapY();
            
            // check that the point is at the edge of the map
            if (pointX == minX || pointX == maxX || pointY == minY || pointY == maxY) {
                while (pointBorderMap[squareX][squareY] == OCCUPIED) {
                    NemoUtils.debug("Border collision, moving "+pointElement);
                    // if the square is occupied, move the point clockwise
                    if (squareX < maxSquareX-1) {
                        squareX++;
                        deltaX++;
                    }
                    else if (squareY < maxSquareY-1) {
                        squareY++;
                        deltaY++;
                    }
                }
                // update the point coordinates
                pointBorderMap[squareX][squareY] = OCCUPIED;
                pointElement.updateBitmapX(pointElement.getBitmapX() + deltaX*BORDER_GRID_SIZE);
                pointElement.updateBitmapY(pointElement.getBitmapY() + deltaY*BORDER_GRID_SIZE);
                pointElement.createRectangleOutline();
            }
        }
        // create new line coordinates to reflect updates in the point coordinates
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement lineElement = (LineElement)filteredLineList.elementAt(i);
            lineElement.resetBitmapCoordinates();
            lineElement.createPolygonOutline();
        }
    }
    
    /**
     * Checks all the lines and moves those that are on top of each other.
     */
    public void adjustOverlappingLines() {
        for (int i = 0; i < lineList.size(); i++) {
            LineElement line1 = (LineElement)lineList.elementAt(i);
            for (int j = 0; j < lineList.size(); j++) {
                LineElement line2 = (LineElement)lineList.elementAt(j);
                int size2 = line2.getSize();
                if ((i != j) && line1.isEnabled() && line2.isEnabled()) {
                    int intersectLength = getIntersectLength(line1, line2);
                    if (intersectLength > size2*0.9) {
                        NemoUtils.debug("Displacing "+line1+" and "+line2);
                        displaceLine(line1, line2);
                    }
                }
            }
        }	
    }
    
    /**
     * Returns the length of the overlap of two lines.
     */
    public int getIntersectLength(LineElement line1, LineElement line2) {
        int originDistance = NemoUtils.distance(line1.getX1(), line1.getY1(), line2.getX1(), line2.getY1());
        int destDistance = NemoUtils.distance(line1.getX2(), line1.getY2(), line2.getX2(), line2.getY2());
        if (originDistance > 5 || destDistance > 5) {
            return 0;
        }
        Area a1 = new Area(line1.getPolygonOutline());
        Area a2 = new Area(line2.getPolygonOutline());
        a2.intersect(a1);
        Rectangle r = a2.getBounds();
        return (int)(Math.sqrt(Math.pow(r.getWidth(), 2) + Math.pow(r.getHeight(), 2)));
    }  
    
    /**
     * Displaces two lines with respect to each other.
     */
    public void displaceLine(LineElement line1, LineElement line2) {
        int newX1, newY1, newX2, newY2;
        int displaceX, displaceY, xlength, ylength;
        double angle;
        
        displaceX = displaceY = 6;
        
        newX1 = line1.getX1();
        newY1 = line1.getY1();
        newX2 = line1.getX2();
        newY2 = line1.getY2();
        
        xlength = Math.abs(newX2-newX1)+1;
        ylength = Math.abs(newY2-newY1);
        angle = Math.atan(ylength/xlength);
        newX1 += (int)(displaceX * Math.sin(angle));
        newX2 += (int)(displaceX * Math.sin(angle));
        newY1 += (int)(displaceY * Math.cos(angle));
        newY2 += (int)(displaceY * Math.cos(angle));
        line1.setX1(newX1);
        line1.setX2(newX2);
        line1.setY1(newY1);
        line1.setY2(newY2);
        line1.createPolygonOutline();
        
        newX1 = line2.getX1();
        newY1 = line2.getY1();
        newX2 = line2.getX2();
        newY2 = line2.getY2();
        
        xlength = Math.abs(newX2-newX1)+1;
        ylength = Math.abs(newY2-newY1);
        angle = Math.atan(ylength/xlength);
        newX1 -= (int)(displaceX * Math.sin(angle));
        newX2 -= (int)(displaceX * Math.sin(angle));
        newY1 -= (int)(displaceY * Math.cos(angle));
        newY2 -= (int)(displaceY * Math.cos(angle));
        line2.setX1(newX1);
        line2.setX2(newX2);
        line2.setY1(newY1);
        line2.setY2(newY2);
        line2.createPolygonOutline();
    }
    
    /**
     * Sets the clean map image, draws a border around it and initializes the text map.
     */
    public void setBackgroundImage(Image image) {
        BufferedImage buf = new BufferedImage(bitmapWidth + BORDER_SIZE*2, bitmapHeight + BORDER_SIZE*2, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = buf.createGraphics();
        g.setColor(new Color(235, 235, 235));
        g.fillRect(0, 0, bitmapWidth + BORDER_SIZE*2, bitmapHeight + BORDER_SIZE*2);
        g.drawImage(image, BORDER_SIZE, BORDER_SIZE, null);
        backgroundImage = buf;
        textMap = new int[backgroundImage.getWidth(null)/GRID_SIZE][backgroundImage.getHeight(null)/GRID_SIZE];
    }
    
    /**
     * Returns the clean map image.
     */
    public Image getBackgroundImage() {
        return backgroundImage;
    }
    
    /**
     * Returns the coordinates of where the text should be placed given a pair
     * of coordinates and text length.
     */
    public int[] getTextCoords(int pointX, int pointY, int length) {
        int startingSquareX = pointX / GRID_SIZE;
        int startingSquareY = pointY / GRID_SIZE;
        int[] returnCoords = new int[2];
        returnCoords[0] = startingSquareX * GRID_SIZE;
        returnCoords[1] = startingSquareY * GRID_SIZE;
        int maxCircle = 10;
        length = length / GRID_SIZE + 1;
        
        for (int circle = 1; circle < maxCircle; circle++) {
            int squareX = pointX / GRID_SIZE;
            int squareY = pointY / GRID_SIZE;
            for (int offsetY = 1; offsetY >= -1; offsetY--) {
                for (int offsetX = 1; offsetX >= -1; offsetX--) {
                    squareX = startingSquareX + circle*offsetX;
                    squareY = startingSquareY + circle*offsetY;
                    boolean occupied = false;
                    if ((squareX+length < textMap.length) && (squareX >= 0) && (squareY < textMap[0].length) && (squareY >= 0)) {
                        for (int i = 0; i < length; i++) {
                            if (textMap[squareX+i][squareY] == OCCUPIED) {
                                occupied = true;
                                //System.out.println("occupied: circle "+circle+" squarex "+squareX+" squarey "+squareY);
                                break;
                            }
                        }
                        if (!occupied) {
                            //System.out.println("free: circle "+circle+" squarex "+squareX+" squarey "+squareY);
                            returnCoords[0] = squareX * GRID_SIZE;
                            returnCoords[1] = squareY * GRID_SIZE;
                            setOccupiedText(squareX, squareY, length);
                            return returnCoords;
                        }
                    }
                    else {
                        //System.out.println("text out of bounds x: "+squareX+" y: "+squareY);
                    }
                }
            }
        }
        setOccupiedText(startingSquareX, startingSquareY, length);
        return returnCoords;
    }
    
    /**
     * Marks the given slot in the text map as occupied.
     */
    public void setOccupiedText(int x, int y, int length) {
        for (int i = 0; i < length; i++) {
            if ((x+length < textMap.length) && (x >= 0) && (y < textMap[0].length) && (y >= 0)) {
                textMap[x+i][y] = OCCUPIED;
            }
        }
    }
    
    /**
     * Resets the text map.
     */
    public void clearTextMap() {
        for (int i = 0; i < textMap.length; i++) {
            for (int j = 0; j < textMap[0].length; j++) {
                textMap[i][j] = FREE;
                if ((j > textMap[0].length-4) || (j < 3) || (i > textMap.length-2) || (i < 2)) {
                    textMap[i][j] = OCCUPIED;
                }
            }
        }
        
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            int squareX = pointElement.getBitmapX() / GRID_SIZE;
            int squareY = pointElement.getBitmapY() / GRID_SIZE;
            if (squareX >= 0 && squareX < textMap.length && squareY >= 0 && squareY < textMap[0].length) {
                textMap[squareX][squareY] = OCCUPIED;
                if (squareY+1 < textMap[0].length)
                    textMap[squareX][squareY+1] = OCCUPIED;
                if (squareY-1 > 0)
                    textMap[squareX][squareY-1] = OCCUPIED;
            }
        }
    }
    
    /**
     * Returns the intersection rectangle, which is in the middle between the map and the border.
     */
    public Rectangle getIntersectionRectangle() {
        return new Rectangle(BORDER_SIZE/2, BORDER_SIZE/2, bitmapWidth + BORDER_SIZE, bitmapHeight + BORDER_SIZE);
    }
    
    /**
     * Returns the contour rectangle, which is the same as the outline of the map.
     */
    public Rectangle getContourRectangle() {
        return new Rectangle(BORDER_SIZE, BORDER_SIZE, bitmapWidth, bitmapHeight);
    }
    
    /**
     * Returns the map image for a given date with lines, points and text drawn on it.
     */
    public Image getMapImage(Date date) {
        // draw the clean map image
        BufferedImage offscreen = new BufferedImage(bitmapWidth + BORDER_SIZE*2, bitmapHeight + BORDER_SIZE*2, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = offscreen.createGraphics();
        g.drawImage(backgroundImage, 0, 0, null);
        clearTextMap();
        
        // sort the lines so that the longest lines are drawn first
        Collections.sort(filteredLineList);
        
        // disable the lines and points that dont exist at the given date
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement lineElement = (LineElement)filteredLineList.elementAt(i);
            lineElement.checkEnabled(date);
        }
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            pointElement.checkEnabled(date);
        }
        
        // move points that overlap each other at the map edges
        adjustOverlappingPoints();
        // move lines that overlap each other (parallell displacement)
        adjustOverlappingLines();
        
        // draw the lines and points
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement lineElement = (LineElement)filteredLineList.elementAt(i);
            lineElement.drawMe(g);
        }
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            pointElement.drawMe(g);
        }
        
        // sort the lines so that the shortest lines are selected first with the mouse
        Collections.reverse(filteredLineList);
        return offscreen;
    }
    
    /**
     * Returns a point element given a pointname.
     */
    public PointElement getPointElement(String pointName) {
        PointElement pointElement = null;
        for (int i = 0; i < pointList.size(); i++) {
            pointElement = (PointElement)pointList.elementAt(i);
            if ((pointElement.toString()).compareTo(pointName) == 0) {
                return pointElement;
            }
        }
        return null;
    }
    
    /**
     * Returns a line element given a linename.
     */
    public LineElement getLineElement(String lineName) {
        LineElement lineElement = null;
        for (int i = 0; i < lineList.size(); i++) {
            lineElement = (LineElement)lineList.elementAt(i);
            if ((lineElement.toString()).compareTo(lineName) == 0) {
                return lineElement;
            }
        }
        return null;
    }
    
    /**
     * Returns true if the given map element is contained inside this map element.
     */
    public boolean contains(MapElement otherElement) {
        boolean inside = false;
        
        if ( (coordX1 < otherElement.getX1()) && (coordY1 < otherElement.getY1()) && 
                (coordX2 > otherElement.getX2()) && (coordY2 > otherElement.getY2()) ) {
            inside = true;
            //System.out.println(this + " contains " + otherElement);
        }
        return inside;
        
    }
    
    public boolean overlaps(MapElement otherElement) {
        boolean overlaps = false;
        
        if ( ((coordX1 < otherElement.getX1()) && (coordX2 > otherElement.getX1()) || 
                (coordX1 < otherElement.getX2()) && (coordX2 > otherElement.getX2())) &&
                ((coordY1 < otherElement.getY1()) && (coordY2 > otherElement.getY1()) ||
                        (coordY1 < otherElement.getY2()) && (coordY2 > otherElement.getY2()))) {
            overlaps = true;
            //NemoUtils.debug(this + " is overlapped by " + otherElement);
        }
        return overlaps;
    }
    
    /**
     * Returns true if the given point is located within this map element.
     */
    public boolean contains(Point point) {
        return rectangleOutline.contains(point);
    }
    
    /**
     * Returns the ratio of coords/pixels along the x-axis.
     */
    public double factorX() {
        if (bitmapFactor != 0)
            return bitmapFactor;
        
        double coordUnits = (double)(coordX2 - coordX1);
        double pixels = (double)(bitmapWidth);
        if (pixels==0) return 0;
        return coordUnits / pixels;
    }
    
    /**
     * Returns the ratio of coords/pixels along the y-axis.
     */
    public double factorY() {
        if (bitmapFactor != 0)
            return bitmapFactor;
        
        double coordUnits = (double)(coordY2 - coordY1);
        double pixels = (double)(bitmapHeight);
        if (pixels==0) return 0;
        return coordUnits / pixels;
    }
    
    public int getX1() {
        return coordX1;
    }
    
    public int getY1() {
        return coordY1;
    }
    
    public int getX2() {
        return coordX2;
    }
    
    public int getY2() {
        return coordY2;
    }
    
    public int getUTMZone() {
        return UTMZone;
    }
    
    public int getZoomLevel() {
        return zoomLevel;
    }
    
    public String getMapName() {
        return mapName;
    }
    
    public URL getImageURL() {
        return myURL;
    }
    
    public int getRegionSize() {
        return (int)(rectangleOutline.getWidth() * rectangleOutline.getHeight());
    }
    
    /**
     * Marks the given map element as a super region for this map element.
     */
    public void createSuperRegion(MapElement superElement) {
        superElement.createRectangleOutline(BORDER_SIZE, BORDER_SIZE, bitmapWidth, bitmapHeight);
        superElement.setSuperRegion(true);
    }
    
    /**
     * Marks the given map element as a sub region for this map element.
     */
    public void createSubRegion(MapElement subElement) {
        int x, y, width, height, subX1, subY1, subX2, subY2;
        int minimum_pixels = 60;
        
        subX1 = (int)subElement.getX1();
        subY1 = (int)subElement.getY1();
        subX2 = (int)subElement.getX2();
        subY2 = (int)subElement.getY2();
        
        if (subElement.getUTMZone() != UTMZone) {
            dPair coordPair1 = UTM_Converter.UTMtoUTM(subElement.getY1(), subElement.getX1(), subElement.getUTMZone(), UTMZone);
            dPair coordPair2 = UTM_Converter.UTMtoUTM(subElement.getY2(), subElement.getX2(), subElement.getUTMZone(), UTMZone);
            subX1 = (int)Math.round(coordPair1.Y);
            subY1 = (int)Math.round(coordPair1.X);
            subX2 = (int)Math.round(coordPair2.Y);
            subY2 = (int)Math.round(coordPair2.X);
        }
        // the x and y represents the top left pixel coordinates of the rectangle
        x = (int)((subX1 - coordX1) / factorX());
        y = (int)((coordY2 - subY2) / factorY());
        // pixel width and pixel height of the rectangle
        width = (int)((subX2 - coordX1) / factorX()) - x;
        height = (int)((coordY2 - subY1) / factorY()) - y;
        
        // make sure the rectangle is at least minimum_pixels x minimum_pixels big
        if (width < minimum_pixels) {
            x = x - ((minimum_pixels-width) / 2);
            width = minimum_pixels;
        }
        if (height < minimum_pixels) {
            y = y - ((minimum_pixels-height) / 2);
            height = minimum_pixels;
        }
        subElement.createRectangleOutline(x + BORDER_SIZE, y + BORDER_SIZE, width, height);
    }
    
    /**
     * Creates a rectangle outline with the given dimensions.
     */    
    public void createRectangleOutline(int x, int y, int width, int height) {
        rectangleOutline = new Rectangle(x, y, width, height);
    }
    
    /**
     * Clears the rectangle outline.
     */
    public void removeRegion() {
        rectangleOutline = new Rectangle();
        isSuperRegion = false;
    }
    
    /**
     * Returns true if this map element is a super region.
     */
    public boolean isSuperRegion() {
        return isSuperRegion;
    }
    
    /**
     * Sets this map element as a super region.
     */
    public void setSuperRegion(boolean isSuper) {
        isSuperRegion = isSuper;
    }
    
    /**
     * Creates regions for all the lines.
     */
    public void createLineRegions() {
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement lineElement = (LineElement)filteredLineList.elementAt(i);
            lineElement.createPolygonOutline();
        }
    }
    
    /**
     * Creates regions for all the points.
     */
    public void createPointRegions() {
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            pointElement.createRectangleOutline();
        }
    }
    
    /**
     * Returns the point element at a given point.
     */
    public PointElement getNodeAtPoint(Point point) {
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement pointElement = (PointElement)filteredPointList.elementAt(i);
            if (pointElement.contains(point)) {
                return pointElement;
            }
        }
        return null;
    }
    
    /**
     * Returns the line element at a given point.
     */
    public LineElement getLineAtPoint(Point point) {
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement lineElement = (LineElement)filteredLineList.elementAt(i);
            if (lineElement.contains(point)) {
                return lineElement;
            }
        }
        return null;
    }
    
    /**
     * Filters all the lines, keeping only those with a spesific linetype.
     */
    public void filterLines(String lineType) {
        filteredLineList.removeAllElements();
        filteredPointList.removeAllElements();
        filteredPointList.addAll(pointList);
        
        // if no filter is given, or the filter includes all, add all elements from lineList
        if ((lineType == null) || (lineType.compareTo("all") == 0)) {
            filteredLineList.addAll(lineList);
            return;
        }
        
        // add only lines that has the correct lineType to filteredLineList
        for (int i = 0; i < lineList.size(); i++) {
            LineElement line = (LineElement)lineList.elementAt(i);
            if (line.getType().compareTo(lineType) == 0) {
                filteredLineList.addElement(line);
            }
        }
        
        // if we have no filteredPointList, make a fresh one
        if (filteredPointList.isEmpty()) {
            filteredPointList.addAll(pointList);
        }
        
        // remove all points from filteredPointList that isn't connected to any line in
        // filteredLineList
        for (int i = 0; i < filteredPointList.size(); i++) {
            PointElement point = (PointElement)filteredPointList.elementAt(i);
            boolean connected = false;
            for (int j = 0; j < filteredLineList.size(); j++) {
                LineElement line = (LineElement)filteredLineList.elementAt(j);
                if (line.getOrigin() == point || line.getDestination() == point) {
                    connected = true;
                }
            }
            if (!connected) {
                filteredPointList.removeElementAt(i);
                i--;
            }
        }
    }
    
    /**
     * Filters all the points, keeping only those with a spesific pointtype.
     */
    public void filterPoints(String pointType) {
        filteredPointList.removeAllElements();
        filteredLineList.removeAllElements();
        filteredLineList.addAll(lineList);
        
        // if no filter is given, or the filter includes all, add all elements from pointList
        if ((pointType == null) || (pointType.compareTo("all") == 0)) {
            filteredPointList.addAll(pointList);
            return;
        }
        
        // add only points that has the correct pointType to filteredPointList
        for (int i = 0; i < pointList.size(); i++) {
            PointElement point = (PointElement)pointList.elementAt(i);
            if (point.getType().compareTo(pointType) == 0) {
                filteredPointList.addElement(point);
            }
        }
        
        // if we have no filteredLineList, make a fresh one
        if (filteredLineList.isEmpty()) {
            filteredLineList.addAll(lineList);
        }
        
        // remove all lines from filteredLineList that isn't connected by its origin or
        // destination to a point in filteredPointList
        for (int i = 0; i < filteredLineList.size(); i++) {
            LineElement line = (LineElement)filteredLineList.elementAt(i);
            boolean originConnected = false;
            boolean destinationConnected = false;
            for (int j = 0; j < filteredPointList.size(); j++) {
                PointElement point = (PointElement)filteredPointList.elementAt(j);
                if (line.getOrigin() == point) {
                    originConnected = true;
                }
                if (line.getDestination() == point) {
                    destinationConnected = true;
                }
            }
            if (!originConnected || !destinationConnected) {
                filteredLineList.removeElementAt(i);
                i--;
            }
        }
    }
    
    /**
     * Returns all the linetypes.
     */
    public Vector getLineTypes() {
        Vector lineTypes = new Vector();
        for (int i = 0; i < lineList.size(); i++) {
            LineElement line = (LineElement)lineList.elementAt(i);
            if ( !lineTypes.contains(line.getType()) ) {
                lineTypes.addElement(line.getType());
            }
        }
        return lineTypes;
    }
    
    /**
     * Returns all the pointtypes.
     */
    public Vector getPointTypes() {
        Vector pointTypes = new Vector();
        for (int i = 0; i < pointList.size(); i++) {
            PointElement point = (PointElement)pointList.elementAt(i);
            if ( !pointTypes.contains(point.getType()) ) {
                pointTypes.addElement(point.getType());
            }
        }
        return pointTypes;
    }
    
    /**
     * Assigns all the available line statistics to line elements.
     */
    public void assignLineStatistics(Vector lineStatList) {
        for (int i = 0; i < lineStatList.size(); i++) {
            LineStatistic lineStat = (LineStatistic)lineStatList.elementAt(i);
            Date statisticDate = new Date((long)lineStat.getTimestamp()*(long)1000);
            
            for (int j = 0; j < lineList.size(); j++) {
                LineElement lineElement = (LineElement)lineList.elementAt(j);
                String lineName = lineElement.toString();
                
                if (lineStat.getLineName().compareTo(lineName) == 0) {
                    lineElement.addTimestamp(statisticDate, lineStat.getDescription());
                    if (lineStat.getType().endsWith("size")) {
                        lineElement.setOriginSize(lineStat.getOriginSize());
                        lineElement.setDestinationSize(lineStat.getDestinationSize());
                    }
                    else if(lineStat.getType().endsWith("colour")) {
                        lineElement.setOriginColour(lineStat.getOriginColour());
                        lineElement.setDestinationColour(lineStat.getDestinationColour());
                    }
                    break;
                }
            }
        }
    }
    
    /**
     * Assigns all the available point statistics to point elements.
     */
    public void assignPointStatistics(Vector pointStatList) {
        for (int i = 0; i < pointStatList.size(); i++) {
            PointStatistic pointStat = (PointStatistic)pointStatList.elementAt(i);
            Date statisticDate = new Date((long)pointStat.getTimestamp()*(long)1000);
            
            for (int j = 0; j < pointList.size(); j++) {
                PointElement pointElement = (PointElement)pointList.elementAt(j);
                String pointName = pointElement.toString();
                
                if (pointStat.getPointName().compareTo(pointName) == 0) {
                    pointElement.addTimestamp(statisticDate, pointStat.getDescription());
                    if (pointStat.getType().endsWith("size")) {
                        pointElement.setSize(pointStat.getSize());
                    }
                    else if (pointStat.getType().endsWith("colour")) {
                        pointElement.setColour(pointStat.getColour());
                    }
                    break;
                }
            }
        }
    }
    
    /**
     * Clears all the point statistics.
     */
    public void resetPointStatistics() {
        for (int i = 0; i < pointList.size(); i++) {
            PointElement pointElement = (PointElement)pointList.elementAt(i);
            pointElement.resetStatistics();
        }
    }
    
    /**
     * Clears all the line statistics.
     */
    public void resetLineStatistics() {
        for (int i = 0; i < lineList.size(); i++) {
            LineElement lineElement = (LineElement)lineList.elementAt(i);
            lineElement.resetStatistics();
        }
    }
    
    /**
     * Draws the map region when selected with the mouse pointer.
     */
    public void drawSelected(Graphics _g) {
        Graphics2D g = (Graphics2D)_g;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        g.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        
        g.setColor(Color.black);
        g.drawRect((int)rectangleOutline.getX(), (int)rectangleOutline.getY(), (int)rectangleOutline.getWidth(), (int)rectangleOutline.getHeight());
        FontRenderContext frc = g.getFontRenderContext();
        Font font = new Font("Times", Font.ITALIC, 16);
        TextLayout tl = new TextLayout(mapName, font, frc);
        g.setColor(Color.white);
        int rectangleXpos = (int)(rectangleOutline.getX());
        int rectangleYpos = (int)(rectangleOutline.getY());
        if (rectangleXpos >= 0 && rectangleXpos <= bitmapWidth && rectangleYpos >= 0) {
            rectangleXpos = (int)(rectangleOutline.getX()+rectangleOutline.getWidth()/2-tl.getVisibleAdvance()/2);
            rectangleYpos = (int)(rectangleOutline.getY()-tl.getAscent()-tl.getDescent());
            g.fillRect(rectangleXpos, rectangleYpos, (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
            g.setColor(Color.black);
            g.drawRect(rectangleXpos, rectangleYpos, (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
            tl.draw(g, (float)(rectangleXpos+1), (float)(rectangleYpos+tl.getAscent()));
        }
        else {
            Rectangle contour = getContourRectangle();
            Rectangle intersection = contour.intersection(rectangleOutline);
            rectangleXpos = (int)(intersection.getX()+intersection.getWidth()/2);
            rectangleYpos = (int)(intersection.getY()+intersection.getHeight()/2);
            g.fillRect(rectangleXpos, rectangleYpos, (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
            g.setColor(Color.black);
            g.drawRect(rectangleXpos, rectangleYpos, (int)tl.getVisibleAdvance()+2, (int)(tl.getAscent()+tl.getDescent()));
            tl.draw(g, (float)(rectangleXpos+1), (float)(rectangleYpos+tl.getAscent()));
        }
    }
    
    /**
     * If map is internal - selects this map when it is clicked on with the mouse.
     * If map is external - opens an external view in the Internet browser
     */
    public void performMouseClickAction(MapFrame mapFrame, MouseEvent e) {
        if (!external){
        	mapFrame.selectMap(this);
        }else{
        	try {
		           BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
		           bs.showDocument(myURL);
		    } catch(UnavailableServiceException ue) {
		           System.err.println("Browser service not available.\n");
		    }      	
        }
    }
    
    /**
     * Returns the name of the map.
     */
    public String toString() {
        return mapName;
    }
    
    /**
     * Returns the string comparison of the map names.
     */
    public int compareTo(Object o) {
        return toString().compareTo(o.toString());
    }
    
    public boolean equals(Object o) {
        if (this.compareTo(o) == 0) {
            return true;
        }
        return false;
    }
    
    //unimplemented since there is no statistics for MapElements
    public void showStatistics(MapFrame mapFrame, MouseEvent e){}

	public void addURL(String urlInfo, String urlString) {
		// unused...
	}
}
