/*
 * MapFrame.java
 *
 * Created on June 12, 2002, 2:12 PM
 */

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import java.net.*;
import javax.jnlp.*;

/**
 * This class is responsible for everything that is related to creating and displaying
 * maps. The class lets the user choose the current network and map to be displayed
 * and the time period for the statistic. The user can also filter the map to show only
 * certain lines or nodes.
 *
 */
public class MapFrame extends javax.swing.JFrame {
    private MapImageComponent mapImageComponent;  // jlabel component holding the mapimage
    private Image mapImage;                       // the mapimage
    private MapMouseListener mapMouseListener;    // listens for mouse events on the map
    private Netmonitor nm;                        // reference to the netmonitor class
    private javax.swing.Timer animationTimer;     // timer that controls the animation
    private int statAnimationFrame;               // the animation frame number
    private boolean showOptions;                  // true if options are shown, false otherwise
    private javax.swing.Timer statisticIntervalTimer;  // timer that updates the statistics regularly
    private SliderListener sliderListener;        // listens for events on the time slider
    private Date date;                            // the date for the map and statistics
    private int timeAxisSize;                     // length of the time axis, day - month - year
    private int timeStepSize;                     // length of the time interval, 5min - hour - day
    private boolean blockDateUpdate;              // used for blocking events on the time slider
    private boolean animationPlaying;             // true if we are animating, false otherwise
    private boolean autoUpdating;                 // true if we are automatically updating statistics, false otherwise
    private Selectable activeElement;             // the currently active element on the map
    private int fps;                              // animation frames per second
    
    /**
     * Creates a new MapFrame instance.
     *
     * @param nm a reference to the Netmonitor class
     */
    public MapFrame(Netmonitor nm) {
        initComponents();
        this.nm = nm;
        
        // check for new network statistics every minute as default
        statisticIntervalTimer = new javax.swing.Timer(60000, new StatisticTimerListener());
        statisticIntervalTimer.start();
        mapMouseListener = new MapMouseListener();
        showOptions = true;
        
        timeStepSize = NemoUtils.FIVEMIN_STEP;
        timeAxisSize = NemoUtils.DAY_AXIS;
        
        mapImage = this.createImage(540, 704);
        mapImageComponent = new MapImageComponent();
        mapImageComponent.addMouseListener(mapMouseListener);
        mapImageComponent.addMouseMotionListener(mapMouseListener);
        JScrollPane mapScrollPane = new JScrollPane(mapImageComponent, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        map_panel.add(mapScrollPane, BorderLayout.CENTER);
        mapImageComponent.setPreferredSize(new Dimension(mapImage.getWidth(this), mapImage.getHeight(this)));
        mapImageComponent.revalidate();
        pack();
        
        animationTimer = new javax.swing.Timer(100, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                performStatAnimation();
            }
        });
        animationTimer.setInitialDelay(0);
        
        network_comboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleChosenNetwork();
            }
        });
        map_comboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                handleChosenMap();
            }
        });
        lines_comboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filterLines();
            }
        });
        nodes_comboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filterPoints();
            }
        });
        frames_comboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setFramesPerSecond();
            }
        });
        autoUpdate_checkBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    statisticIntervalTimer.setInitialDelay(0);
                    statisticIntervalTimer.start();
                }
                else if (e.getStateChange() == ItemEvent.DESELECTED) {
                    statisticIntervalTimer.stop();
                }
            }
        });
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                stop();
                System.exit(0);
            }
        });
        
        sliderListener = new SliderListener();
        time_slider.addChangeListener(sliderListener);
        time_slider.setSnapToTicks(true);
        date = getCurrentDate();
        blockDateUpdate = false;
        autoUpdating = false;
        setFramesPerSecond();
        setStepSize(NemoUtils.FIVEMIN_STEP);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        doc_button = new javax.swing.JButton();
        doc_panel = new javax.swing.JPanel();
        buttonGroup = new javax.swing.ButtonGroup();
        map_panel = new javax.swing.JPanel();
        slider_panel = new javax.swing.JPanel();
        time_slider = new javax.swing.JSlider();
        slider_label = new javax.swing.JLabel();
        toolbar = new javax.swing.JToolBar();
        decDay_button = new javax.swing.JButton();
        day_label = new javax.swing.JLabel();
        incDay_button = new javax.swing.JButton();
        blankSpace1_label = new javax.swing.JLabel();
        decMonth_button = new javax.swing.JButton();
        month_label = new javax.swing.JLabel();
        incMonth_button = new javax.swing.JButton();
        blankSpace2_label = new javax.swing.JLabel();
        decYear_button = new javax.swing.JButton();
        year_label = new javax.swing.JLabel();
        incYear_button = new javax.swing.JButton();
        blankSpace3_label = new javax.swing.JLabel();
        showOptions_button = new javax.swing.JButton();
        tabbedPane = new javax.swing.JTabbedPane();
        maplist_panel = new javax.swing.JPanel();
        mapSelection_panel = new javax.swing.JPanel();
        selectStaticLabel_panel = new javax.swing.JPanel();
        network_label = new javax.swing.JLabel();
        map_label = new javax.swing.JLabel();
        selectChoice_panel = new javax.swing.JPanel();
        network_comboBox = new javax.swing.JComboBox();
        map_comboBox = new javax.swing.JComboBox();
        showNodesLines_panel = new javax.swing.JPanel();
        show_panel = new javax.swing.JPanel();
        showStaticLabel_panel = new javax.swing.JPanel();
        nodes_label = new javax.swing.JLabel();
        lines_label = new javax.swing.JLabel();
        showChoice_panel = new javax.swing.JPanel();
        nodes_comboBox = new javax.swing.JComboBox();
        lines_comboBox = new javax.swing.JComboBox();
        timeOptions_panel = new javax.swing.JPanel();
        intervalSize_panel = new javax.swing.JPanel();
        staticIntervalSize_label = new javax.swing.JLabel();
        zoomIn_button = new javax.swing.JButton();
        zoomOut_button = new javax.swing.JButton();
        dynIntervalSize_label = new javax.swing.JLabel();
        autoUpdate_panel = new javax.swing.JPanel();
        autoUpdate_checkBox = new javax.swing.JCheckBox();
        autoUpdateBlank_label = new javax.swing.JLabel();
        animation_panel = new javax.swing.JPanel();
        animation_label = new javax.swing.JLabel();
        playPause_panel = new javax.swing.JPanel();
        stepBackward_button = new javax.swing.JButton();
        play_button = new javax.swing.JButton();
        stepForward_button = new javax.swing.JButton();
        frames_panel = new javax.swing.JPanel();
        frames_label = new javax.swing.JLabel();
        frames_comboBox = new javax.swing.JComboBox();
        framesBlank_label = new javax.swing.JLabel();
        statistic_panel = new javax.swing.JPanel();
        description_panel = new javax.swing.JPanel();
        description_scrollPane = new javax.swing.JScrollPane();
        statIntervals_panel = new javax.swing.JPanel();
        assoc_panel = new javax.swing.JPanel();
        statAssocLabel_panel = new javax.swing.JPanel();
        lineColour_label = new javax.swing.JLabel();
        lineSize_label = new javax.swing.JLabel();
        pointColour_label = new javax.swing.JLabel();
        pointSize_label = new javax.swing.JLabel();
        assocComboBox_panel = new javax.swing.JPanel();
        lineColour_comboBox = new javax.swing.JComboBox();
        lineSize_comboBox = new javax.swing.JComboBox();
        pointColour_comboBox = new javax.swing.JComboBox();
        pointSize_comboBox = new javax.swing.JComboBox();
        
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("NEMO");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        
        map_panel.setLayout(new java.awt.BorderLayout(0, 5));
        
        map_panel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)));
        slider_panel.setLayout(new java.awt.BorderLayout());
        
        time_slider.setMaximum(365);
        time_slider.setMinimum(1);
        time_slider.setPaintLabels(true);
        slider_panel.add(time_slider, java.awt.BorderLayout.CENTER);
        
        slider_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        slider_label.setText("20. September 2002");
        slider_panel.add(slider_label, java.awt.BorderLayout.NORTH);
        
        map_panel.add(slider_panel, java.awt.BorderLayout.SOUTH);
        
        decDay_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Back24.gif")));
        decDay_button.setMaximumSize(new java.awt.Dimension(26, 28));
        decDay_button.setMinimumSize(new java.awt.Dimension(26, 28));
        decDay_button.setPreferredSize(new java.awt.Dimension(26, 28));
        decDay_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decDay_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(decDay_button);
        
        day_label.setFont(new java.awt.Font("Dialog", 0, 12));
        day_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        day_label.setText("Day");
        day_label.setBorder(new javax.swing.border.EtchedBorder());
        day_label.setMaximumSize(new java.awt.Dimension(35, 28));
        day_label.setMinimumSize(new java.awt.Dimension(35, 28));
        day_label.setPreferredSize(new java.awt.Dimension(35, 28));
        toolbar.add(day_label);
        
        incDay_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Forward24.gif")));
        incDay_button.setMaximumSize(new java.awt.Dimension(26, 28));
        incDay_button.setPreferredSize(new java.awt.Dimension(26, 28));
        incDay_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                incDay_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(incDay_button);
        
        blankSpace1_label.setText(" ");
        blankSpace1_label.setMaximumSize(new java.awt.Dimension(35, 25));
        blankSpace1_label.setMinimumSize(new java.awt.Dimension(35, 25));
        blankSpace1_label.setPreferredSize(new java.awt.Dimension(35, 25));
        toolbar.add(blankSpace1_label);
        
        decMonth_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Back24.gif")));
        decMonth_button.setMaximumSize(new java.awt.Dimension(26, 28));
        decMonth_button.setMinimumSize(new java.awt.Dimension(26, 28));
        decMonth_button.setPreferredSize(new java.awt.Dimension(26, 28));
        decMonth_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decMonth_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(decMonth_button);
        
        month_label.setFont(new java.awt.Font("Dialog", 0, 12));
        month_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        month_label.setText("Month");
        month_label.setBorder(new javax.swing.border.EtchedBorder());
        month_label.setMaximumSize(new java.awt.Dimension(45, 28));
        month_label.setMinimumSize(new java.awt.Dimension(45, 28));
        month_label.setPreferredSize(new java.awt.Dimension(45, 28));
        toolbar.add(month_label);
        
        incMonth_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Forward24.gif")));
        incMonth_button.setMaximumSize(new java.awt.Dimension(26, 28));
        incMonth_button.setMinimumSize(new java.awt.Dimension(26, 28));
        incMonth_button.setPreferredSize(new java.awt.Dimension(26, 28));
        incMonth_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                incMonth_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(incMonth_button);
        
        blankSpace2_label.setText(" ");
        blankSpace2_label.setMaximumSize(new java.awt.Dimension(35, 25));
        blankSpace2_label.setMinimumSize(new java.awt.Dimension(35, 25));
        blankSpace2_label.setPreferredSize(new java.awt.Dimension(35, 25));
        toolbar.add(blankSpace2_label);
        
        decYear_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Back24.gif")));
        decYear_button.setMaximumSize(new java.awt.Dimension(26, 28));
        decYear_button.setMinimumSize(new java.awt.Dimension(26, 28));
        decYear_button.setPreferredSize(new java.awt.Dimension(26, 28));
        decYear_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                decYear_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(decYear_button);
        
        year_label.setFont(new java.awt.Font("Dialog", 0, 12));
        year_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        year_label.setText("Year");
        year_label.setBorder(new javax.swing.border.EtchedBorder());
        year_label.setMaximumSize(new java.awt.Dimension(45, 28));
        year_label.setMinimumSize(new java.awt.Dimension(45, 28));
        year_label.setPreferredSize(new java.awt.Dimension(45, 28));
        toolbar.add(year_label);
        
        incYear_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Forward24.gif")));
        incYear_button.setMaximumSize(new java.awt.Dimension(26, 28));
        incYear_button.setMinimumSize(new java.awt.Dimension(26, 28));
        incYear_button.setPreferredSize(new java.awt.Dimension(26, 28));
        incYear_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                incYear_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(incYear_button);
        
        blankSpace3_label.setText(" ");
        blankSpace3_label.setMaximumSize(new java.awt.Dimension(35, 25));
        blankSpace3_label.setMinimumSize(new java.awt.Dimension(35, 25));
        blankSpace3_label.setPreferredSize(new java.awt.Dimension(35, 25));
        toolbar.add(blankSpace3_label);
        
        showOptions_button.setText("<<");
        showOptions_button.setMaximumSize(new java.awt.Dimension(32, 28));
        showOptions_button.setMinimumSize(new java.awt.Dimension(32, 28));
        showOptions_button.setPreferredSize(new java.awt.Dimension(32, 28));
        showOptions_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showOptions_buttonActionPerformed(evt);
            }
        });
        
        toolbar.add(showOptions_button);
        
        map_panel.add(toolbar, java.awt.BorderLayout.NORTH);
        
        getContentPane().add(map_panel, java.awt.BorderLayout.CENTER);
        
        maplist_panel.setLayout(new java.awt.BorderLayout());
        
        maplist_panel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(10, 10, 10, 10)));
        mapSelection_panel.setLayout(new java.awt.BorderLayout(5, 0));
        
        mapSelection_panel.setBorder(new javax.swing.border.TitledBorder("Location"));
        selectStaticLabel_panel.setLayout(new java.awt.GridLayout(2, 1));
        
        network_label.setFont(new java.awt.Font("Dialog", 0, 12));
        network_label.setText("Network:");
        selectStaticLabel_panel.add(network_label);
        
        map_label.setFont(new java.awt.Font("Dialog", 0, 12));
        map_label.setText("Map:");
        selectStaticLabel_panel.add(map_label);
        
        mapSelection_panel.add(selectStaticLabel_panel, java.awt.BorderLayout.WEST);
        
        selectChoice_panel.setLayout(new java.awt.GridLayout(2, 1));
        
        network_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        selectChoice_panel.add(network_comboBox);
        
        map_comboBox.setMaximumRowCount(20);
        map_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        selectChoice_panel.add(map_comboBox);
        
        mapSelection_panel.add(selectChoice_panel, java.awt.BorderLayout.CENTER);
        
        maplist_panel.add(mapSelection_panel, java.awt.BorderLayout.NORTH);
        
        showNodesLines_panel.setLayout(new javax.swing.BoxLayout(showNodesLines_panel, javax.swing.BoxLayout.Y_AXIS));
        
        show_panel.setLayout(new java.awt.BorderLayout(5, 0));
        
        show_panel.setBorder(new javax.swing.border.TitledBorder("Filter"));
        showStaticLabel_panel.setLayout(new java.awt.GridLayout(2, 1));
        
        nodes_label.setFont(new java.awt.Font("Dialog", 0, 12));
        nodes_label.setText("Nodes:");
        showStaticLabel_panel.add(nodes_label);
        
        lines_label.setFont(new java.awt.Font("Dialog", 0, 12));
        lines_label.setText("Lines:");
        showStaticLabel_panel.add(lines_label);
        
        show_panel.add(showStaticLabel_panel, java.awt.BorderLayout.WEST);
        
        showChoice_panel.setLayout(new java.awt.GridLayout(2, 1));
        
        nodes_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "all", "sted", "stedsky", "satellitt" }));
        showChoice_panel.add(nodes_comboBox);
        
        lines_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "all", "intern", "knutepunkt", "aksess", "stamnett" }));
        showChoice_panel.add(lines_comboBox);
        
        show_panel.add(showChoice_panel, java.awt.BorderLayout.CENTER);
        
        showNodesLines_panel.add(show_panel);
        
        maplist_panel.add(showNodesLines_panel, java.awt.BorderLayout.SOUTH);
        
        timeOptions_panel.setLayout(new java.awt.BorderLayout(0, 5));
        
        timeOptions_panel.setBorder(new javax.swing.border.TitledBorder("Time"));
        intervalSize_panel.setLayout(new java.awt.BorderLayout());
        
        staticIntervalSize_label.setFont(new java.awt.Font("Dialog", 0, 12));
        staticIntervalSize_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        staticIntervalSize_label.setText("Interval Size");
        intervalSize_panel.add(staticIntervalSize_label, java.awt.BorderLayout.NORTH);
        
        zoomIn_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/zoom_in.png")));
        zoomIn_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomIn_buttonActionPerformed(evt);
            }
        });
        
        intervalSize_panel.add(zoomIn_button, java.awt.BorderLayout.WEST);
        
        zoomOut_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/zoom_out.png")));
        zoomOut_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomOut_buttonActionPerformed(evt);
            }
        });
        
        intervalSize_panel.add(zoomOut_button, java.awt.BorderLayout.EAST);
        
        dynIntervalSize_label.setFont(new java.awt.Font("Dialog", 0, 14));
        dynIntervalSize_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        dynIntervalSize_label.setText("5 min");
        intervalSize_panel.add(dynIntervalSize_label, java.awt.BorderLayout.CENTER);
        
        autoUpdate_panel.setLayout(new java.awt.BorderLayout());
        
        doc_panel.setLayout(new javax.swing.BoxLayout(doc_panel, javax.swing.BoxLayout.Y_AXIS));
        doc_button.setText("View documentation");
        doc_button.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
        doc_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewDocumentation();
            }
        });
        doc_panel.add(new javax.swing.JLabel(" "));
        doc_panel.add(doc_button);
        
        autoUpdate_checkBox.setSelected(true);
        autoUpdate_checkBox.setText("Auto-update statistics");
        autoUpdate_panel.add(autoUpdate_checkBox, java.awt.BorderLayout.WEST);
        
        autoUpdateBlank_label.setMaximumSize(new java.awt.Dimension(45, 15));
        autoUpdateBlank_label.setMinimumSize(new java.awt.Dimension(45, 15));
        autoUpdateBlank_label.setPreferredSize(new java.awt.Dimension(45, 15));
        autoUpdate_panel.add(autoUpdateBlank_label, java.awt.BorderLayout.NORTH);
        autoUpdate_panel.add(doc_panel, java.awt.BorderLayout.SOUTH);
        
        intervalSize_panel.add(autoUpdate_panel, java.awt.BorderLayout.SOUTH);
        
        timeOptions_panel.add(intervalSize_panel, java.awt.BorderLayout.NORTH);
        
        animation_panel.setLayout(new java.awt.BorderLayout(5, 10));
        
        animation_label.setFont(new java.awt.Font("Dialog", 0, 12));
        animation_label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        animation_label.setText("Animation");
        animation_panel.add(animation_label, java.awt.BorderLayout.NORTH);
        
        stepBackward_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Stepback24.gif")));
        stepBackward_button.setMinimumSize(new java.awt.Dimension(48, 34));
        stepBackward_button.setPreferredSize(new java.awt.Dimension(48, 34));
        stepBackward_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepBackward_buttonActionPerformed(evt);
            }
        });
        
        playPause_panel.add(stepBackward_button);
        
        play_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Play24.gif")));
        play_button.setMinimumSize(new java.awt.Dimension(48, 34));
        play_button.setPreferredSize(new java.awt.Dimension(48, 34));
        play_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                play_buttonActionPerformed(evt);
            }
        });
        
        playPause_panel.add(play_button);
        
        stepForward_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Stepforward24.gif")));
        stepForward_button.setMinimumSize(new java.awt.Dimension(48, 34));
        stepForward_button.setPreferredSize(new java.awt.Dimension(48, 34));
        stepForward_button.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepForward_buttonActionPerformed(evt);
            }
        });
        
        playPause_panel.add(stepForward_button);
        
        animation_panel.add(playPause_panel, java.awt.BorderLayout.CENTER);
        
        frames_panel.setLayout(new java.awt.BorderLayout(5, 0));
        
        frames_label.setFont(new java.awt.Font("Dialog", 0, 12));
        frames_label.setText("Frames / Sec:");
        frames_panel.add(frames_label, java.awt.BorderLayout.WEST);
        
        frames_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1", "2", "3", "4", "5" }));
        frames_comboBox.setSelectedIndex(1);
        frames_comboBox.setPreferredSize(new java.awt.Dimension(40, 23));
        frames_panel.add(frames_comboBox, java.awt.BorderLayout.CENTER);
        
        framesBlank_label.setText(" ");
        framesBlank_label.setPreferredSize(new java.awt.Dimension(60, 15));
        frames_panel.add(framesBlank_label, java.awt.BorderLayout.EAST);
        
        animation_panel.add(frames_panel, java.awt.BorderLayout.SOUTH);
        
        timeOptions_panel.add(animation_panel, java.awt.BorderLayout.SOUTH);
        
        maplist_panel.add(timeOptions_panel, java.awt.BorderLayout.CENTER);
        
        tabbedPane.addTab("Map", maplist_panel);
        
        statistic_panel.setLayout(new java.awt.BorderLayout());
        
        statistic_panel.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(5, 5, 5, 5)));
        description_panel.setLayout(new java.awt.BorderLayout());
        
        description_panel.setBorder(new javax.swing.border.TitledBorder("Statistic Descriptions"));
        description_panel.setPreferredSize(new java.awt.Dimension(161, 202));
        description_scrollPane.setHorizontalScrollBarPolicy(javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        statIntervals_panel.setLayout(new javax.swing.BoxLayout(statIntervals_panel, javax.swing.BoxLayout.Y_AXIS));
        
        description_scrollPane.setViewportView(statIntervals_panel);
        
        description_panel.add(description_scrollPane, java.awt.BorderLayout.CENTER);
        
        statistic_panel.add(description_panel, java.awt.BorderLayout.CENTER);
        
        assoc_panel.setLayout(new java.awt.BorderLayout());
        
        assoc_panel.setBorder(new javax.swing.border.TitledBorder("Associations"));
        statAssocLabel_panel.setLayout(new java.awt.GridLayout(4, 1));
        
        lineColour_label.setFont(new java.awt.Font("Dialog", 0, 12));
        lineColour_label.setText("Linecolour:");
        statAssocLabel_panel.add(lineColour_label);
        
        lineSize_label.setFont(new java.awt.Font("Dialog", 0, 12));
        lineSize_label.setText("Linesize:");
        statAssocLabel_panel.add(lineSize_label);
        
        pointColour_label.setFont(new java.awt.Font("Dialog", 0, 12));
        pointColour_label.setText("Pointcolour:");
        statAssocLabel_panel.add(pointColour_label);
        
        pointSize_label.setFont(new java.awt.Font("Dialog", 0, 12));
        pointSize_label.setText("Pointsize:");
        statAssocLabel_panel.add(pointSize_label);
        
        assoc_panel.add(statAssocLabel_panel, java.awt.BorderLayout.WEST);
        
        assocComboBox_panel.setLayout(new java.awt.GridLayout(4, 1, 0, 2));
        
        lineColour_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        lineColour_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineColour_comboBoxActionPerformed(evt);
            }
        });
        
        assocComboBox_panel.add(lineColour_comboBox);
        
        lineSize_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        lineSize_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineSize_comboBoxActionPerformed(evt);
            }
        });
        
        assocComboBox_panel.add(lineSize_comboBox);
        
        pointColour_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        pointColour_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pointColour_comboBoxActionPerformed(evt);
            }
        });
        
        assocComboBox_panel.add(pointColour_comboBox);
        
        pointSize_comboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "<none>" }));
        pointSize_comboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pointSize_comboBoxActionPerformed(evt);
            }
        });
        
        assocComboBox_panel.add(pointSize_comboBox);
        
        assoc_panel.add(assocComboBox_panel, java.awt.BorderLayout.CENTER);
        
        statistic_panel.add(assoc_panel, java.awt.BorderLayout.NORTH);
        
        tabbedPane.addTab("Statistics", statistic_panel);
        
        getContentPane().add(tabbedPane, java.awt.BorderLayout.EAST);
        
        pack();
    }//GEN-END:initComponents
    
    private void stepBackward_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stepBackward_buttonActionPerformed
        // Add your handling code here:
        stopAnimation();
        statAnimationFrame = 0;
        time_slider.setValue(statAnimationFrame);
    }//GEN-LAST:event_stepBackward_buttonActionPerformed
    
    private void pointSize_comboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pointSize_comboBoxActionPerformed
        // Add your handling code here:
        handleStatisticComboBox();
    }//GEN-LAST:event_pointSize_comboBoxActionPerformed
    
    private void pointColour_comboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pointColour_comboBoxActionPerformed
        // Add your handling code here:
        handleStatisticComboBox();
    }//GEN-LAST:event_pointColour_comboBoxActionPerformed
    
    private void lineSize_comboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lineSize_comboBoxActionPerformed
        // Add your handling code here:
        handleStatisticComboBox();
    }//GEN-LAST:event_lineSize_comboBoxActionPerformed
    
    private void lineColour_comboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lineColour_comboBoxActionPerformed
        // Add your handling code here:
        handleStatisticComboBox();
    }//GEN-LAST:event_lineColour_comboBoxActionPerformed
    
    private void stepForward_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stepForward_buttonActionPerformed
        // Add your handling code here:
        stopAnimation();
        date = getCurrentDate();
        autoUpdating = true;
        createTimeAxis();
        autoUpdating = false;
    }//GEN-LAST:event_stepForward_buttonActionPerformed
    
    private void play_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_play_buttonActionPerformed
        // Add your handling code here:
        if (animationPlaying == false) {
            Date nowDate = getCurrentDate();
            if (date.before(nowDate)) {
                startAnimation();
            }
        }
        else {
            stopAnimation();
        }
    }//GEN-LAST:event_play_buttonActionPerformed
    
    private void zoomOut_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomOut_buttonActionPerformed
        // Add your handling code here:
        if (timeStepSize == NemoUtils.FIVEMIN_STEP) {
            setStepSize(NemoUtils.HOUR_STEP);
        }
        else if (timeStepSize == NemoUtils.HOUR_STEP) {
            setStepSize(NemoUtils.DAY_STEP);
        }
    }//GEN-LAST:event_zoomOut_buttonActionPerformed
    
    private void zoomIn_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomIn_buttonActionPerformed
        // Add your handling code here:
        if (timeStepSize == NemoUtils.HOUR_STEP) {
            setStepSize(NemoUtils.FIVEMIN_STEP);
        }
        else if (timeStepSize == NemoUtils.DAY_STEP) {
            setStepSize(NemoUtils.HOUR_STEP);
        }
    }//GEN-LAST:event_zoomIn_buttonActionPerformed
    
    private void incYear_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_incYear_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_incYear_buttonActionPerformed
    
    private void decYear_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decYear_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_decYear_buttonActionPerformed
    
    private void incMonth_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_incMonth_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_incMonth_buttonActionPerformed
    
    private void decMonth_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decMonth_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_decMonth_buttonActionPerformed
    
    private void incDay_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_incDay_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_incDay_buttonActionPerformed
    
    private void decDay_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_decDay_buttonActionPerformed
        // Add your handling code here:
        handleToolbarClick(evt);
    }//GEN-LAST:event_decDay_buttonActionPerformed
    
    private void showOptions_buttonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showOptions_buttonActionPerformed
        // Add your handling code here:
        if (showOptions) {
            showOptions = false;
            tabbedPane.setVisible(false);
            showOptions_button.setText(">>");
            pack();
        }
        else {
            showOptions = true;
            tabbedPane.setVisible(true);
            showOptions_button.setText("<<");
            pack();
        }
    }//GEN-LAST:event_showOptions_buttonActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        dispose();
    }//GEN-LAST:event_exitForm
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton doc_button;
    private javax.swing.JPanel doc_panel;
    private javax.swing.JButton incMonth_button;
    private javax.swing.JLabel animation_label;
    private javax.swing.JLabel autoUpdateBlank_label;
    private javax.swing.JLabel lineSize_label;
    private javax.swing.JButton stepForward_button;
    private javax.swing.JButton incYear_button;
    private javax.swing.JPanel timeOptions_panel;
    private javax.swing.JLabel frames_label;
    private javax.swing.JComboBox pointSize_comboBox;
    private javax.swing.JPanel description_panel;
    private javax.swing.JButton incDay_button;
    private javax.swing.JPanel mapSelection_panel;
    private javax.swing.JPanel map_panel;
    private javax.swing.JButton zoomIn_button;
    private javax.swing.JPanel assoc_panel;
    private javax.swing.JScrollPane description_scrollPane;
    private javax.swing.JLabel day_label;
    private javax.swing.JButton decYear_button;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JPanel showNodesLines_panel;
    private javax.swing.JButton decDay_button;
    private javax.swing.JComboBox lineSize_comboBox;
    private javax.swing.JPanel maplist_panel;
    private javax.swing.JPanel selectChoice_panel;
    private javax.swing.JLabel month_label;
    private javax.swing.JPanel autoUpdate_panel;
    private javax.swing.JComboBox lineColour_comboBox;
    private javax.swing.JLabel network_label;
    private javax.swing.JPanel intervalSize_panel;
    private javax.swing.JComboBox map_comboBox;
    private javax.swing.JComboBox pointColour_comboBox;
    private javax.swing.JPanel statAssocLabel_panel;
    private javax.swing.JLabel year_label;
    private javax.swing.JLabel pointColour_label;
    private javax.swing.JPanel statistic_panel;
    private javax.swing.ButtonGroup buttonGroup;
    private javax.swing.JSlider time_slider;
    private javax.swing.JLabel lineColour_label;
    private javax.swing.JToolBar toolbar;
    private javax.swing.JLabel map_label;
    private javax.swing.JComboBox network_comboBox;
    private javax.swing.JPanel showChoice_panel;
    private javax.swing.JPanel statIntervals_panel;
    private javax.swing.JButton play_button;
    private javax.swing.JButton decMonth_button;
    private javax.swing.JPanel animation_panel;
    private javax.swing.JPanel showStaticLabel_panel;
    private javax.swing.JLabel staticIntervalSize_label;
    private javax.swing.JButton zoomOut_button;
    private javax.swing.JPanel assocComboBox_panel;
    private javax.swing.JLabel framesBlank_label;
    private javax.swing.JLabel blankSpace2_label;
    private javax.swing.JLabel nodes_label;
    private javax.swing.JPanel slider_panel;
    private javax.swing.JButton showOptions_button;
    private javax.swing.JButton stepBackward_button;
    private javax.swing.JComboBox frames_comboBox;
    private javax.swing.JLabel slider_label;
    private javax.swing.JLabel blankSpace3_label;
    private javax.swing.JLabel dynIntervalSize_label;
    private javax.swing.JPanel playPause_panel;
    private javax.swing.JComboBox lines_comboBox;
    private javax.swing.JLabel lines_label;
    private javax.swing.JComboBox nodes_comboBox;
    private javax.swing.JLabel pointSize_label;
    private javax.swing.JPanel frames_panel;
    private javax.swing.JCheckBox autoUpdate_checkBox;
    private javax.swing.JPanel selectStaticLabel_panel;
    private javax.swing.JLabel blankSpace1_label;
    private javax.swing.JPanel show_panel;
    // End of variables declaration//GEN-END:variables
    
    /**
     * Creates a new map and draws it on the screen. This method calls the
     * relevant methods for creating clickable point, line and map regions.
     * Then the redrawMap and retrieveStatistics methods are called.
     *
     * @param mapElement the MapElement object that is drawn
     */
    public void createMap(final MapElement mapElement) {
        Image baseMap = mapElement.getBackgroundImage();
        
        setTitle("NEMO - "+mapElement);
        
        makeMapRegions(mapElement);
        mapElement.createPointRegions();
        mapElement.createLineRegions();
        //makeFilterChoices(mapElement);
        
        mapElement.filterPoints("all");
        mapElement.filterLines("all");
        redrawMap();
        nm.retrieveStatistics(date, timeStepSize);
    }
    
    /**
     * Redraws the map. This method first calls getMapImage to draw new lines and points on
     * the base map image. Then the repaint method is called on the JLabel containing
     * the map image.
     */
    public void redrawMap() {
        NetworkElement networkElement = nm.getCurrentNetwork();
        if (networkElement == null) {
            mapImage = this.createImage(540, 704);
        }
        else {
            MapElement mapElement = networkElement.getCurrentMap();
            if (mapElement == null) {
                mapImage = this.createImage(540, 704);
            }
            else {
                mapImage = mapElement.getMapImage(date);
            }
        }
        
        mapImageComponent.repaint();
    }
    
    public void stop() {
        nm.stop();
    }
    
    
    /**
     * Fills up the filter comboboxes with the relevant choices for this map. All the
     * lines and points on the map are checked to determine how many different types
     * the user can choose from.
     */
    public void makeFilterChoices(MapElement mapElement) {
        Vector pointTypes = mapElement.getPointTypes();
        Vector lineTypes = mapElement.getLineTypes();
        
        // removes all node types
        int nodeCount = nodes_comboBox.getItemCount();
        for (int i = 1; i < nodeCount; i++) {
            nodes_comboBox.removeItemAt(i);
        }
        NemoUtils.debug("combonodes before:  "+nodes_comboBox.getItemCount());
        //nodes_comboBox.addItem("all");
        // add all point type choices to the combobox
        for (int i = 0; i < pointTypes.size(); i++) {
            NemoUtils.debug("pointType: "+pointTypes.elementAt(i));
            nodes_comboBox.addItem(pointTypes.elementAt(i));
        }
        NemoUtils.debug("combonodes after:  "+nodes_comboBox.getItemCount());
        
        
        // removes all line types
        int lineCount = lines_comboBox.getItemCount();
        for (int i = 1; i < lineCount; i++) {
            lines_comboBox.removeItemAt(i);
        }
        NemoUtils.debug("combolines before:  "+lines_comboBox.getItemCount());
        //lines_comboBox.addItem("all");
        // add all line type choices to the combobox
        for (int i = 0; i < lineTypes.size(); i++) {
            NemoUtils.debug("lineType: "+lineTypes.elementAt(i));
            lines_comboBox.addItem(lineTypes.elementAt(i));
        }
        NemoUtils.debug("combolines after:  "+lines_comboBox.getItemCount());
        
        lines_comboBox.setEnabled(true);
        nodes_comboBox.setEnabled(true);
    }
    
    /**
     * Filter the lines according to the type chosen in the linetypes combobox.
     */
    public void filterLines() {
        NetworkElement networkElement = nm.getCurrentNetwork();
        if (networkElement == null) return;
        MapElement mapElement = networkElement.getCurrentMap();
        if (mapElement == null) return;
        
        String filter = (String)(lines_comboBox.getSelectedItem());
        if (filter == null) return;
        mapElement.filterLines(filter);
        if (filter.compareTo("all") == 0) {
            nodes_comboBox.setEnabled(true);
        }
        else {
            nodes_comboBox.setEnabled(false);
        }
        redrawMap();
    }
    
    /*
     * Filter the points according to the type chosen in the pointtypes combobox.
     */
    public void filterPoints() {
        NetworkElement networkElement = nm.getCurrentNetwork();
        if (networkElement == null) return;
        MapElement mapElement = networkElement.getCurrentMap();
        if (mapElement == null) return;
        
        String filter = (String)(nodes_comboBox.getSelectedItem());
        if (filter == null) return;
        mapElement.filterPoints(filter);
        if (filter.compareTo("all") == 0) {
            lines_comboBox.setEnabled(true);
        }
        else {
            lines_comboBox.setEnabled(false);
        }
        redrawMap();
    }
    
    public void setFramesPerSecond() {
        String fpsString = (String)frames_comboBox.getSelectedItem();
        try {
            fps = Integer.parseInt(fpsString);
        } catch (NumberFormatException e) {
        }
        animationTimer.setDelay(1000/fps);
    }
    
    /**
     * Generates a list of clickable map regions.
     */
    private void makeMapRegions(MapElement currentMap) {
        DefaultListModel mapList = nm.getCurrentNetwork().getMapList();
        MapElement superRegion = null;
        
        // clear the old regions
        for (int i = 0; i < mapList.size(); i++) {
            MapElement mapElement = (MapElement)mapList.elementAt(i);
            mapElement.removeRegion();
        }
        
        // find the correct superRegion
        double smallestSize = (double)3000000*(double)3000000;
        for (int i = 0; i < mapList.size(); i++) {
            MapElement mapElement = (MapElement)mapList.elementAt(i);
            double mapArea = Math.abs((double)mapElement.getY2()-(double)mapElement.getY1()) * Math.abs((double)mapElement.getX2()-(double)mapElement.getX1());
            if (mapElement.contains(currentMap) && (mapArea < smallestSize) && (mapElement != currentMap)) {
                superRegion = mapElement;
                smallestSize = mapArea;
            }
        }
        if (superRegion != null) {
            currentMap.createSuperRegion(superRegion);
        }
        
        // find all subRegions
        for (int i = 0; i < mapList.size(); i++) {
            MapElement mapElement = (MapElement)mapList.elementAt(i);
            if (currentMap.contains(mapElement) && ((mapElement.getZoomLevel() == currentMap.getZoomLevel()) ||
                    (mapElement.getZoomLevel() == currentMap.getZoomLevel() + 1)) && (mapElement != currentMap)) {
                
                currentMap.createSubRegion(mapElement);
            }
            else if ((currentMap.overlaps(mapElement) || mapElement.overlaps(currentMap))
                    && (mapElement.getZoomLevel() == currentMap.getZoomLevel()) && (mapElement != currentMap) && (!mapElement.isSuperRegion())) {
                
                currentMap.createSubRegion(mapElement);
            }
        }
    }
    
    /**
     * Called when the mouse pointer moves over the map. Selects the region, line or
     * node the mouse is pointing at.
     */
    public void mouseMovedOverMap(MouseEvent event) {
        Selectable oldElement = activeElement;
        
        NetworkElement networkElement = nm.getCurrentNetwork();
        if (networkElement == null) return;
        MapElement mapElement = networkElement.getCurrentMap();
        if (mapElement == null) return;
        
        Point mousePoint = event.getPoint();
        
        if (mapElement.getNodeAtPoint(mousePoint) != null) {
            activeElement = mapElement.getNodeAtPoint(mousePoint);
        }
        else if (mapElement.getLineAtPoint(mousePoint) != null) {
            activeElement = mapElement.getLineAtPoint(mousePoint);
        }
        else if (networkElement.getMapAtPoint(mousePoint) != null) {
            activeElement = networkElement.getMapAtPoint(mousePoint);
        }
        else {
            activeElement = null;
        }
        
        if (activeElement != oldElement) {
            mapImageComponent.repaint();
        }
    }
    
    /**
     * Called when the mouse pointer moves outside the map area. Sets the currently
     * selected items to none.
     */
    public void mouseExitedMap(MouseEvent e) {
        activeElement = null;
        mapImageComponent.repaint();
    }
    
    /**
     * Called when the mouse is clicked while over the map. If a node or line is
     * selected, a popup menu is shown.
     */
    public void mouseClickedOnMap(MouseEvent e) {
        if (activeElement != null) {
            activeElement.performMouseClickAction(this, e);
        }
    }
    
    /**
     * Updates the list of map regions.
     *
     * @param listModel the DefaultListModel containing the new map regions
     */
    public void updateMapRegionList(DefaultListModel listModel) {
        while (map_comboBox.getItemCount() > 1) {
            map_comboBox.removeItemAt(1);
        }
        Object[] mapArray = listModel.toArray();
        Arrays.sort(mapArray);
        for (int i = 0; i < mapArray.length; i++) {
            map_comboBox.addItem(mapArray[i]);
        }
    }
    
    /**
     * Updates the list of networks.
     */
    public void updateNetworkList(DefaultListModel listModel) {
        for (int i = 0; i < listModel.size(); i++) {
            network_comboBox.addItem(listModel.elementAt(i));
        }
    }
    
    /**
     * Updates the list of statistics.
     */
    public void updateStatisticList(DefaultListModel listModel) {
        while (lineColour_comboBox.getItemCount() > 1) {
            lineColour_comboBox.removeItemAt(1);
        }
        while (lineSize_comboBox.getItemCount() > 1) {
            lineSize_comboBox.removeItemAt(1);
        }
        while (pointColour_comboBox.getItemCount() > 1) {
            pointColour_comboBox.removeItemAt(1);
        }
        while (pointSize_comboBox.getItemCount() > 1) {
            pointSize_comboBox.removeItemAt(1);
        }
        for (int i = 0; i < listModel.size(); i++) {
            StatisticElement statElement = (StatisticElement)listModel.elementAt(i);
            if (statElement.getType().compareTo("linecolour") == 0) {
                lineColour_comboBox.addItem(statElement);
            }
            else if (statElement.getType().compareTo("linesize") == 0) {
                lineSize_comboBox.addItem(statElement);
            }
            else if (statElement.getType().compareTo("pointcolour") == 0) {
                pointColour_comboBox.addItem(statElement);
            }
            else if (statElement.getType().compareTo("pointsize") == 0) {
                pointSize_comboBox.addItem(statElement);
            }
        }
        pack();
    }
    
    public void selectStatistics(DefaultListModel list) {
    	if (list==null) return;
        NetworkElement networkElement = nm.getCurrentNetwork();
        DefaultListModel currentList = networkElement.getStatisticList();
        NemoUtils.debug("statlist size: "+list.size());
        for (int i = 0; i < list.size(); i++) {
            StatisticElement cachedElement = (StatisticElement)list.elementAt(i);
            StatisticElement statElement = (StatisticElement)currentList.elementAt(currentList.indexOf(cachedElement));
            if (statElement == null) continue;
            NemoUtils.debug("choosing statistic: "+statElement);
            if (statElement.getType().compareTo("linecolour") == 0) {
                lineColour_comboBox.setSelectedItem(statElement);
            }
            if (statElement.getType().compareTo("linesize") == 0) {
                lineSize_comboBox.setSelectedItem(statElement);
            }
            if (statElement.getType().compareTo("pointcolour") == 0) {
                pointColour_comboBox.setSelectedItem(statElement);
            }
            if (statElement.getType().compareTo("pointsize") == 0) {
                pointSize_comboBox.setSelectedItem(statElement);
            }
        }
    }
    
    /**
     * Selects the newly chosen network.
     */
    public void handleChosenNetwork() {
        Object selectedItem = network_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") == 0) {
            //nm.setCurrentNetwork(null);
            redrawMap();
            return;
        }
        NetworkElement networkElement = (NetworkElement)selectedItem;
        NetworkElement currentNetwork = nm.getCurrentNetwork();
        if (currentNetwork != null) {
            if (networkElement.toString().compareTo(currentNetwork.toString()) == 0) {
                return;
            }
        }
        nm.setCurrentNetwork(networkElement);
    }
    
    /**
     * Selects the newly chosen map.
     */
    public void handleChosenMap() {
        Object selectedItem = map_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") == 0) {
            //nm.getCurrentNetwork().setCurrentMap(null);
            redrawMap();
            return;
        }
        
        MapElement mapElement = (MapElement)selectedItem;
        if (mapElement != null) {
            if (animationPlaying) {
                stopAnimation();
            }
            nm.retrieveMap(mapElement);
        }
    }
    
    /**
     * Selects the newly chosen statistics.
     */
    public void handleStatisticComboBox() {
        Object selectedItem = null;
        NetworkElement networkElement = nm.getCurrentNetwork();
        networkElement.removeAllStatistics();
        removeIntervalDescriptions();
        if (animationPlaying) {
            stopAnimation();
        }
        
        selectedItem = lineColour_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") != 0) {
            StatisticElement statElement = (StatisticElement)selectedItem;
            networkElement.addStatistic(statElement);
            updateIntervalDescriptions(statElement);
        }
        selectedItem = lineSize_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") != 0) {
            StatisticElement statElement = (StatisticElement)selectedItem;
            networkElement.addStatistic(statElement);
            updateIntervalDescriptions(statElement);
        }
        selectedItem = pointColour_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") != 0) {
            StatisticElement statElement = (StatisticElement)selectedItem;
            networkElement.addStatistic(statElement);
            updateIntervalDescriptions(statElement);
        }
        selectedItem = pointSize_comboBox.getSelectedItem();
        if (selectedItem.toString().compareTo("<none>") != 0) {
            StatisticElement statElement = (StatisticElement)selectedItem;
            networkElement.addStatistic(statElement);
            updateIntervalDescriptions(statElement);
        }
        nm.retrieveStatistics(date, timeStepSize);
        nm.checkAvailableStatistics(timeStepSize);
    }
    
    /**
     * Clears the statistic interval description panel.
     */
    public void removeIntervalDescriptions() {
        statIntervals_panel.removeAll();
        JLabel blankLabel1 = new JLabel(" ");
        blankLabel1.setAlignmentX(CENTER_ALIGNMENT);
        JLabel grayLabel = new JLabel("** gray colour / square shapes = no statistic **");
        grayLabel.setAlignmentX(CENTER_ALIGNMENT);
        JLabel blankLabel2 = new JLabel(" ");
        blankLabel2.setAlignmentX(CENTER_ALIGNMENT);
        statIntervals_panel.add(blankLabel1);
        statIntervals_panel.add(grayLabel);
        statIntervals_panel.add(blankLabel2);
        statIntervals_panel.repaint();
    }
    
    /**
     * Updates the statistic interval panel with a new statistic.
     */
    public void updateIntervalDescriptions(StatisticElement statElement) {
        JLabel statLabel = new JLabel(statElement.getDescription());
        statLabel.setAlignmentX(CENTER_ALIGNMENT);
        statIntervals_panel.add(statLabel);
        Vector descriptions = statElement.getIntervalDescriptions();
        for (int i = 0; i < descriptions.size(); i++) {
            StatisticElement.IntervalDescription desc = (StatisticElement.IntervalDescription)descriptions.elementAt(i);
            statIntervals_panel.add(desc);
        }
        JLabel spaceLabel = new JLabel(" ");
        spaceLabel.setAlignmentX(CENTER_ALIGNMENT);
        statIntervals_panel.add(spaceLabel);
        statIntervals_panel.repaint();
        description_scrollPane.revalidate();
    }
    
    /**
     * Selects a new map in the combobox.
     *
     * @param map the name of the map we want to show
     */
    public void selectMap(MapElement map) {
        if (animationPlaying) {
            stopAnimation();
        }
        if (map != null) {
            map_comboBox.setSelectedItem(map);
        }
        else {
            map_comboBox.setSelectedItem("<none>");
        }
    }
    
    /**
     * Selects a new statistic in the combobox.
     */
    public void selectStatistic(StatisticElement statElement) {
        if (statElement.getType().compareTo("linecolour") == 0) {
            lineColour_comboBox.setSelectedItem(statElement);
        }
        if (statElement.getType().compareTo("linesize") == 0) {
            lineSize_comboBox.setSelectedItem(statElement);
        }
        if (statElement.getType().compareTo("pointcolour") == 0) {
            pointColour_comboBox.setSelectedItem(statElement);
        }
        if (statElement.getType().compareTo("pointsize") == 0) {
            pointSize_comboBox.setSelectedItem(statElement);
        }
    }
    
    
    public void retrieveEquipmentStatistics(PointElement pointElement) {
        nm.retrieveEquipmentStatistics(date, timeStepSize, pointElement);
    }
    
    public void retrievePointURL(PointElement pointElement) {
        nm.retrievePointURL(pointElement);
    }
    
    public void retrieveLineURL(LineElement lineElement) {
    	nm.retrieveLineURL(lineElement);
    }
    
    public void retrieveLineValueStatistics(LineElement lineElement) {
        nm.retrieveLineValueStatistics(date, timeStepSize, lineElement);
    }
    
    /**
     * Selects a network in the network combobox.
     *
     * @param network the name of the network to be selected
     */
    public void selectNetwork(NetworkElement network) {
        network_comboBox.setSelectedItem(network);
    }
    
    public void viewDocumentation() {
        try {
            BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
            bs.showDocument(new URL("http://drift.uninett.no/kart/nemo/"));
        } catch(UnavailableServiceException ue) {
            System.err.println("Browser service not available.\n");
        } catch(MalformedURLException me) {
            System.err.println("Malformed url.\n");
        }
    }
    
    public void performStatAnimation() {
        time_slider.setValue(statAnimationFrame);
        statAnimationFrame++;
        
        Date nowDate = getCurrentDate();
        
        if (timeStepSize == NemoUtils.FIVEMIN_STEP || timeStepSize == NemoUtils.HOUR_STEP) {
            if (date.after(nowDate)) {
                stopAnimation();
            }
        }
        else if (timeStepSize == NemoUtils.DAY_STEP) {
            // find yesterday's date and stop the animation at that point
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.DAY_OF_MONTH, -1);
            cal.set(Calendar.HOUR, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            nowDate = cal.getTime();
            if (date.after(nowDate)) {
                stopAnimation();
            }
        }
    }
    
    public void stopAnimation() {
        animationPlaying = false;
        play_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Play24.gif")));
        animationTimer.stop();
        
        // wait a bit and retrieve stats, just to make sure we're up to sync
        try {
            Thread.sleep(250);
        } catch (InterruptedException e) {
        }
        nm.retrieveStatistics(date, timeStepSize);
    }
    
    public void startAnimation() {
        animationPlaying = true;
        play_button.setIcon(new javax.swing.ImageIcon(getClass().getResource("/Pause24.gif")));
        animationTimer.start();
    }
    
    public Date getCurrentDate() {
        Calendar currentCalendar = Calendar.getInstance();
        currentCalendar.set(Calendar.SECOND, 59);
        Date currentDate = currentCalendar.getTime();
        return currentDate;
    }
    
    public void setStepSize(int step) {
        timeStepSize = step;
        if (animationPlaying) {
            stopAnimation();
        }
        nm.checkAvailableStatistics(timeStepSize);
        if (timeStepSize == NemoUtils.FIVEMIN_STEP) {
            dynIntervalSize_label.setText("5 min");
            zoomIn_button.setEnabled(false);
            zoomOut_button.setEnabled(true);
            timeAxisSize = NemoUtils.DAY_AXIS;
        }
        else if (timeStepSize == NemoUtils.HOUR_STEP) {
            dynIntervalSize_label.setText("hour");
            zoomIn_button.setEnabled(true);
            zoomOut_button.setEnabled(true);
            timeAxisSize = NemoUtils.DAY_AXIS;
        }
        else if (timeStepSize == NemoUtils.DAY_STEP) {
            dynIntervalSize_label.setText("day");
            zoomIn_button.setEnabled(true);
            zoomOut_button.setEnabled(false);
            timeAxisSize = NemoUtils.YEAR_AXIS;
        }
        createTimeAxis();
    }
    
    public void showMissingStatistics(Vector missingStats, String stepString) {
        /*
         String statString = new String();
         for (int i = 0; i < missingStats.size(); i++) {
         StatisticElement statElement = (StatisticElement)missingStats.elementAt(i);
         statString = statString + statElement.toString() + "\n";
         }
         NemoUtils.showErrorMessage("The following statistics does not have data for "+stepString+" intervals:\n"+statString, this);
         */
    }
    
    public void createTimeAxis() {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        Date labelDate = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("HH");
        Hashtable sliderTable = new Hashtable();
        int minSliderValue = 0;
        int maxSliderValue = 0;
        int sliderValue = 0;
        
        if (timeAxisSize == NemoUtils.DAY_AXIS) {
            maxSliderValue = 24*60/timeStepSize - 1;
            formatter.applyPattern("HH");
            for (int i = 0; i < 24; i+=2) {  // a label for every other hour
                cal.set(Calendar.HOUR_OF_DAY, i);
                labelDate = cal.getTime();
                sliderTable.put(new Integer(i*60/timeStepSize), new JLabel(formatter.format(labelDate)));
            }
            cal.setTime(date);
            sliderValue = cal.get(Calendar.HOUR_OF_DAY) * 60 / timeStepSize + cal.get(Calendar.MINUTE) / timeStepSize;
        }
        else if (timeAxisSize == NemoUtils.MONTH_AXIS) {
            int days_in_month = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
            maxSliderValue = days_in_month*24*60/timeStepSize - 1;
            formatter.applyPattern("dd.");
            for (int i = 1; i <= days_in_month; i+=2) {
                cal.set(Calendar.DAY_OF_MONTH, i);
                labelDate = cal.getTime();
                sliderTable.put(new Integer((i-1)*24*60/timeStepSize), new JLabel(formatter.format(labelDate)));
            }
            cal.setTime(date);
            sliderValue = (cal.get(Calendar.DAY_OF_MONTH)-1) * 24 * 60 / timeStepSize +
            cal.get(Calendar.HOUR_OF_DAY) * 60 / timeStepSize + cal.get(Calendar.MINUTE) / timeStepSize;
        }
        else if (timeAxisSize == NemoUtils.YEAR_AXIS) {
            int days_in_year = cal.getActualMaximum(Calendar.DAY_OF_YEAR);
            maxSliderValue = days_in_year*24*60/timeStepSize - 1;
            cal.set(Calendar.DAY_OF_MONTH, 1);
            formatter.applyPattern("MMM");
            for (int i = 1; i <= 12; i++) {
                cal.set(Calendar.MONTH, i);
                labelDate = cal.getTime();
                sliderTable.put(new Integer(cal.get(Calendar.DAY_OF_YEAR)*24*60/timeStepSize), new JLabel(formatter.format(labelDate)));
            }
            cal.setTime(date);
            sliderValue = (cal.get(Calendar.DAY_OF_YEAR)-1) * 24 * 60 / timeStepSize +
            cal.get(Calendar.HOUR_OF_DAY) * 60 / timeStepSize + cal.get(Calendar.MINUTE) / timeStepSize;
        }
        blockDateUpdate = true;
        time_slider.setMinimum(minSliderValue);
        time_slider.setMaximum(maxSliderValue);
        time_slider.setLabelTable(sliderTable);
        blockDateUpdate = false;
        
        autoUpdating = true;
        time_slider.setValue(sliderValue);
        autoUpdating = false;
    }
    
    public void setAutoUpdate(boolean newState) {
        if (autoUpdate_checkBox.isSelected() && (newState == false)) {
            autoUpdate_checkBox.doClick();
        }
        else if (!autoUpdate_checkBox.isSelected() && (newState == true)) {
            autoUpdate_checkBox.doClick();
        }
    }
    
    public void handleToolbarClick(ActionEvent evt) {
        setAutoUpdate(false);
        Calendar cal = Calendar.getInstance();
        SimpleDateFormat formatter = new SimpleDateFormat("HH");
        cal.setTime(date);
        JButton button = (JButton)evt.getSource();
        if (button == decDay_button) {
            cal.add(Calendar.DAY_OF_YEAR, -1);
        }
        else if (button == incDay_button) {
            cal.add(Calendar.DAY_OF_YEAR, 1);
        }
        else if (button == decMonth_button) {
            cal.add(Calendar.MONTH, -1);
        }
        else if (button == incMonth_button) {
            cal.add(Calendar.MONTH, 1);
        }
        else if (button == decYear_button) {
            cal.add(Calendar.YEAR, -1);
        }
        else if (button == incYear_button) {
            cal.add(Calendar.YEAR, 1);
        }
        date = cal.getTime();
        updateTimeLabels();
        createTimeAxis();
        switch (timeStepSize) {
        case NemoUtils.FIVEMIN_STEP :
            formatter.applyPattern("HH:mm dd. MMM yyyy");
            break;
        case NemoUtils.HOUR_STEP :
            formatter.applyPattern("'Hour' HH - dd. MMM yyyy");
            break;
        case NemoUtils.DAY_STEP :
            formatter.applyPattern("dd. MMMM yyyy");
            break;
        }
        slider_label.setText(formatter.format(date));
        if (!animationPlaying) {
            nm.retrieveStatistics(date, timeStepSize);
        }
    }
    
    public void updateTimeLabels() {
        SimpleDateFormat formatter = new SimpleDateFormat("dd");
        day_label.setText(formatter.format(date));
        
        formatter.applyPattern("MMM");
        month_label.setText(formatter.format(date));
        
        formatter.applyPattern("yyyy");
        year_label.setText(formatter.format(date));
    }
    
    public Date getDate() {
        return date;
    }
    
    
    /**
     * Class that handles mouse events on the map. Calls the corresponding methods
     * in MapFrame to handle the events.
     */
    class MapMouseListener extends MouseInputAdapter {
        public void mouseClicked(MouseEvent e) {
            mouseClickedOnMap(e);
        }
        
        public void mouseExited(MouseEvent e) {
            mouseExitedMap(e);
        }
        
        public void mouseMoved(MouseEvent e) {
            mouseMovedOverMap(e);
        }
    }
    
    /**
     * Class that contains the map image. It extends the JLabel class and overrides
     * the paintComponent method in order to draw the graphics elements on the map
     * correctly.
     */
    class MapImageComponent extends JLabel {
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.drawImage(mapImage, 0, 0, this);
            if (activeElement != null) {
                activeElement.drawSelected(g);
            }
        }
    }
    
    class SliderListener implements ChangeListener {
        private Calendar cal;
        private SimpleDateFormat formatter;
        
        public SliderListener() {
            cal = Calendar.getInstance();
            formatter = new SimpleDateFormat("dd. MMMM yyyy");
        }
        
        public void stateChanged(ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
            if (source == time_slider && !blockDateUpdate) {
                cal.setTime(date);
                switch (timeStepSize) {
                // convert the value from the slider into a time value based on the size of the
                // time unit steps
                case NemoUtils.FIVEMIN_STEP :
                    cal.set(Calendar.HOUR_OF_DAY, ((int)(source.getValue())*timeStepSize/60) % 24);
                    cal.set(Calendar.MINUTE, ((int)(source.getValue())*timeStepSize) % 60);
                    formatter.applyPattern("HH:mm dd. MMM yyyy");
                    break;
                case NemoUtils.HOUR_STEP :
                    cal.set(Calendar.HOUR_OF_DAY, (int)(source.getValue()) % 24);
                    formatter.applyPattern("'Hour' HH - dd. MMM yyyy");
                    break;
                case NemoUtils.DAY_STEP :
                    formatter.applyPattern("dd. MMMM yyyy");
                    break;
                }
                if (timeAxisSize == NemoUtils.MONTH_AXIS) {
                    cal.set(Calendar.DAY_OF_MONTH, 1 + ((int)(source.getValue())*timeStepSize) / (24*60));
                }
                else if (timeAxisSize == NemoUtils.YEAR_AXIS) {
                    cal.set(Calendar.DAY_OF_YEAR, 1 + ((int)(source.getValue())*timeStepSize) / (24*60));
                }
                date = cal.getTime();
                slider_label.setText(formatter.format(date));
                updateTimeLabels();
                
                if (!source.getValueIsAdjusting()) {
                    if (!autoUpdating) {
                        setAutoUpdate(false);
                    }
                    if (!animationPlaying) {
                        nm.retrieveStatistics(date, timeStepSize);
                        statAnimationFrame = (int)source.getValue();
                    }
                    else {
                        nm.retrieveStatistics(date, timeStepSize);
                        if (source.getValue() == source.getMaximum()) {
                            stopAnimation();
                        }
                    }
                }
            }
        }
    }
    
    class StatisticTimerListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            date = getCurrentDate();
            autoUpdating = true;
            createTimeAxis();
            autoUpdating = false;
        }
    }
}
