/*
 * Netmonitor.java
 *
 * Created on June 6, 2002, 12:26 PM
 * NOTE: NetBeans (www.netbeans.org) is the software engineering tool used to create all classes.
 */

import java.net.*;
import javax.swing.*;
import javax.jnlp.*;
import java.awt.*;
import java.util.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import java.text.*;
import java.util.regex.*;
import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;

/**
 * Netmonitor is the main class and it is responsible for managing the SQL calls and
 * creating the data objects. It initializes a MapFrame object which acts as Nemo's
 * main GUI window. All classes who needs data from the database calls the respective
 * method in this class, which starts a new thread that performs the SQL query. When
 * the SQL thread finishes, the data is used to build objects that are passed on to other
 * classes.
 *
 * @author Gunnar Gauslaa Bergem
 * @version 2.0
 * @see MapFrame
 */
public class Netmonitor extends JFrame {
    private NetworkElement currentNetwork;     // the currently selected network
    private DefaultListModel networkList = new DefaultListModel();   // list of all the network elements
    private MapFrame mapFrame;                 // instance of the mapframe class
    private MapElement cachedMapElement;
    private DefaultListModel cachedStatistics;
    
    private boolean autoUpdateLoad = true;     // true if statistics should be automatically updated, false otherwise
    private boolean doneFetchingMaps = false;  // true if we're done fetching maps in the startup phase (command-line arg given)
    private boolean doneFetchingStats = false; // true if we're done fetching stats in the startup phase (command-line arg given)
    
    private Connection SQLconnection;              // connection to the sql database
    private AnimationFrame currentAnimationFrame;  // animation frame holding the current line and point statistics
    private AnimationQueue animationQueue;         // fifo buffer containing animation frames
    private FrameProducer frameProducer;           // puts frames into the buffer
    private FrameConsumer frameConsumer;           // gets frames from the buffer and displays them
    
    // constants defining types of sql queries
    private static final int MAPS = 1;
    private static final int POINTS = 2;
    private static final int LINES = 3;
    private static final int POINT_STATS = 4;
    private static final int LINE_STATS = 5;
    private static final int NETWORKS = 6;
    private static final int STATISTICS = 7;
    private static final int POINTURL = 8;
    private static final int LINEURL = 9;
    private static final int EQ_STATS = 10;
    private static final int LINEVALUE_STATS = 11;
    
    /** Creates new form Netmonitor */
    public Netmonitor() {
        
        // use the aqua skin from www.l2fprod.com
        try {
            ClassLoader cl = this.getClass().getClassLoader();
            Skin skin = SkinLookAndFeel.loadThemePack(cl.getResource("aquathemepack.zip"));
            SkinLookAndFeel.setSkin(skin);
            UIManager.setLookAndFeel("com.l2fprod.gui.plaf.skin.SkinLookAndFeel");
            //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            NemoUtils.showErrorMessage("Unable to set look and feel: " + e.getMessage(), this);
        }
        
        // set up a new animationqueue with room for one frame in the buffer
        animationQueue = new AnimationQueue(1);
        frameProducer = new FrameProducer(animationQueue);
        frameConsumer = new FrameConsumer(animationQueue);
        
        // create and display the main window
        mapFrame = new MapFrame(this);
        mapFrame.setLocation(50, 50);
        mapFrame.setVisible(true);
        
        // load the postgresql database driver
        try {
            Class.forName("org.postgresql.Driver");
        } catch (ClassNotFoundException e) {
            NemoUtils.showErrorMessage("SQL driver not found: " + e.getMessage(), this);
        }
    }
    
    /**
     * Takes all the command-line parameters as arguments and uses them to select a network, map or statistic.
     *
     * @param networkString name of the network to be selected at startup
     * @param mapString name of the map to be selected at startup
     * @param lineColourString name of the statistic used to represent the linecolour at startup
     * @param lineSizeString name of the statistic used to represent the linesize at startup
     * @param pointColourString name of the statistic used to represent the pointcolour at startup
     * @param pointSizeString name of the statistic used to represent the pointsize at startup
     *
     */
    public void init(String dbname, String username, String password, String networkString, String mapString,
            String lineColourString, String lineSizeString, String pointColourString, String pointSizeString) {
        String hostname = new String();
        String databaseURL = new String();
        
        try {
            BasicService bs = (BasicService)ServiceManager.lookup("javax.jnlp.BasicService");
            hostname = bs.getCodeBase().getHost();
            NemoUtils.debug("Im located on "+hostname+" and doing good.");
        } catch (UnavailableServiceException ue) {
        }
        if (dbname.compareTo("") == 0) {
            databaseURL = "jdbc:postgresql://pot.uninett.no/nemo2";
        }
        else {
            databaseURL = "jdbc:postgresql://"+dbname;
        }
        NemoUtils.debug("databaseURL: "+databaseURL);
        
        connectSQLdatabase(databaseURL, username, password);
        
        // perform the network query and update list of networks in the mapframe
        SQLquery networkQuery = new SQLquery(NETWORKS, "SELECT net, description FROM net");
        networkQuery.start();
        while (networkQuery.isAlive()) {
            try {
                // wait for the query to finish
                Thread.sleep(200);
            } catch (InterruptedException e) {
                System.err.println(e.getMessage());
            }
        }
        mapFrame.updateNetworkList(networkList);
        
        if (networkString.compareTo("") != 0) {
            // select the network given from the command-line in the mapframe list
            for (int i = 0; i < networkList.size(); i++) {
                NetworkElement networkElement = (NetworkElement)networkList.elementAt(i);
                if (networkElement.toString().compareTo(networkString) == 0) {
                    mapFrame.selectNetwork(networkElement);
                    break;
                }
            }
            while (!doneFetchingMaps || !doneFetchingStats) {
                try {
                    // wait for the map and statistic queries to finish
                    Thread.sleep(500);
                }
                catch (InterruptedException e) {
                    System.err.println(e.getMessage());
                }
            }
            // select the statistics from the command-line in the mapframe stats list
            if (lineColourString.compareTo("") != 0) {
                for (int i = 0; i < currentNetwork.getStatisticList().size(); i++) {
                    StatisticElement statElement = (StatisticElement)currentNetwork.getStatisticList().elementAt(i);
                    if (statElement.toString().compareTo(lineColourString) == 0) {
                        mapFrame.selectStatistic(statElement);
                        break;
                    }
                }
            }
            if (lineSizeString.compareTo("") != 0) {
                for (int i = 0; i < currentNetwork.getStatisticList().size(); i++) {
                    StatisticElement statElement = (StatisticElement)currentNetwork.getStatisticList().elementAt(i);
                    if (statElement.toString().compareTo(lineSizeString) == 0) {
                        mapFrame.selectStatistic(statElement);
                        break;
                    }
                }
            }
            if (pointColourString.compareTo("") != 0) {
                for (int i = 0; i < currentNetwork.getStatisticList().size(); i++) {
                    StatisticElement statElement = (StatisticElement)currentNetwork.getStatisticList().elementAt(i);
                    if (statElement.toString().compareTo(pointColourString) == 0) {
                        mapFrame.selectStatistic(statElement);
                        break;
                    }
                }
            }
            if (pointSizeString.compareTo("") != 0) {
                for (int i = 0; i < currentNetwork.getStatisticList().size(); i++) {
                    StatisticElement statElement = (StatisticElement)currentNetwork.getStatisticList().elementAt(i);
                    if (statElement.toString().compareTo(pointSizeString) == 0) {
                        mapFrame.selectStatistic(statElement);
                        break;
                    }
                }
            }
            // select the map given from the command-line
            if (mapString.compareTo("") != 0) {
                for (int i = 0; i < currentNetwork.getMapList().size(); i++) {
                    MapElement mapElement = (MapElement)currentNetwork.getMapList().elementAt(i);
                    if (mapElement.toString().compareTo(mapString) == 0) {
                        mapFrame.selectMap(mapElement);
                        break;
                    }
                }
            }
        }
    }
    
    /**
     * Execution of Nemo starts here. A new Netmonitor object is initialized and the command-line
     * arguments are read in and passed on to the init() method.
     */
    public static void main(String[] args) {
        String dbnameString = "drift.uninett.no/nemo";  // default database string
        String usernameString = "nemoread";             // default username
        String passwordString = "reader";               // default password
        String networkString = new String();
        String mapString = new String();
        String lineColourString = new String();
        String lineSizeString = new String();
        String pointColourString = new String();
        String pointSizeString = new String();
        
        for (int i = 0; i < args.length; i++) {
            if ((args[i].compareTo("-dbname") == 0) && (args.length > i)) {
                dbnameString = args[i+1];
            }
            if ((args[i].compareTo("-username") == 0) && (args.length > i)) {
                usernameString = args[i+1];
            }
            if ((args[i].compareTo("-password") == 0) && (args.length > i)) {
                passwordString = args[i+1];
            }
            if ((args[i].compareTo("-net") == 0) && (args.length > i)) {
                networkString = args[i+1];
            }
            if ((args[i].compareTo("-map") == 0) && (args.length > i)) {
                mapString = args[i+1];
            }
            if ((args[i].compareTo("-linecolour") == 0) && (args.length > i)) {
                lineColourString = args[i+1];
            }
            if ((args[i].compareTo("-linesize") == 0) && (args.length > i)) {
                lineSizeString = args[i+1];
            }
            if ((args[i].compareTo("-pointcolour") == 0) && (args.length > i)) {
                pointColourString = args[i+1];
            }
            if ((args[i].compareTo("-pointsize") == 0) && (args.length > i)) {
                pointSizeString = args[i+1];
            }
            if ((args[i].compareTo("-debug") == 0) && (args.length > i)) {
                NemoUtils.setDebugging(true);
            }
        }
        Netmonitor nm = new Netmonitor();
        nm.init(dbnameString, usernameString, passwordString, networkString, mapString, lineColourString, lineSizeString, pointColourString, pointSizeString);
    }
    
    /**
     * Connects to the SQL database given in the databaseURL argument.
     *
     * @param databaseURL the location of the database.
     * @param username the username to log into the database with.
     * @param password the password to log into the database with.
     */
    public void connectSQLdatabase(String databaseURL, String username, String password) {
        try {
            SQLconnection = DriverManager.getConnection(databaseURL, username, password);
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("Unable to open connection to database: " + e.getMessage(), this);
        }
    }
    
    /**
     * Sets the current network to the networkelement given in the argument. The maplist and the
     * statisticlist of the network are then retrieved.
     *
     * @param network a NetworkElement that will be used as the current network.
     */
    public void setCurrentNetwork(NetworkElement network) {
        if (currentNetwork != null) {
            cachedMapElement = currentNetwork.getCurrentMap();
            cachedStatistics = currentNetwork.getCurrentStatistics();
        }
        currentNetwork = network;
        retrieveMapList();
        retrieveStatisticList();
    }
    
    /**
     * Returns the current network.
     *
     * @return the NetworkElement that represents the current network.
     */
    public NetworkElement getCurrentNetwork() {
        return currentNetwork;
    }
    
    /**
     * Adds a statistic to the list of chosen statistics in the current network.
     *
     * @param statElement a StatisticElement to be added to the list of chosen statistics.
     */
    public void addStatistic(StatisticElement statElement) {
        currentNetwork.addStatistic(statElement);
    }
    
    /**
     * Creates a new SQLquery thread that asks for all the maps that belongs to the current network.
     */
    public void retrieveMapList() {
        SQLquery mapListQuery = new SQLquery(MAPS, "SELECT map_id, coord_type, bitmap_width, bitmap_height, x1, y1, x2, y2, " +
       "zoomlevel, mapurl, bitmap_factor, external FROM map WHERE net = '"+currentNetwork+"'");
        mapListQuery.start();
    }
    
    /**
     * Creates a new SQLquery thread that asks for all the statistics that belongs to the current network.
     */
    public void retrieveStatisticList() {
        SQLquery statisticListQuery = new SQLquery(STATISTICS, "SELECT statistic.stat_name, stat_type, statistic_table, statistic_column1, " +
                "statistic_column2, equipmentvalue_table, equipmentvalue_column, unit, description, timespan, interval_start, " +
                "interval_end, colour, size, symbol_url, label FROM statistic, conversion WHERE statistic.stat_name = conversion.stat_name AND " +
                "net = '"+currentNetwork+"' ORDER BY interval_start ASC");
        statisticListQuery.start();
    }
    
    /**
     * Creates SQLqueries that asks for the image of the map to be retrieved, and all the lines and points belonging to the map.
     * The points are retrieved first, and we wait until they are in place before we retrieve the lines, since the lines depends
     * on having the points in place in order to connect to them properly. A ThreadCoordinator object makes sure that the
     * map is drawn after both the lines and the mapimage are retrieved.
     *
     * @param mapElement the map to be retrieved.
     */
    public void retrieveMap(MapElement mapElement) {
        /*
         if (mapElement.isLoaded()) {
         currentNetwork.setCurrentMap(mapElement);
         mapFrame.createMap(mapElement);
         return;
         }
         */
        currentNetwork.setCurrentMap(mapElement);
        ThreadCoordinator threadCoordinator = new ThreadCoordinator(mapElement);
        ImageIcon imageIcon = new ImageIcon(mapElement.getImageURL());
        
        SQLquery pointQuery = new SQLquery(POINTS, "SELECT location_name, from_date, to_date, coord_type, map_point.x, map_point.y, point_type FROM " +
                "location, map_point WHERE map_point.location_id = location.location_id AND map_id = '"+mapElement+"' AND map_point.net = " +
                "'"+currentNetwork+"'");
        
        // we need all the points in place before we can retrieve the lines between them, so lets wait for this query
        // to finish
        pointQuery.start();
        while (pointQuery.isAlive()) {
            try {
                // wait for the query to finish
                Thread.sleep(50);
            } catch (InterruptedException e) {
                System.err.println(e.getMessage());
            }
        }
        
        SQLquery lineQuery = new SQLquery(LINES, "SELECT line_name, from_date, to_date, line_type, (SELECT location_name FROM " +
                "location WHERE location_id = from_loc) AS from_name, (SELECT location_name FROM location WHERE location_id = to_loc) " +
                "AS to_name FROM line, map_line WHERE map_line.line_id = line.line_id AND map_id = '"+mapElement+"' AND map_line.net = " +
                "'"+currentNetwork+"'");
        
        threadCoordinator.addImageIcon(imageIcon);
        threadCoordinator.addThread(lineQuery);
        lineQuery.start();
        threadCoordinator.start();
    }
    
    /**
     * Tells the FrameProducer and FrameConsumer objects that a new frame is requested. A frame is composed lists of statistic objects
     * that contains the colour and size of the lines and points on the map. The frame is eventually fetched, and its contents
     * are drawn on the map.
     *
     * @param date the date the statistic is from.
     * @param stepSize the size of the steps in time between each recorded measure point. E.g. 5 minutes, 1 hour, 1 day.
     */
    public void retrieveStatistics(Date date, int stepSize) {
        if (currentNetwork == null) return;
        MapElement mapElement = currentNetwork.getCurrentMap();
        if (mapElement == null) return;
        
        // For day statistics we need the SQL query to ask near the end of the day to be sure that
        // the statistic's timestamp is caught.
         /*
         if (stepSize == NemoUtils.DAY_STEP) {
         Calendar calendar = Calendar.getInstance();
         calendar.setTime(date);
         calendar.set(Calendar.HOUR_OF_DAY, 23);
         date = calendar.getTime();
         }
         */
        
        frameProducer.setStatisticDate(date, stepSize);
        
        // start the FrameProducer and FrameConsumer threads if they're not started already
        if (!frameProducer.isAlive() || !frameConsumer.isAlive()) {
            frameProducer.start();
            frameConsumer.start();
        }
        frameProducer.setFrameRequested(true);
        //frameConsumer.setFrameRequested(true);
    }
    
    
    public String parsePointStatQuery(StatisticElement.PointSQLTable table, Date date, MapElement mapElement, StatisticElement statElement) {
        String pointTable = table.point_table;
        String pointColumn = table.point_column;
        int stepSize = table.stepSize;
        String statName = statElement.toString();
        long sec_time = date.getTime() / 1000;
        int time_window = stepSize * 10 * 60;
        String stepSizeString = new String();
        
        if (stepSize == NemoUtils.FIVEMIN_STEP) {
            stepSizeString = "5min";
        } else if (stepSize == NemoUtils.HOUR_STEP) {
            stepSizeString = "hour";
            time_window = 60*60;
        } else if (stepSize == NemoUtils.DAY_STEP) {
            stepSizeString = "day";
            // find the beginning and end timestamps for the day we wish to query
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.HOUR, 23);
            date = cal.getTime();
            sec_time = date.getTime() / 1000;
            time_window = 24*60*60;
        }
        
        String queryString =
            "SELECT "+
            "   DISTINCT ON ("+pointTable+".point_id) "+
            "   timestamp, "+
            "   location_name, "+
            "   statistic.stat_name, "+
            "   statistic.stat_type, "+
            "   colour, "+
            "   size "+
            "FROM "+
            "   "+pointTable+", location, statistic, map_point, conversion "+
            "WHERE "+
            "   "+pointTable+"."+pointColumn+" BETWEEN interval_start AND interval_end "+
            "   AND conversion.stat_name = '"+statName+"'"+
            "   AND statistic.stat_name = '"+statName+"'"+
            "   AND "+pointTable+".point_id = location.location_id "+
            "   AND map_point.location_id = location.location_id "+
            "   AND "+pointTable+".timestamp BETWEEN "+(sec_time-time_window)+" AND "+sec_time+
            "   AND map_point.net = '"+currentNetwork+"'"+
            "   AND map_point.map_id = '"+mapElement+"'"+
            "   AND statistic.timespan = '"+stepSizeString+"' "+
            "ORDER BY "+
            "   "+pointTable+".point_id ASC, "+
            "   "+pointTable+".timestamp DESC";
        
        //System.out.println(queryString);
        return queryString;
    }
    
    public String parseLineStatQuery(StatisticElement.LineSQLTable table, Date date, MapElement mapElement, StatisticElement statElement) {
        String lineTable = table.table;
        String lineColumn1 = table.column1;
        String lineColumn2 = table.column2;
        int stepSize = table.stepSize;
        String statName = statElement.toString();
        long sec_time = date.getTime() / 1000;
        int time_window = stepSize * 5 * 60;
        String stepSizeString = new String();
        
        if (stepSize == NemoUtils.FIVEMIN_STEP) {
            stepSizeString = "5min";
        } else if (stepSize == NemoUtils.HOUR_STEP) {
            stepSizeString = "hour";
            time_window = 60*60;
        } else if (stepSize == NemoUtils.DAY_STEP) {
            stepSizeString = "day";
            // find the beginning and end timestamps for the day we wish to query
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.HOUR, 23);
            date = cal.getTime();
            sec_time = date.getTime() / 1000;
            time_window = 24*60*60;
        }
        
        String queryString =
            "SELECT "+
            "   DISTINCT ON ("+lineTable+".line_id) "+
            "   timestamp, "+
            "   line_name, "+
            "   statistic.stat_name, "+
            "   statistic.stat_type, "+
            "   c_in.colour AS in_colour, "+
            "   c_out.colour AS out_colour, "+
            "   c_in.size AS in_size, "+
            "   c_out.size AS out_size "+
            "FROM "+
            "   "+lineTable+", line, statistic, map_line, conversion c_in, conversion c_out "+
            "WHERE "+
            "   "+lineTable+"."+lineColumn1+" BETWEEN c_out.interval_start AND c_out.interval_end "+
            "   AND c_out.stat_name = '"+statName+"'"+
            "   AND "+lineTable+"."+lineColumn2+" BETWEEN c_in.interval_start AND c_in.interval_end "+
            "   AND c_in.stat_name = '"+statName+"'"+
            "   AND "+lineTable+".line_id = line.line_id "+
            "   AND map_line.line_id = line.line_id "+
            "   AND statistic.stat_name = '"+statName+"'"+
            "   AND "+lineTable+".timestamp BETWEEN "+(sec_time-time_window)+" AND "+sec_time+
            "   AND map_line.net = '"+currentNetwork+"'"+
            "   AND map_line.map_id = '"+mapElement+"'"+
            "   AND statistic.timespan = '"+stepSizeString+"' "+
            "ORDER BY "+
            "   "+lineTable+".line_id ASC, "+
            "   "+lineTable+".timestamp DESC";
        
        //System.out.println(queryString);
        return queryString;
    }
    
    public void retrieveEquipmentStatistics(Date date, int stepSize, PointElement pointElement) {
        if (currentNetwork == null) return;
        
        DefaultListModel statList = currentNetwork.getCurrentStatistics();
        for (int i = 0; i < statList.size(); i++) {
            StatisticElement statElement = (StatisticElement)statList.elementAt(i);
            if (statElement.isPointStatistic()) {
                StatisticElement.PointSQLTable table = statElement.getPointSQLTable(stepSize);
                if (table != null) {
                    String equipmentQueryString = parseEquipmentStatQuery(table, date, pointElement, statElement);
                    SQLquery equipmentStatQuery = new SQLquery(EQ_STATS, equipmentQueryString);
                    equipmentStatQuery.start();
                }
            }
        }
    }
    
    public String parseEquipmentStatQuery(StatisticElement.PointSQLTable table, Date date, PointElement pointElement, StatisticElement statElement) {
        String eqTable = table.eq_table;
        String eqColumn = table.eq_column;
        int stepSize = table.stepSize;
        String statName = statElement.toString();
        String pointName = pointElement.toString();
        long sec_time = date.getTime() / 1000;
        int time_window = 60*60;
        
        if (stepSize == NemoUtils.DAY_STEP) {
            // find the beginning and end timestamps for the day we wish to query
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.HOUR, 23);
            date = cal.getTime();
            time_window = 24*60*60;
        }
        String queryString = "SELECT DISTINCT ON ("+eqTable+".eq_id) location_name, stat_name, stat_url, timestamp, "+
        "unit, equipment.eq_id, "+eqColumn+" AS value "+
        "FROM "+eqTable+", equipment, location, statistic WHERE equipment.location_id = location.location_id AND "+
        "stat_name = '"+statName+"' AND "+eqTable+".eq_id = equipment.eq_id AND location.location_name = '"+pointName+"' AND "+
        eqTable+".timestamp BETWEEN "+(sec_time-time_window)+" AND "+sec_time+" ORDER BY "+eqTable+".eq_id ASC, "+
        eqTable+".timestamp DESC";
        return queryString;
    }
    
    public void retrievePointURL(PointElement pointElement) {
        if (currentNetwork == null) return;
        
        String queryString = "SELECT link, url_info, location_name FROM location_url, location WHERE location_url.location_id = "+
        "location.location_id AND location.location_name = '"+pointElement+"' AND location_url.net = '"+currentNetwork+"'";
        SQLquery pointURLQuery = new SQLquery(POINTURL, queryString);
        pointURLQuery.start();
        while (pointURLQuery.isAlive()) {
            try {
                // wait for the query to finish
                Thread.sleep(200);
            } catch (InterruptedException e) {
                System.err.println(e.getMessage());
            }
        }
    }
    
    public void retrieveLineURL(LineElement lineElement) {
        if (currentNetwork == null) return;
        
        String queryString = "SELECT link, url_info, line_name FROM line_url, line WHERE line_url.line_id = "+
        "line.line_id AND line.line_name = '"+lineElement+"' AND line_url.net = '"+currentNetwork+"'";
        SQLquery lineURLQuery = new SQLquery(LINEURL, queryString);
        lineURLQuery.start();
        while (lineURLQuery.isAlive()) {
            try {
                // wait for the query to finish
                Thread.sleep(200);
            } catch (InterruptedException e) {
                System.err.println(e.getMessage());
            }
        }
    }
    
    public void retrieveLineValueStatistics(Date date, int stepSize, LineElement lineElement) {
        if (currentNetwork == null) return;
        
        DefaultListModel statList = currentNetwork.getCurrentStatistics();
        for (int i = 0; i < statList.size(); i++) {
            StatisticElement statElement = (StatisticElement)statList.elementAt(i);
            if (statElement.isLineStatistic()) {
                StatisticElement.LineSQLTable table = statElement.getLineSQLTable(stepSize);
                if (table != null) {
                    String lineValueQueryString = parseLineValueStatQuery(table, date, lineElement, statElement);
                    SQLquery lineValueStatQuery = new SQLquery(LINEVALUE_STATS, lineValueQueryString);
                    lineValueStatQuery.start();
                }
            }
        }
    }
    
    public String parseLineValueStatQuery(StatisticElement.LineSQLTable table, Date date, LineElement lineElement, StatisticElement statElement) {
        String lineTable = table.table;
        String lineColumn1 = table.column1;
        String lineColumn2 = table.column2;
        int stepSize = table.stepSize;
        String statName = statElement.toString();
        long sec_time = date.getTime() / 1000;
        int time_window = stepSize * 5 * 60;
        
        String queryString = "SELECT DISTINCT ON ("+lineTable+".line_id) line_name, stat_name, stat_url, timestamp, "+
        "unit, "+lineColumn1+" AS in_value, "+lineColumn2+" AS out_value "+
        "FROM "+lineTable+", line, statistic WHERE stat_name = '"+statName+"' AND "+lineTable+".line_id = line.line_id "+
        "AND line.line_name = '"+lineElement.toString()+"' AND "+
        lineTable+".timestamp BETWEEN "+(sec_time-time_window)+" AND "+sec_time+" ORDER BY "+lineTable+".line_id ASC, "+
        lineTable+".timestamp DESC";
        return queryString;
    }
    
    public void checkAvailableStatistics(int stepSize) {
        if (currentNetwork == null) return;
        
        DefaultListModel statList = currentNetwork.getCurrentStatistics();
        Vector missingStatistics = new Vector();
        String stepString = new String();
        
        if (stepSize == NemoUtils.FIVEMIN_STEP) {
            stepString = "5 minutes";
        }
        else if (stepSize == NemoUtils.HOUR_STEP) {
            stepString = "hour";
        }
        else if (stepSize == NemoUtils.DAY_STEP) {
            stepString = "day";
        }
        
        for (int i = 0; i < statList.size(); i++) {
            StatisticElement statElement = (StatisticElement)statList.elementAt(i);
            Vector stepSizes = statElement.getStepSizes();
            if (!stepSizes.contains(stepString)) {
                missingStatistics.addElement(statElement);
            }
        }
        if (missingStatistics.size() > 0) {
            mapFrame.showMissingStatistics(missingStatistics, stepString);
        }
    }
    
    public void handleSQLResult(int queryid, ResultSet rs) {
        switch (queryid) {
        case MAPS:
            createMapElements(rs);
            break;
        case POINTS:
            createPointElements(rs);
            break;
        case LINES:
            createLineElements(rs);
            break;
        case STATISTICS:
            createStatisticElements(rs);
            break;
        case POINT_STATS:
            createPointStatistics(rs);
            break;
        case LINE_STATS:
            createLineStatistics(rs);
            break;
        case EQ_STATS:
            createEquipmentStatistics(rs);
            break;
        case LINEVALUE_STATS:
            createLineValueStatistics(rs);
            break;
        case NETWORKS:
            createNetworkElements(rs);
            break;
        case POINTURL:
            createPointURL(rs);
            break;
        case LINEURL:
            createLineURL(rs);
            break;
        }
    }
    
    public void createMapElements(ResultSet rs) {
        final DefaultListModel mapList = new DefaultListModel();
        try {
            while (rs.next()) {
                String mapName = rs.getString("map_id");
                String coordType = rs.getString("coord_type");
                int width = rs.getInt("bitmap_width");
                int height = rs.getInt("bitmap_height");
                int x1 = rs.getInt("x1");
                int y1 = rs.getInt("y1");
                int x2 = rs.getInt("x2");
                int y2 = rs.getInt("y2");
                int zoomlevel = rs.getInt("zoomlevel");
                URL mapURL = new URL(rs.getString("mapurl"));
                float bitmapFactor = rs.getFloat("bitmap_factor");
                boolean external = rs.getBoolean("external");
                MapElement mapElement = new MapElement(mapName, coordType, width, height, x1, y1, x2, y2, zoomlevel, mapURL, bitmapFactor,external);
                mapList.addElement(mapElement);
            }
        } catch (Exception e) {
            NemoUtils.showErrorMessage("Exception caught: " + e.getMessage(), this);
        }
        
        currentNetwork.setMapList(mapList);
        NemoUtils.debug("cached mapelement: "+cachedMapElement);
        if (mapFrame != null) {
            Runnable runGUI = new Runnable() {
                public void run() {
                    try {
                        mapFrame.updateMapRegionList(mapList);
                        if (cachedMapElement!=null)
                        	mapFrame.selectMap((MapElement)mapList.elementAt(mapList.indexOf(cachedMapElement)));
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            };
            SwingUtilities.invokeLater(runGUI);
        }
        doneFetchingMaps = true;
    }
    
    public void createPointElements(ResultSet rs) {
        Vector pointList = new Vector();
        MapElement currentMap = currentNetwork.getCurrentMap();
        try {
            while (rs.next()) {
                String pointType = rs.getString("point_type");
                String pointName = rs.getString("location_name");
                long from = rs.getInt("from_date");
                long to = rs.getInt("to_date");
                if (to == -1) { to = 1577836800; }  // We treat the year 2020 as infinity
                String coordType = rs.getString("coord_type");
                int x = rs.getInt("x");
                int y = rs.getInt("y");
                
                Date from_date = new Date(from*1000);
                Date to_date = new Date(to*1000);
                PointElement pointElement = new PointElement(currentMap, pointName, pointType, coordType, x, y, from_date, to_date);
                pointList.addElement(pointElement);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        
        currentMap.setPointList(pointList);
    }
    
    public void createLineElements(ResultSet rs) {
        Vector lineList = new Vector();
        MapElement currentMap = currentNetwork.getCurrentMap();
        try {
            while (rs.next()) {
                String lineName = rs.getString("line_name");
                String lineType = rs.getString("line_type");
                String fromPoint = rs.getString("from_name");
                String toPoint = rs.getString("to_name");
                long from = rs.getInt("from_date");
                long to = rs.getInt("to_date");
                if (to == -1) { to = 1577836800; }  // We treat the year 2020 as infinity
                
                Date from_date = new Date(from*1000);
                Date to_date = new Date(to*1000);
                PointElement p1 = currentMap.getPointElement(fromPoint);
                PointElement p2 = currentMap.getPointElement(toPoint);
                if (p1 != null && p2 != null) {
                    LineElement lineElement = new LineElement(currentMap, p1, p2, lineName, lineType, from_date, to_date);
                    lineList.addElement(lineElement);
                }
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        
        currentMap.setLineList(lineList);
    }
    
    public void createStatisticElements(ResultSet rs) {
        final DefaultListModel statisticList = new DefaultListModel();
        StatisticElement statElement = null;
        int stepSize = 0;
        ImageIcon imageIcon = null;
        
        try {
            while (rs.next()) {
                String statisticName = rs.getString("stat_name");
                String statisticType = rs.getString("stat_type");
                String description = rs.getString("description");
                String unit = rs.getString("unit");
                
                for (int i = 0; i < statisticList.size(); i++) {
                    statElement = (StatisticElement)statisticList.elementAt(i);
                    if (statElement.toString().compareTo(statisticName) == 0) {
                        break;
                    }
                    else {
                        statElement = null;
                    }
                }
                
                if (statElement == null) {
                    statElement = new StatisticElement(statisticName, statisticType, description, unit);
                    statisticList.addElement(statElement);
                }
                
                String statTable = rs.getString("statistic_table");
                String col1 = rs.getString("statistic_column1");
                String col2 = rs.getString("statistic_column2");
                String eqTable = rs.getString("equipmentvalue_table");
                String eqColumn = rs.getString("equipmentvalue_column");
                String timespan = rs.getString("timespan");
                int interval_start = rs.getInt("interval_start");
                int interval_end = rs.getInt("interval_end");
                String colourString = rs.getString("colour");
                int size = rs.getInt("size");
                String symbolString = rs.getString("symbol_url");
                String label = rs.getString("label");
                Color colour = null;
                
                if (timespan.compareTo("5min") == 0) {
                    stepSize = NemoUtils.FIVEMIN_STEP;
                }
                else if (timespan.compareTo("hour") == 0) {
                    stepSize = NemoUtils.HOUR_STEP;
                }
                else if (timespan.compareTo("day") == 0) {
                    stepSize = NemoUtils.DAY_STEP;
                }
                
                if (statElement.isPointStatistic()) {
                    statElement.addPointSQLTable(statTable, col1, eqTable, eqColumn, stepSize);
                }
                if (statElement.isLineStatistic()) {
                    statElement.addLineSQLTable(statTable, col1, col2, stepSize);
                }
                
                if (symbolString != null && symbolString.length() > 0) {
                    try {
                        URL symbolURL = new URL(symbolString);
                        imageIcon = new ImageIcon(symbolURL);
                    } catch (MalformedURLException e) {
                        NemoUtils.showErrorMessage("MalformedURLException caught: " + e.getMessage(), this);
                    }
                }
                
                if (colourString != null && colourString.length() > 0) {
                    StringTokenizer st = new StringTokenizer(colourString, " ");
                    try {
                        int red = Integer.parseInt(st.nextToken());
                        int green = Integer.parseInt(st.nextToken());
                        int blue = Integer.parseInt(st.nextToken());
                        colour = new Color(red, green, blue);
                    } catch (NumberFormatException e) {
                        NemoUtils.showErrorMessage("NumberFormatException caught: " + e.getMessage(), this);
                    }
                }
                
                statElement.addIntervalDescription(interval_start, interval_end, colour, size, imageIcon, label);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        currentNetwork.setStatisticList(statisticList);
        if (mapFrame != null) {
            Runnable runGUI = new Runnable() {
                public void run() {
                    try {
                        mapFrame.updateStatisticList(statisticList);
                        mapFrame.selectStatistics(cachedStatistics);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            };
            SwingUtilities.invokeLater(runGUI);
        }
        doneFetchingStats = true;
    }
    
    public void createPointStatistics(ResultSet rs) {
        PointStatistic pointStat = null;
        Color colour = null;
        Vector pointStatList = new Vector();
        try {
            while (rs.next()) {
                String location = rs.getString("location_name");
                String statName = rs.getString("stat_name");
                String statType = rs.getString("stat_type");
                String colourString = rs.getString("colour");
                int timestamp = rs.getInt("timestamp");
                int size = rs.getInt("size");
                if (statType.endsWith("size")) {
                    pointStat = new PointStatistic(statType, statName, location, timestamp, size);
                    pointStatList.addElement(pointStat);
                }
                else if(statType.endsWith("colour")) {
                    if (colourString != null) {
                        StringTokenizer st = new StringTokenizer(colourString, " ");
                        try {
                            int red = Integer.parseInt(st.nextToken());
                            int green = Integer.parseInt(st.nextToken());
                            int blue = Integer.parseInt(st.nextToken());
                            colour = new Color(red, green, blue);
                            pointStat = new PointStatistic(statType, statName, location, timestamp, colour);
                            pointStatList.addElement(pointStat);
                        } catch (NumberFormatException e) {
                            NemoUtils.showErrorMessage("NumberFormatException caught: " + e.getMessage(), this);
                        }
                    }
                }
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        currentAnimationFrame.addPointStatisticList(pointStatList);
    }
    
    public void createLineStatistics(ResultSet rs) {
        LineStatistic lineStat = null;
        Color inColour = null;
        Color outColour = null;
        Vector lineStatList = new Vector();
        try {
            while (rs.next()) {
                String lineName = rs.getString("line_name");
                String statName = rs.getString("stat_name");
                String statType = rs.getString("stat_type");
                String inColourString = rs.getString("in_colour");
                String outColourString = rs.getString("out_colour");
                int timestamp = rs.getInt("timestamp");
                int inSize = rs.getInt("in_size");
                int outSize = rs.getInt("out_size");
                if (statType.endsWith("size")) {
                    lineStat = new LineStatistic(statType, statName, lineName, timestamp, inSize, outSize);
                    lineStatList.addElement(lineStat);
                }
                else if(statType.endsWith("colour")) {
                    int inRed, inGreen, inBlue, outRed, outGreen, outBlue;
                    
                    StringTokenizer st = new StringTokenizer(inColourString, " ");
                    try {
                        inRed = Integer.parseInt(st.nextToken());
                        inGreen = Integer.parseInt(st.nextToken());
                        inBlue = Integer.parseInt(st.nextToken());
                        inColour = new Color(inRed, inGreen, inBlue);
                    } catch (NumberFormatException e) {
                        NemoUtils.showErrorMessage("NumberFormatException caught: " + e.getMessage(), this);
                    }
                    st = new StringTokenizer(outColourString, " ");
                    try {
                        outRed = Integer.parseInt(st.nextToken());
                        outGreen = Integer.parseInt(st.nextToken());
                        outBlue = Integer.parseInt(st.nextToken());
                        outColour = new Color(outRed, outGreen, outBlue);
                    } catch (NumberFormatException e) {
                        NemoUtils.showErrorMessage("NumberFormatException caught: " + e.getMessage(), this);
                    }
                    lineStat = new LineStatistic(statType, statName, lineName, timestamp, inColour, outColour);
                    lineStatList.addElement(lineStat);
                }
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        currentAnimationFrame.addLineStatisticList(lineStatList);
    }
    
    public void createEquipmentStatistics(ResultSet rs) {
        String statName = new String();
        Hashtable hash = new Hashtable();
        String pointName = new String();
        String eqName = new String();
        String URLtemplate = new String();
        int timestamp = 0;
        MapElement mapElement = currentNetwork.getCurrentMap();
        PointElement pointElement = null;
        
        try {
            while (rs.next()) {
                pointName = rs.getString("location_name");
                statName = rs.getString("stat_name");
                String unit = rs.getString("unit");
                eqName = rs.getString("eq_id");
                URLtemplate = rs.getString("stat_url");
                int value = rs.getInt("value");
                timestamp = rs.getInt("timestamp");
                hash.put(eqName, value+" "+unit);
                pointElement = mapElement.getPointElement(pointName);
                //pointElement.addStatisticURL(eqName, URLtemplate, timestamp, statName);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        if (pointElement != null) {
            /*
             Runnable runGUI = new Runnable() {
             public void run() {
             try {
             pointElement.addStatisticColumn(hash, statName.toString());
             } catch (Exception ex) {
             ex.printStackTrace();
             }
             }
             };
             SwingUtilities.invokeLater(runGUI);
             */
            pointElement.addStatisticColumn(hash, statName);
        }
    }
    
    public void createLineValueStatistics(ResultSet rs) {
        String lineName = new String();
        String statName = new String();
        String URLtemplate = new String();
        int timestamp = 0;
        Hashtable hash = new Hashtable();
        MapElement mapElement = currentNetwork.getCurrentMap();
        LineElement lineElement = null;
        
        try {
            while (rs.next()) {
                lineName = rs.getString("line_name");
                statName = rs.getString("stat_name");
                String unit = rs.getString("unit");
                URLtemplate = rs.getString("stat_url");
                int inValue = rs.getInt("in_value");
                int outValue = rs.getInt("out_value");
                timestamp = rs.getInt("timestamp");
                StringTokenizer st = new StringTokenizer(lineName, "-");
                String source = st.nextToken();
                String destination = st.nextToken();
                hash.put(destination+" -> "+source, inValue+" "+unit);
                hash.put(source+" -> "+destination, outValue+" "+unit);
                lineElement = mapElement.getLineElement(lineName);
                //lineElement.addStatisticURL(lineName, URLtemplate, timestamp, statName);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        if (lineElement != null) {
            /*
             Runnable runGUI = new Runnable() {
             public void run() {
             try {
             lineElement.addStatisticColumn(hash, statName);
             } catch (Exception ex) {
             ex.printStackTrace();
             }
             }
             };
             SwingUtilities.invokeLater(runGUI);
             */
            lineElement.addStatisticColumn(hash, statName);
        }
    }
    
    public void createNetworkElements(ResultSet rs) {
        try {
            while (rs.next()) {
                String networkName = rs.getString("net");
                String description = rs.getString("description");
                NetworkElement networkElement = new NetworkElement(networkName, description);
                networkList.addElement(networkElement);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
    }
    
    public void createPointURL(ResultSet rs) {
        String pointName = new String();
        String pointURL = new String();
        String URLInfo = new String();
        PointElement pointElement = null;
        Date statisticDate = null;
        MapElement mapElement = currentNetwork.getCurrentMap();
        
        try {
            while (rs.next()) {
                pointName = rs.getString("location_name");
                pointURL = rs.getString("link");
                URLInfo = rs.getString("url_info");
                pointElement = mapElement.getPointElement(pointName);
                statisticDate = mapFrame.getDate();
                parseURLdates(pointElement,pointURL, URLInfo, statisticDate);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        /*
         Runnable runGUI = new Runnable() {
         public void run() {
         try {
         pointElement.addURLlist(linkContent);
         } catch (Exception ex) {
         ex.printStackTrace();
         }
         }
         };
         SwingUtilities.invokeLater(runGUI);
         */
    }
    
    public void createLineURL(ResultSet rs) {
        String lineName = new String();
        String lineURL = new String();
        String URLInfo = new String();
        LineElement lineElement = null;
        Date statisticDate = null;
        MapElement mapElement = currentNetwork.getCurrentMap();
        
        try {
            while (rs.next()) {
                lineName = rs.getString("line_name");
                lineURL = rs.getString("link");
                URLInfo = rs.getString("url_info");
                lineElement = mapElement.getLineElement(lineName);
                statisticDate = mapFrame.getDate();
                parseURLdates(lineElement,lineURL,URLInfo,statisticDate);
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), this);
        }
        /*
         Runnable runGUI = new Runnable() {
         public void run() {
         try {
         lineElement.addURLlist(linkContent);
         } catch (Exception ex) {
         ex.printStackTrace();
         }
         }
         };
         SwingUtilities.invokeLater(runGUI);
         */
        //lineElement.addURLlist(linkContent);
    }
    
    public void parseURLdates(Selectable element,String urlString, String urlInfo, Date date) {
        Pattern pattern = null;
        Matcher matcher = null;
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        String linkContent = new String();
        
        pattern = Pattern.compile("#YYYY-#MM-#DD");
        matcher = pattern.matcher(urlString);
        if (matcher.find()) {
            urlString = matcher.replaceAll(formatter.format(date));
        }
        
        pattern = Pattern.compile("#YYYY-#MM-#PDD");
        matcher = pattern.matcher(urlString);
        if (matcher.find()) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.add(Calendar.DAY_OF_MONTH, -1);
            Date prevday_date = cal.getTime();
            formatter.applyPattern("yyyy-MM-dd");
            urlString = matcher.replaceAll(formatter.format(prevday_date));
        }
        
        pattern = Pattern.compile("#YY#MM#PDD");
        matcher = pattern.matcher(urlString);
        if (matcher.find()) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.add(Calendar.DAY_OF_MONTH, -1);
            Date prevday_date = cal.getTime();
            formatter.applyPattern("yyMMdd");
            urlString = matcher.replaceAll(formatter.format(prevday_date));
        }
        
        pattern = Pattern.compile("#YYYY--#PWW");
        matcher = pattern.matcher(urlString);
        if (matcher.find()) {
            Calendar cal = Calendar.getInstance();
            cal.setTime(date);
            cal.add(Calendar.WEEK_OF_YEAR, -1);
            Date prevweek_date = cal.getTime();
            formatter.applyPattern("yyyy--ww");
            urlString = matcher.replaceAll(formatter.format(prevweek_date));
        }
        element.addURL(urlInfo,urlString);
    }
    
    public void setMaxBufferSize(int size) {
        animationQueue.setMaxBufferSize(size);
    }
    
    /**
     * Disconnects from sql database.
     */
    public void stop() {
        try {
            if (SQLconnection != null) {
                SQLconnection.close();
            }
        } catch (SQLException e) {
            NemoUtils.showErrorMessage("SQLException caught: "+e.getMessage(), this);
        }
    }
    
    public boolean getAutoUpdateLoad() {
        return autoUpdateLoad;
    }
    
    /**
     * Class holding lists of point and line statistics.
     */
    class AnimationFrame {
        private Vector lineStatisticLists;
        private Vector pointStatisticLists;
        
        public AnimationFrame() {
            lineStatisticLists = new Vector();
            pointStatisticLists = new Vector();
        }
        
        public void addLineStatisticList(Vector list) {
            lineStatisticLists.addElement(list);
        }
        
        public void addPointStatisticList(Vector list) {
            pointStatisticLists.addElement(list);
        }
        
        public Vector getLineStatisticLists() {
            return lineStatisticLists;
        }
        
        public Vector getPointStatisticLists() {
            return pointStatisticLists;
        }
    }
    
    /**
     * Fifo circular buffer that holds animation frames.
     */
    class AnimationQueue {
        private AnimationFrame[] animationFrames;
        private int maxBufferSize;
        private int framesInBuffer;
        private int head;
        private int tail;
        
        public AnimationQueue(int maxSize) {
            maxBufferSize = maxSize;
            animationFrames = new AnimationFrame[maxBufferSize];
            framesInBuffer = 0;
            head = 0;
            tail = 0;
        }
        
        public synchronized AnimationFrame getFrame() {
            while (framesInBuffer == 0) {
                try {
                    wait();
                } catch (InterruptedException e) {
                }
            }
            AnimationFrame frame = animationFrames[head];
            if (maxBufferSize > 1) {
                head = (head + 1) % maxBufferSize;
            }
            framesInBuffer--;
            if (framesInBuffer == (maxBufferSize - 1)) {
                notifyAll();
            }
            return frame;
        }
        
        public synchronized void putFrame(AnimationFrame frame) {
            while (framesInBuffer >= maxBufferSize) {
                try {
                    wait();
                } catch (InterruptedException e) {
                }
            }
            animationFrames[tail] = frame;
            if (maxBufferSize > 1) {
                tail = (tail + 1) % maxBufferSize;
            }
            framesInBuffer++;
            if (framesInBuffer == maxBufferSize) {
                notifyAll();
            }
        }
        
        public synchronized void setMaxBufferSize(int size) {
            maxBufferSize = size;
            animationFrames = new AnimationFrame[maxBufferSize];
            head = 0;
            tail = 0;
            framesInBuffer = 0;
            notifyAll();
        }
    }
    
    
    class FrameProducer extends Thread {
        private boolean frameRequested;
        private boolean animating;
        private boolean allThreadsFinished;
        private Date statisticDate;
        private int stepSize;
        private Vector threadList;
        private AnimationQueue animationQueue;
        
        public FrameProducer(AnimationQueue queue) {
            frameRequested = false;
            animating = false;
            allThreadsFinished = false;
            animationQueue = queue;
            threadList = new Vector();
        }
        
        public void run() {
            while (true) {
                while (frameRequested || animating) {
                    allThreadsFinished = false;
                    currentAnimationFrame = new AnimationFrame();
                    threadList.removeAllElements();
                    MapElement mapElement = currentNetwork.getCurrentMap();
                    DefaultListModel statList = currentNetwork.getCurrentStatistics();
                    
                    for (int i = 0; i < statList.size(); i++) {
                        StatisticElement statElement = (StatisticElement)statList.elementAt(i);
                        if (statElement.isPointStatistic()) {
                            StatisticElement.PointSQLTable table = statElement.getPointSQLTable(stepSize);
                            if (table != null) {
                                String pointQueryString = parsePointStatQuery(table, statisticDate, mapElement, statElement);
                                SQLquery pointStatQuery = new SQLquery(POINT_STATS, pointQueryString);
                                threadList.addElement(pointStatQuery);
                                pointStatQuery.start();
                            }
                        }
                        else if (statElement.isLineStatistic()) {
                            StatisticElement.LineSQLTable table = statElement.getLineSQLTable(stepSize);
                            if (table != null) {
                                String lineQueryString = parseLineStatQuery(table, statisticDate, mapElement, statElement);
                                SQLquery lineStatQuery = new SQLquery(LINE_STATS, lineQueryString);
                                threadList.addElement(lineStatQuery);
                                lineStatQuery.start();
                            }
                        }
                    }
                    
                    while (!allThreadsFinished) {
                        allThreadsFinished = true;
                        for (int i = 0; i < threadList.size(); i++) {
                            Thread thread = (Thread)threadList.elementAt(i);
                            if (thread != null) {
                                if (thread.isAlive()) {
                                    allThreadsFinished = false;
                                }
                            }
                        }
                        try {
                            Thread.sleep(50);
                        } catch (InterruptedException e) {
                        }
                    }
                    
                    animationQueue.putFrame(currentAnimationFrame);
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(statisticDate);
                    calendar.add(Calendar.MINUTE, stepSize);
                    statisticDate = calendar.getTime();
                    frameRequested = false;
                }
                
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }
        }
        
        public void setStatisticDate(Date date, int step) {
            statisticDate = date;
            stepSize = step;
        }
        
        public void setAnimating(boolean state) {
            animating = state;
        }
        
        public void setFrameRequested(boolean state) {
            frameRequested = state;
        }
    }
    
    class FrameConsumer extends Thread {
        private AnimationQueue animationQueue;
        
        
        public FrameConsumer(AnimationQueue queue) {
            animationQueue = queue;
        }
        
        public void run() {
            while (true) {
                AnimationFrame frame = animationQueue.getFrame();
                MapElement mapElement = currentNetwork.getCurrentMap();
                mapElement.resetPointStatistics();
                mapElement.resetLineStatistics();
                if (mapFrame != null) {
                    Vector pointLists = frame.getPointStatisticLists();
                    Vector lineLists = frame.getLineStatisticLists();
                    for (int i = 0; i < pointLists.size(); i++) {
                        Vector pointList = (Vector)pointLists.elementAt(i);
                        mapElement.assignPointStatistics(pointList);
                    }
                    for (int i = 0; i < lineLists.size(); i++) {
                        Vector lineList = (Vector)lineLists.elementAt(i);
                        mapElement.assignLineStatistics(lineList);
                    }
                    NemoUtils.debug("FrameConsumer calling redrawMap");
                    mapFrame.redrawMap();
                }
            }
        }
    }
    
    class SQLquery extends Thread {
        private int queryid;
        private String queryString;
        
        public SQLquery(int queryid, String query) {
            this.queryid = queryid;
            this.queryString = query;
        }
        
        public void run() {
            try {
                if (SQLconnection != null) {
                    Date beforeQueryDate = new Date();
                    long beforeQuery = beforeQueryDate.getTime();
                    NemoUtils.debug("Executing query: "+queryid);
                    Statement stmt = SQLconnection.createStatement();
                    ResultSet rs = stmt.executeQuery(queryString);
                    Date afterQueryDate = new Date();
                    long afterQuery = afterQueryDate.getTime();
                    NemoUtils.debug("Query "+queryid+" done in "+(afterQuery-beforeQuery)+" ms ("+rs.getFetchSize()+" rows)");
                    handleSQLResult(queryid, rs);
                    stmt.close();
                }
            } catch (SQLException e) {
                NemoUtils.showErrorMessage("SQLException caught: " + e.getMessage(), Netmonitor.this);
            }
        }
    }
    
    class ThreadCoordinator extends Thread {
        private boolean allFinished = false;
        private MapElement mapElement;
        private ImageIcon imageIcon;
        private Vector threadList;
        
        public ThreadCoordinator(MapElement mapElement) {
            this.mapElement = mapElement;
            threadList = new Vector();
        }
        
        public ThreadCoordinator() {
            threadList = new Vector();
        }
        
        public void run() {
            while (!allFinished) {
                allFinished = true;
                if (imageIcon != null) {
                    if (imageIcon.getImageLoadStatus() != MediaTracker.COMPLETE) {
                        allFinished = false;
                    }
                    else {
                        mapElement.setBackgroundImage(imageIcon.getImage());
                    }
                }
                for (int i = 0; i < threadList.size(); i++) {
                    Thread thread = (Thread)threadList.elementAt(i);
                    if (thread != null) {
                        if (thread.isAlive()) {
                            allFinished = false;
                        }
                    }
                }
                try {
                    Thread.sleep(50);
                } catch (InterruptedException e) {
                }
            }
            if (mapElement != null) {
                mapFrame.createMap(mapElement);
            }
            else {
                mapFrame.redrawMap();
            }
        }
        
        public void addImageIcon(ImageIcon imageIcon) {
            this.imageIcon = imageIcon;
        }
        
        public void addThread(Thread thread) {
            threadList.addElement(thread);
        }
    }
}
