/*
 * PointElement.java
 *
 * Created on July 4, 2003, 9:46 AM
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.text.*;

/**
 * This class contains the both the bitmap and UTM coordinates of the locations on
 * the map and various other information. It has methods for drawing the point on
 * the map and creating the bitmap coordinates.
 *
 * @author Gunnar Gauslaa Bergem
 */
public class PointElement implements Selectable {
    private MapElement parentMap;      // map that contains this point
    private String pointName;          // name of the point
    private String pointType;          // type of the point
    private String coordType;          // type of coordinates, like UTM-32
    private int coordX;                // x coordinate
    private int coordY;                // y coordinate
    private int bitmapX;               // bitmap x coordinate
    private int bitmapY;               // bitmap y coordinate
    private Date fromDate;             // when the point started to exist
    private Date toDate;               // when the point ceased to exist
    private int size;                  // diameter of the point
    private Color colour;              // colour of the point
    private Rectangle rectangleOutline;  // rectangular outline of the point, used for hit testing
    private int[] textCoords;            // pair of coordinates describing the text placement location
    private Vector dateStrings;          // dates for the point statistics
    private Vector measureStrings;       // names for the point statistics
    private StatisticDialog dialog;      // dialog that shows stats and urls for the point
    private static Color defaultColour = new Color(100, 100, 100);   // grey default colour
    private static int defaultSize = 8;  // default diameter
    private boolean sizeIsSet = false;   // true if the point has received a size statistic, false otherwise (no stat is drawn as a square)
    private boolean enabled;             // true if the point should be drawn, false otherwise
    private PopupMenu popup;
    private ArrayList URLvalues, URLnames;
    
    /** Creates a new instance of PointElement */
    public PointElement(MapElement parent, String name, String pointType, String coordType, int x, int y, Date from, Date to) {
        parentMap = parent;
        pointName = name;
        this.pointType = pointType;
        this.coordType = coordType;
        coordX = x;
        coordY = y;
        fromDate = from;
        toDate = to;
        colour = defaultColour;
        size = defaultSize;
        dateStrings = new Vector();
        measureStrings = new Vector();
        rectangleOutline = new Rectangle();
        textCoords = new int[2];
        URLvalues=new ArrayList();
        URLnames=new ArrayList();
        
        // check if the coordType has a format like UTM-32
        if (coordType.startsWith("UTM")) {
            StringTokenizer st = new StringTokenizer(coordType, "-");
            st.nextToken(); // skip the "UTM" substring
            try {
                int pointUTMZone = Integer.parseInt(st.nextToken());
                int mapUTMZone = parentMap.getUTMZone();
                if (pointUTMZone != mapUTMZone) {
                    // convert the point to the same UTM zone as the map it belongs to
                    dPair UTM = UTM_Converter.UTMtoUTM(coordY, coordX, pointUTMZone, mapUTMZone);
                    coordX = (int)Math.round(UTM.Y);
                    coordY = (int)Math.round(UTM.X);
                }
            } catch (NumberFormatException e) {
            }
        }
        createBitmapCoords();
    }
    
    /**
     * Create a set of bitmap coordinates.
     */
    private void createBitmapCoords() {
        bitmapX = (int)Math.round((coordX - (parentMap.getX1())) / parentMap.factorX());
        bitmapY = (int)Math.round(((parentMap.getY2() - parentMap.getY1()) - (coordY - parentMap.getY1())) / parentMap.factorY());
        bitmapX += MapElement.BORDER_SIZE;
        bitmapY += MapElement.BORDER_SIZE;
    }
    
    public int getBitmapX() {
        return bitmapX;
    }
    
    public int getBitmapY() {
        return bitmapY;
    }
    
    /**
     * Returns the distance to the supplied pointelement.
     */
    public int distance(PointElement otherPoint) {
        int x1 = bitmapX;
        int x2 = otherPoint.getBitmapX();
        int y1 = bitmapY;
        int y2 = otherPoint.getBitmapY();
        
        int distance = (int)(Math.sqrt(Math.pow((x2-x1), 2) + Math.pow((y2-y1), 2)));
        
        return distance;
    }
    
    public void updateBitmapX(int x) {
        bitmapX = x;
    }
    
    public void updateBitmapY(int y) {
        bitmapY = y;
    }
    
    public String getType() {
        return pointType;
    }
    
    public void setSize(int size) {
        this.size = size;
        sizeIsSet = true;
    }
    
    public void setColour(Color colour) {
        this.colour = colour;
    }
    
    /**
     * Set all the statistics to default values.
     */
    public void resetStatistics() {
        size = defaultSize;
        colour = defaultColour;
        sizeIsSet = false;
        dateStrings.removeAllElements();
        measureStrings.removeAllElements();
    }
    
    /**
     * Check if the rectangle outline of this point contains the supplied point.
     */
    public boolean contains(Point point) {
        if (!enabled) {
            return false;
        }
        return rectangleOutline.contains(point);
    }
    
    /**
     * Enable this point if it exists within the given point, otherwise disable it.
     */
    public void checkEnabled(Date date) {
        if (fromDate.after(date) || toDate.before(date)) {
            enabled = false;
            NemoUtils.debug(pointName+" disabled, does not exist at the given date.");
        }
        else {
            enabled = true;
        }
    }
    
    public boolean isEnabled() {
        return enabled;
    }
    
    public void createRectangleOutline() {
        rectangleOutline = new Rectangle(bitmapX - size, bitmapY - size, size * 2, size * 2);
    }
    
    /**
     * Creates strings that displays the name of the statistic and the time at which
     * it was measured and adds them the list of strings.
     */
    public void addTimestamp(Date statisticDate, String statName) {
        SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss  dd. MMM yyyy");
        String dateString = formatter.format(statisticDate);
        String measureString = "Measure point for "+statName+":";
        dateStrings.addElement(dateString);
        measureStrings.addElement(measureString);
    }
    
    /**
     * Draw the selected version of the point.
     */
    public void drawSelected(Graphics _g) {
        Graphics2D g = (Graphics2D)_g;
        Color selectedColor = new Color(0, 0, 205);
        int maxTextDistance = 10;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        g.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        
        g.setColor(selectedColor);
        if (pointType.compareTo("sky") == 0) {
            int skySize = size * 2/3;
            g.drawArc(bitmapX - skySize, bitmapY - skySize*2, skySize*2, skySize*2, 0, 180);
            g.drawArc(bitmapX - skySize*2, bitmapY - skySize, skySize*2, skySize*2, 90, 180);
            g.drawArc(bitmapX - skySize, bitmapY, skySize*2, skySize*2, 180, 180);	    
            g.drawArc(bitmapX, bitmapY - skySize, skySize*2, skySize*2, 270, 180);
        }
        else {
            g.drawOval((int)rectangleOutline.getX(), (int)rectangleOutline.getY(), (int)rectangleOutline.getWidth(), (int)rectangleOutline.getHeight());
        }
        
        FontRenderContext frc = g.getFontRenderContext();
        Font font = new Font("Helvetica", Font.PLAIN, 12);
        TextLayout tl = new TextLayout(pointName, font, frc);
        g.setColor(selectedColor);
        tl.draw(g, textCoords[0], textCoords[1]);
        
        if (Math.abs(textCoords[1] - bitmapY) > maxTextDistance) {
            g.setColor(selectedColor);
            g.setStroke(new BasicStroke(1.0f));
            g.drawLine(bitmapX, bitmapY, textCoords[0]-2, textCoords[1]-4);
        }
    }
    
    /**
     * Draw the unselected version of the point.
     */
    public void drawMe(Graphics _g) {
        Graphics2D g = (Graphics2D)_g;
        
        if (!enabled) return;
        int maxTextDistance = 10;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        g.setRenderingHint(RenderingHints.KEY_RENDERING,
                RenderingHints.VALUE_RENDER_QUALITY);
        
        g.setColor(colour);
        if (sizeIsSet) {
            // we have a size statistic, draw a circle with the specified size
            g.fillOval(bitmapX - size/2, bitmapY - size/2, size, size);
            g.setColor(colour.darker());
            g.drawOval(bitmapX - size/2, bitmapY - size/2, size, size);
        }
        else {
            // no size statistic is given, draw a default square
            g.fillRect(bitmapX - 4, bitmapY - 4, 8, 8);
            g.setColor(colour.darker());
            g.drawRect(bitmapX - 4, bitmapY - 4, 8, 8);
        }
        if (pointType.compareTo("sky") == 0) {
            g.setColor(colour.darker());
            int skySize = size * 2/3;
            g.drawArc(bitmapX - skySize, bitmapY - skySize*2, skySize*2, skySize*2, 0, 180);
            g.drawArc(bitmapX - skySize*2, bitmapY - skySize, skySize*2, skySize*2, 90, 180);
            g.drawArc(bitmapX - skySize, bitmapY, skySize*2, skySize*2, 180, 180);	    
            g.drawArc(bitmapX, bitmapY - skySize, skySize*2, skySize*2, 270, 180);
        }
        //NemoUtils.debug("Drawing "+pointName+" at bitmap ("+bitmapX+", "+bitmapY+") and UTM ("+coordX+", "+coordY+")"); 
        
        FontRenderContext frc = g.getFontRenderContext();
        Font font = new Font("Helvetica", Font.PLAIN, 12);
        TextLayout tl = new TextLayout(pointName, font, frc);
        g.setColor(Color.black);
        textCoords = parentMap.getTextCoords(bitmapX, bitmapY, (int)tl.getVisibleAdvance()+2);
        tl.draw(g, textCoords[0], textCoords[1]);
        
        if ((Math.abs(textCoords[1] - bitmapY) > maxTextDistance) || (Math.abs(textCoords[0] - bitmapX) > maxTextDistance*2)) {
            g.setColor(Color.black);
            g.setStroke(new BasicStroke(1.0f));
            g.drawLine(bitmapX, bitmapY, textCoords[0]-2, textCoords[1]-4);
        }
    }
    
    /**
     * Adds a column of statistics to the dialog showing stats for this point.
     */
    public void addStatisticColumn(Hashtable hash, String title) {
        if (dialog == null || hash.isEmpty())
            return;
        dialog.addStatisticColumn(hash, title);
    }
    
    public void addURL(String name,String value){
    	URLnames.add(name);
    	URLvalues.add(value);
    }
    /**
     * Adds an url button the the dialog showing stats for this point.
     */
    public void addURLlist() {
        if (dialog == null)
            return;
        
        String linkContent="";
        for (int i=0;i<URLnames.size();i++){
        	linkContent+="<p><a href="+URLvalues.get(i)+">"+URLnames.get(i)+"</a>\n";
        }
        dialog.addURLlist(linkContent);
    }
    
    /**
     * Opens a statistic dialog showing stats and urls when the mouse is clicked on the line.
     */
    public void showStatistics(MapFrame mapFrame, MouseEvent e) {
        dialog = new StatisticDialog(mapFrame, false, "Statistics for "+pointName, measureStrings, dateStrings);
        Point p = mapFrame.getLocationOnScreen();
        dialog.setLocation((int)p.getX()+50, (int)p.getY()+50);
        dialog.show();
        mapFrame.retrieveEquipmentStatistics(this);
        addURLlist();
        
    }
    public void performMouseClickAction(MapFrame mapFrame,MouseEvent e){
    	URLnames=new ArrayList();
        URLvalues=new ArrayList();
    	mapFrame.retrievePointURL(this);
    	popup = new PopupMenu(this,mapFrame);
    	popup.addURLkeys(URLnames);
    	popup.addURLvalues(URLvalues);
    	
    	popup.show(e.getComponent(),e.getX(), e.getY());
    }
    
    public String toString() {
        return pointName;
    }
    
}
