/*
 * StatisticElement.java
 *
 * Created on July 4, 2003, 11:00 AM
 */

import java.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * The StatisticElement class holds information such as the names of the
 * SQL tables that a spesific statistic obtains information from. It is also
 * used for representing graphical information about the statistic to the
 * user.
 *
 * @author Gunnar Gauslaa Bergem
 */
public class StatisticElement implements Comparable {
    private String statisticName;
    private String statisticType;  // appearance of statistic, like "pointsize", "linecolour", etc.
    private String statisticDescription;
    private String statisticUnit;
    private Vector pointSQLTables;
    private Vector lineSQLTables;
    private Vector intervalList;

    /** Creates a new instance of StatisticElement */
    public StatisticElement(String name, String type, String description, String unit) {
        statisticName = name;
        statisticType = type;
        statisticDescription = description;
        statisticUnit = unit;
        pointSQLTables = new Vector();
        lineSQLTables = new Vector();
        intervalList = new Vector();
    }

    public void addLineSQLTable(String line_table, String column1, String column2, int stepSize) {
        LineSQLTable table = new LineSQLTable(line_table, column1, column2, stepSize);
        lineSQLTables.addElement(table);
    }

    public void addPointSQLTable(String point_table, String point_column, String eq_table, String eq_column, int stepSize) {
        PointSQLTable table = new PointSQLTable(point_table, point_column, eq_table, eq_column, stepSize);
        pointSQLTables.addElement(table);
    }

    public void addIntervalDescription(int start, int end, Color colour, int size, ImageIcon symbol, String label) {
        // check if the interval is already in the list, this prevents duplicates
        for (int i = 0; i < intervalList.size(); i++) {
            IntervalDescription interval = (IntervalDescription)intervalList.elementAt(i);
            if (interval.intervalStart == start && interval.intervalEnd == end)
                return;
        }
        IntervalDescription interval = new IntervalDescription(start, end, colour, size, symbol, label);
        intervalList.addElement(interval);
    }

    public boolean isPointStatistic() {
        return statisticType.startsWith("point");
    }

    public boolean isLineStatistic() {
        return statisticType.startsWith("line");
    }

    public String getType() {
        return statisticType;
    }

    public String getDescription() {
        return statisticDescription;
    }

    public LineSQLTable getLineSQLTable(int stepSize) {
        for (int i = 0; i < lineSQLTables.size(); i++) {
            LineSQLTable table = (LineSQLTable)lineSQLTables.elementAt(i);
            if (table.stepSize == stepSize) {
                return table;
            }
        }
        return null;
    }

    public PointSQLTable getPointSQLTable(int stepSize) {
        for (int i = 0; i < pointSQLTables.size(); i++) {
            PointSQLTable table = (PointSQLTable)pointSQLTables.elementAt(i);
            if (table.stepSize == stepSize) {
                return table;
            }
        }
        return null;
    }

    public Vector getIntervalDescriptions() {
        return intervalList;
    }

    public Vector getStepSizes() {
        Vector stepSizes = new Vector();
        String stepString = new String();

        for (int i = 0; i < pointSQLTables.size(); i++) {
            PointSQLTable table = (PointSQLTable)pointSQLTables.elementAt(i);
            if (table.stepSize == NemoUtils.FIVEMIN_STEP) {
                stepString = "5 minutes";
            }
            else if (table.stepSize == NemoUtils.HOUR_STEP) {
                stepString = "hour";
            }
            else if (table.stepSize == NemoUtils.DAY_STEP) {
                stepString = "day";
            }
            if (!stepSizes.contains(stepString)) {
                stepSizes.addElement(stepString);
            }
        }
        for (int i = 0; i < lineSQLTables.size(); i++) {
          LineSQLTable table = (LineSQLTable)lineSQLTables.elementAt(i);
            if (table.stepSize == NemoUtils.FIVEMIN_STEP) {
                stepString = "5 minutes";
            }
            else if (table.stepSize == NemoUtils.HOUR_STEP) {
                stepString = "hour";
            }
            else if (table.stepSize == NemoUtils.DAY_STEP) {
                stepString = "day";
            }
            if (!stepSizes.contains(stepString)) {
                stepSizes.addElement(stepString);
            }
        }
        return stepSizes;
    }

    public String toString() {
        return statisticName;
    }
    
    public int compareTo(Object o) {
        return toString().compareTo(o.toString());
    }
    
    public boolean equals(Object o) {
        if (this.compareTo(o) == 0) {
            return true;
        }
        return false;
    }

    class LineSQLTable {
        public String table;
        public String column1;
        public String column2;
        public int stepSize;

        public LineSQLTable(String table, String column1, String column2, int stepSize) {
            this.table = table;
            this.column1 = column1;
            this.column2 = column2;
            this.stepSize = stepSize;
        }
    }

    class PointSQLTable {
        public String point_table;
        public String point_column;
        public String eq_table;
        public String eq_column;
        public int stepSize;

        public PointSQLTable(String point_table, String point_column, String eq_table, String eq_column, int stepSize) {
            this.point_table = point_table;
            this.point_column = point_column;
            this.eq_table = eq_table;
            this.eq_column = eq_column;
            this.stepSize = stepSize;
        }
    }

    class IntervalDescription extends JPanel {
        public int intervalStart;
        public int intervalEnd;
        public Color colour;
        public int size;
        public String label;
        private ImageIcon symbol;
        private LineLabel line_label;
        private PointLabel point_label;
        private JLabel number_label;

        public IntervalDescription(int start, int end, Color statColour, int size, ImageIcon symbol, String label) {
            intervalStart = start;
            intervalEnd = end;
            this.colour = statColour;
            this.size = size;
            this.symbol = symbol;
            this.label = label;

            if (colour == null) {
                colour = Color.black;
            }


            number_label = new JLabel();
            this.setLayout(new BorderLayout());

            if (isLineStatistic()) {
                this.setMaximumSize(new Dimension(200, 15));
                if (size == 0) {
                    size = 4;
                }
                line_label = new LineLabel(size, colour);
                this.add(line_label, BorderLayout.CENTER);
            }

            if (isPointStatistic()) {
                this.setMaximumSize(new Dimension(200, 20));
                if (size == 0) {
                    size = 8;
                }
                point_label = new PointLabel(size, colour);
                this.add(point_label, BorderLayout.CENTER);
            }

            if (label == null || label.compareTo("") == 0) {
            	number_label.setText(" "+start+"-"+end+" "+statisticUnit);
			}
			else {
				number_label.setText(label);
			}
            this.add(number_label, BorderLayout.EAST);
        }

        class LineLabel extends JLabel {
            private int size;
            private Color colour;

            public LineLabel(int size, Color colour) {
                this.size = size;
                this.colour = colour;
            }

            public void paintComponent(Graphics g) {
                super.paintComponent(g);
                g.setColor(colour);
                g.fillRect(2, 2, 45, size);
            }
        }

        class PointLabel extends JLabel {
            private int size;
            private Color colour;

            public PointLabel(int size, Color colour) {
                this.size = size;
                this.colour = colour;
            }

            public void paintComponent(Graphics _g) {
                super.paintComponent(_g);
                Graphics2D g = (Graphics2D)_g;

                g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                           RenderingHints.VALUE_ANTIALIAS_ON);
                g.setRenderingHint(RenderingHints.KEY_RENDERING,
                           RenderingHints.VALUE_RENDER_QUALITY);
                g.setColor(colour);
                g.fillOval(10-size/2, 2, size, size);
            }
        }
    }
}
